/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Check it for the first GROUP !!! */

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gtk/gtk.h>
#include <stdio.h>		/* sprintf */
#include <gdk/gdkkeysyms.h>
#include "krunenv.h"
#include "certdesign.h"
#include "printing.h"
#include "menubar.h"
#include "preferences.h"
#include "database_modules.h"
#include "gui.h"

#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600
#define MAX_CLIST_COLUMNS 9
#define CLIST_SHEET_COLUMNS 6

#define CERT_COL_SORT_BY_PRINTED 0
#define CERT_COL_SORT_BY_NR 1
#define CERT_COL_SORT_BY_LASTNAME 2
#define CERT_COL_SORT_BY_PRENAME 3
#define CERT_COL_SORT_BY_TEAM 4
#define CERT_COL_SORT_BY_BIRTHDAY 5
#define CERT_COL_SORT_BY_CLASS 6
#define CERT_COL_SORT_BY_TIME 7
#define CERT_COL_SORT_BY_PLACE 8

/* The dialogs .. */
static GtkWidget *print_certificates_dialog;
static GtkWidget *create_certdesign;

static guint    clist_row;
static guint    clist_column;

extern PrintObject *print_object;
static GtkWidget *sort_arrow[MAX_CLIST_COLUMNS];
static char     local_clist_sort_type;
static void     create_cert_name_dialog (void);
static void     dirlist_button_clicked (GtkCList * clist, gint column);
static void     sort_button_clicked (gint column, GtkCList * clist);
static void     fill_list (GtkWidget * widget, gpointer data);

static GtkWidget *cert_name_dialog;

static void
quit_print_certificates_dialog (void)
{
  gtk_widget_destroy (print_certificates_dialog);
  print_certificates_dialog = NULL;
}


/* Print Certifications */
static void
menuitem_response (gpointer event_number)
{
  fill_list (NULL, NULL);
}

static void
select_runners (gint number)
{
  gint            a, rows;
  gchar          *place;
  gtk_clist_unselect_all (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")));
  rows = GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist"))->rows;
  for (a = 0; a <= rows; a++) {
    gtk_clist_get_text (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")), a, 8, &place);

    switch (number) {
    case SELECT_FIRST:
      if (atoi (place) == 1) {
	gtk_clist_select_row (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")), a, 0);
      }
      break;
    case SELECT_THREE:
      if (atoi (place) == 1 || atoi (place) == 2 || atoi (place) == 3) {
	gtk_clist_select_row (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")), a, 0);
      }
      break;
    case SELECT_WITHOUT_ONE:
      if (atoi (place) != 1) {
	gtk_clist_select_row (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")), a, 0);
      }
      break;
    case SELECT_WITHOUT_THREE:
      if (atoi (place) != 1 && atoi (place) != 2 && atoi (place) != 3) {
	gtk_clist_select_row (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")), a, 0);
      }
      break;
    default:
    }
  }
}

static void
menuitem_response_x (gpointer number)
{
  switch (GPOINTER_TO_INT (number)) {
  case SELECT_ALL:
    gtk_clist_select_all (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")));
    break;
  case SELECT_FIRST:
    select_runners (SELECT_FIRST);
    break;
  case SELECT_THREE:
    select_runners (SELECT_THREE);
    break;
  case SELECT_WITHOUT_THREE:
    select_runners (SELECT_WITHOUT_THREE);
    break;
  case SELECT_WITHOUT_ONE:
    select_runners (SELECT_WITHOUT_ONE);
    break;
  case UNSELECT_ALL:
    gtk_clist_unselect_all (GTK_CLIST (get_widget (print_certificates_dialog, "runner_clist")));
    break;
  default:
  }
}
static          gint
menu_popup (GtkWidget * widget, GdkEvent * event)
{
  if (event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bevent = (GdkEventButton *) event;
    if (bevent->button == 3) {
      gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL, bevent->button, bevent->time);
      return TRUE;
    }
  }
  return FALSE;
}

void
reset_aktiv_state (GtkWidget * widget, gpointer data)
{
  res = execSQL ("update runner set aktiv = 0 where aktiv <> 0 and runner.lauf_nr = %d ", KRunning_event_number.event_number);
  /* gtrans_ifase_result_free (res); */
  fill_list (NULL, NULL);
}

void
create_print_certificates_dialog ()
{
  GtkWidget      *dialog_vbox1;
  GtkWidget      *table1;
  GtkWidget      *scrolledwindow1;
  GtkWidget      *vbox1;
  GtkWidget      *vbox2;

  GtkWidget      *event_menue;

  GtkWidget      *label;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox1;
  GtkWidget      *print_button;
  GtkWidget      *cancel_button;
  GtkWidget      *vbuttonbox;
  GtkWidget      *reset_button;

  /* Pop-up Menu */
  GtkWidget      *menu;
  GtkWidget      *menu_items;

  gint            a;
  gchar          *clist_titles[] = {
    N_("Printed"),
    N_("Nr."),
    N_("Last name"),
    N_("First name"),
    N_("Team"),
    N_("Year of Birth"),
    N_("Class"),
    N_("Time"),
    N_("Place")
  };

  if (print_certificates_dialog != NULL) {
    gdk_window_raise (print_certificates_dialog->window);
    return;
  }
  print_object = (PrintObject *) g_malloc (sizeof (PrintObject));

  clist_col_sort = CERT_COL_SORT_BY_TIME;

  print_certificates_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "print_certificates_dialog", print_certificates_dialog);
  GTK_WINDOW (print_certificates_dialog)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (print_certificates_dialog), _("Print Certificates"));
  gtk_window_set_policy (GTK_WINDOW (print_certificates_dialog), TRUE, TRUE, FALSE);
  gtk_widget_set_usize (print_certificates_dialog, WINDOW_WIDTH, WINDOW_HEIGHT);
  gtk_signal_connect (GTK_OBJECT (print_certificates_dialog), "destroy", GTK_SIGNAL_FUNC (quit_print_certificates_dialog), NULL);
  print_object->parent_window = print_certificates_dialog;	/* To set it modal while saving .. */

  dialog_vbox1 = GTK_DIALOG (print_certificates_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "table1", table1);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), table1, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (table1), BORDER_WIDTH);

  /* The Pop-Up Menu */
  menu = gtk_menu_new ();
  gtk_menu_set_title (GTK_MENU (menu), _("Select"));
  menu_separator (menu);
  menu_items = gtk_menu_item_new_with_label (_("All"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response_x), (gpointer) SELECT_ALL);
  gtk_widget_show (menu_items);

  menu_items = gtk_menu_item_new_with_label (_("Nothing"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response_x), (gpointer) UNSELECT_ALL);
  gtk_widget_show (menu_items);


  menu_separator (menu);
  menu_items = gtk_menu_item_new_with_label (_("First from each Class"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response_x), (gpointer) SELECT_FIRST);
  gtk_widget_show (menu_items);
  menu_items = gtk_menu_item_new_with_label (_("First 3 of each Class"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response_x), (gpointer) SELECT_THREE);
  gtk_widget_show (menu_items);
  menu_separator (menu);
  menu_items = gtk_menu_item_new_with_label (_("All without the first"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response_x), (gpointer) SELECT_WITHOUT_ONE);
  gtk_widget_show (menu_items);
  menu_items = gtk_menu_item_new_with_label (_("All without the first three"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response_x), (gpointer) SELECT_WITHOUT_THREE);
  gtk_widget_show (menu_items);
  menu_separator (menu);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "scrolledwindow1", scrolledwindow1);
  gtk_widget_show (scrolledwindow1);
  gtk_table_attach (GTK_TABLE (table1), scrolledwindow1, 1, 2, 1, 2, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

  print_object->main_clist = GTK_CLIST (gtk_clist_new_with_titles (MAX_CLIST_COLUMNS, clist_titles));
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "runner_clist", print_object->main_clist);
  gtk_widget_show (GTK_WIDGET (print_object->main_clist));
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), GTK_WIDGET (print_object->main_clist));
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 0, 40);
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 1, 40);
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 2, 80);
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 3, 80);
  gtk_clist_set_column_auto_resize (GTK_CLIST (print_object->main_clist), 4, TRUE);
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 5, 80);
  gtk_clist_set_column_auto_resize (GTK_CLIST (print_object->main_clist), 6, TRUE);
  gtk_clist_set_column_auto_resize (GTK_CLIST (print_object->main_clist), 7, TRUE);
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 8, 80);

  gtk_clist_set_selection_mode (GTK_CLIST (print_object->main_clist),	/*GTK_SELECTION_EXTENDED */
				GTK_SELECTION_MULTIPLE);
  gtk_clist_set_reorderable (GTK_CLIST (print_object->main_clist), TRUE);
  gtk_clist_column_titles_show (GTK_CLIST (print_object->main_clist));

  gtk_signal_connect_object (GTK_OBJECT (print_object->main_clist), "event", GTK_SIGNAL_FUNC (menu_popup), GTK_OBJECT (menu));
  gtk_widget_realize (GTK_WIDGET (print_object->main_clist));
  gdk_window_set_cursor (GTK_WIDGET (print_object->main_clist)->window, gdk_cursor_new (GDK_HAND1));

  for (a = 1; a < MAX_CLIST_COLUMNS; a++)
    sort_arrow[a] = clist_title_with_arrow (GTK_WIDGET (print_object->main_clist), a, clist_titles[a]);

  gtk_widget_show (sort_arrow[CERT_COL_SORT_BY_TIME]);
  gtk_clist_set_sort_column (GTK_CLIST (print_object->main_clist), CERT_COL_SORT_BY_TIME);
  gtk_clist_set_sort_type (GTK_CLIST (print_object->main_clist), SORT_DOWN);

  gtk_signal_connect (GTK_OBJECT (print_object->main_clist), "click_column", GTK_SIGNAL_FUNC (dirlist_button_clicked), NULL);


  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_table_attach (GTK_TABLE (table1), vbox1, 0, 1, 1, 2, (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);

  event_menue = create_event_menu ((void (*)(GtkWidget *, gpointer)) menuitem_response, FALSE);
  gtk_table_attach (GTK_TABLE (table1), event_menue, 0, 1, 0, 1, (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_widget_show (event_menue);

  label = gtk_label_new (_("Select Runners to print"));
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "label", label);
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 0, 1, (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);

  /* Some selection Menus coming here */

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "vbox2", vbox2);
  gtk_widget_show (vbox2);
  gtk_table_attach (GTK_TABLE (table1), vbox2, 0, 1, 1, 2, (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);

  label = gtk_label_new (_("Select Runners to shoot"));
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "label", label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox2), label, TRUE, TRUE, 0);

  vbuttonbox = gtk_vbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "vbuttonbox", vbuttonbox);
  gtk_widget_show (vbuttonbox);
  gtk_box_pack_start (GTK_BOX (vbox2), vbuttonbox, TRUE, TRUE, 0);

  reset_button = gtk_button_new_with_label (_("Reset Printed state"));
  gtk_signal_connect (GTK_OBJECT (reset_button), "clicked", GTK_SIGNAL_FUNC (reset_aktiv_state), NULL);
  gtk_widget_show (reset_button);
  gtk_container_add (GTK_CONTAINER (vbuttonbox), reset_button);

  dialog_action_area1 = GTK_DIALOG (print_certificates_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "hbuttonbox1", hbuttonbox1);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);

  print_object->event_number = -1;
  print_object->listtype = CERTIFICATE;

/*  print_object->main_clist = GTK_CLIST (print_object->main_clist); */


  print_button = gtk_button_new_with_label (_("Print"));
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "print_button", print_button);

  gtk_signal_connect (GTK_OBJECT (print_button), "clicked", GTK_SIGNAL_FUNC (print_clist), print_object);


  gtk_widget_show (print_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), print_button);

  /* We have to set print_certificates_dialog=NULL in here */
  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (print_certificates_dialog), "cancel_button", cancel_button);
  gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (quit_print_certificates_dialog), NULL);

  gtk_widget_show (cancel_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel_button);
  gtk_widget_show (print_certificates_dialog);
  fill_list (NULL, NULL);
}

static void
fill_list (GtkWidget * widget, gpointer data)
{
  gint            a, row;
  gint            time[3];
  guint32         seconds = 0;
  gchar           text[MAX_CLIST_COLUMNS][50];
  gchar          *texts[MAX_CLIST_COLUMNS];

  print_object->event_number = KRunning_event_number.event_number;
  gtk_clist_clear (GTK_CLIST (print_object->main_clist));
  for (a = 0; a <= MAX_CLIST_COLUMNS; a++) {
    texts[a] = text[a];
  }
  /* We should handle the ordering more flexible here */
  res = execSQL ("select runner.number,name,vorname,vereine.v_name,geb_jahr, "
		 "runner.sex,times.result_seconds,runner.place_class,runner.aktiv " "from runner, vereine,times " "where runner.verein = vereine.number " "and runner.number = times.runner_number " "and runner.lauf_nr = %d ", KRunning_event_number.event_number);
  gtk_clist_freeze (GTK_CLIST (print_object->main_clist));
  for (a = 0; a < PQntuples (res); a++) {
    if (atoi (PQgetvalue (res, a, 0)) > 0) {
      sprintf (text[1], "%s", PQgetvalue (res, a, 0));	/* Number */
      sprintf (text[2], "%s", PQgetvalue (res, a, 1));	/* Name */
      sprintf (text[3], "%s", PQgetvalue (res, a, 2));	/* Pre-Name */
      sprintf (text[4], "%s", PQgetvalue (res, a, 3));	/* Team - Name */
      sprintf (text[5], "%s", PQgetvalue (res, a, 4));	/* Year of Birth */
      sprintf (text[6], "%s", show_klasse (atoi (PQgetvalue (res, a, 4)), atoi (PQgetvalue (res, a, 5))));
      seconds = atoi (PQgetvalue (res, a, 6));	/* Time */
      time[0] = seconds / 3600L;
      seconds -= time[0] * 3600L;
      time[1] = seconds / 60L;
      time[2] = seconds - time[1] * 60L;
      sprintf (text[7], "%02d:%02d:%02d", time[0], time[1], time[2]);
      sprintf (text[8], "%s", PQgetvalue (res, a, 7));
      row = gtk_clist_append (print_object->main_clist, texts);

      if (atoi (PQgetvalue (res, a, 8)) != 0) {
	gtk_clist_set_pixtext (print_object->main_clist, row, 0, PQgetvalue (res, a, 8), 10, KRunningxpm.checkedbox, KRunningxpm.checkedbox_mask);
      } else {
	gtk_clist_set_pixtext (print_object->main_clist, row, 0, "0", 10, KRunningxpm.crossbox, KRunningxpm.crossbox_mask);
      }
    }
  }
  PQclear (res);
  gtk_clist_sort (GTK_CLIST (print_object->main_clist));
  gtk_clist_thaw (GTK_CLIST (print_object->main_clist));
}

static void
dirlist_button_clicked (GtkCList * clist, gint column)
{
  switch (column) {
  case 0:
    gdk_beep ();
    break;
  default:
    sort_button_clicked (column, clist);
  }
}
static void
sort_button_clicked (gint column, GtkCList * clist)
{
  if (clist_col_sort == column)
    local_clist_sort_type = !local_clist_sort_type;
  else {
    gtk_widget_hide (sort_arrow[(int) clist_col_sort]);
    gtk_widget_show (sort_arrow[column]);
    local_clist_sort_type = SORT_DOWN;
  }
  clist_col_sort = column;
  gtk_clist_set_sort_type (GTK_CLIST (print_object->main_clist), local_clist_sort_type);
  clist_title_with_arrow_set_up_dow (sort_arrow[column], (local_clist_sort_type == SORT_UP));

  gtk_clist_freeze (GTK_CLIST (print_object->main_clist));
  gtk_clist_set_sort_column (GTK_CLIST (print_object->main_clist), column);
  gtk_clist_sort (GTK_CLIST (print_object->main_clist));
  gtk_clist_thaw (GTK_CLIST (print_object->main_clist));

}





gint
hscrollbar_update (GtkAdjustment * adjustment, GtkWidget * scrollbar)
{
  return FALSE;
}


gint
vscrollbar_update (GtkAdjustment * adjustment, GtkWidget * scrollbar)
{
  return FALSE;
}

static void
read_certdesigns (GtkWidget * clist, gchar * certdesign_name)
{
  gint            clist_rows = 0;
/*  gchar text[CLIST_SHEET_COLUMNS][50];*/
  gchar          *texts[CLIST_SHEET_COLUMNS];
  if (cfg.certificates)
    gtk_clist_clear (GTK_CLIST (clist));
  gtk_clist_insert (GTK_CLIST (clist), clist_rows, texts);

}
static void
delete_certdesign (GtkWidget * widget, gpointer data)
{
}

static void
save_certdesign (GtkWidget * widget, gpointer data)
{
/* Save the design */
  gchar          *clist_data = NULL;
  gint            row;
  for (row = 0; row < GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist"))->rows; row++) {
    gtk_clist_get_text (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")), row, 1, &clist_data);
    cert.entry[row]->value_y = atoi (clist_data);
    gtk_clist_get_text (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")), row, 2, &clist_data);
    cert.entry[row]->align = atoi (clist_data);
    /* changable entrys */
    if (row > 3 && row < 9) {
      if (cert.entry[row]->string) {
	g_free (cert.entry[row]->string);
	cert.entry[row]->string = NULL;
      }
      gtk_clist_get_text (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")), row, 3, &clist_data);
      cert.entry[row]->string = g_strdup (clist_data);
    }
    gtk_clist_get_text (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")), row, 4, &clist_data);
    cert.entry[row]->font_size = atoi (clist_data);
    gtk_clist_get_text (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")), row, 5, &clist_data);


    if (g_strcasecmp (clist_data, _("Yes")) == 0)
      cert.entry[row]->print = TRUE;
    else
      cert.entry[row]->print = FALSE;
  }
  cert.paper_size = get_paper_number (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (get_widget (create_certdesign, "combo_box"))->entry)));
  cert.size = n_cert_values;
  save_certification (NULL);
}

static void
set_paper_size (GtkWidget * widget, gpointer data)
{
  gint            i;
  gchar          *size_string;

  i = get_paper_number (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (get_widget (create_certdesign, "combo_box"))->entry)));
  size_string = g_strdup_printf ("%s x %s", get_paper_x_size (i), get_paper_y_size (i));

  gtk_label_set (GTK_LABEL (get_widget (create_certdesign, "paper_size_label")), size_string);
}

static void
insert_raw_clist_values ()
{
  gint            i;
  gchar           text[CERTIFICATION_ENTRYS][50];
  gchar          *texts[CERTIFICATION_ENTRYS];
  for (i = 0; i <= CERTIFICATION_ENTRYS; i++) {
    texts[i] = text[i];
  }
  gtk_clist_clear (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")));
  for (i = 0; i < n_cert_values; i++) {
    sprintf (text[0], "%s", cert.entry[i]->value_name);
    sprintf (text[1], "%d", cert.entry[i]->value_y);
    sprintf (text[2], "%d", cert.entry[i]->align);
    switch (i) {
    case 0:
      sprintf (text[3], "%s", cfg.event_number);
      break;
    case 1:
      sprintf (text[3], "%s", cfg.main_event_name);
      break;
    case 2:
      sprintf (text[3], "%s", cfg.sub_event_name);
      break;
    case 3:
      sprintf (text[3], "%s", cfg.organisation_name);
      break;
    case 9:
      sprintf (text[3], "%s", cfg.event_date);
      break;
    default:
      sprintf (text[3], "%s", cert.entry[i]->string);
    }
    sprintf (text[4], "%d", cert.entry[i]->font_size);
    if (cert.entry[i]->print == TRUE)
      sprintf (text[5], "%s", _("Yes"));
    else
      sprintf (text[5], "%s", _("No"));
    gtk_clist_insert (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")), i, texts);
  }
}


static void
change_cert_entry (char *name)
{
  load_certification (NULL, name);

  /* Set the actual certdesign here */
  if (cfg.actual_certificate)
    g_free (cfg.actual_certificate);
  cfg.actual_certificate = g_strdup (name);
  /* Save ?? */
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (get_widget (create_certdesign, "combo_box"))->entry), get_paper_format (cert.paper_size));
  insert_raw_clist_values ();
  gtk_entry_set_text (GTK_ENTRY (get_widget (create_certdesign, "format_string_entry")), "");
  gtk_label_set_text (GTK_LABEL (get_widget (create_certdesign, "entry_desc_label")), _("  Edit your settings here  "));

  /*  gtk_entry_set_text(GTK_ENTRY(get_widget(create_certdesign,"height_spinbutton")),get_paper_y_size(cert.paper_size));
     gtk_entry_set_text(GTK_ENTRY(get_widget(create_certdesign,"width_spinbutton")),get_paper_x_size(cert.paper_size)); */
}

void
change_number (gint row, gint column, gint number)
{

}

void
eval_format_string (GtkWidget * widget, gpointer data)
{
  gchar          *input_string;
  guint           tmp;
  gint            row, column;
  input_string = gtk_entry_get_text (GTK_ENTRY (get_widget (create_certdesign, "format_string_entry")));
  gtk_clist_set_text (GTK_CLIST (get_widget (create_certdesign, "data_sheet_clist")), clist_row, clist_column, input_string);


/* otherwise we'll lose terminator */
/*
   while ((c = *input_string++) != '\0')
   {
   tmp = 1;
   if (c == '%')
   {
   switch (*input_string++)
   {
   case 0:
   input_string--;      

   case '%':
   ans[length] = '%';
   break;
   case CLASS_STRING:
   tmp = 3;                                     
   strncpy(&ans[length], id3->artist, tmp);
   break;
   case ID3_TITLE:
   tmp = strlen(id3->title);
   if(tmp != 0)
   got_field = TRUE;
   strncpy(&ans[length], id3->title, tmp);
   break;
   case ID3_ALBUM:
   tmp = strlen(id3->album);
   if(tmp != 0)
   got_field = TRUE;
   strncpy(&ans[length], id3->album, tmp);
   break;
   case ID3_YEAR:
   tmp = strlen(id3->year);
   if(tmp != 0)
   got_field = TRUE;
   strncpy(&ans[length], id3->year, tmp);
   break;
   case ID3_COMMENT:
   tmp = strlen(id3->comment);                                  
   if(tmp != 0)
   got_field = TRUE;
   strncpy(&ans[length], id3->comment, tmp);
   break;
   case ID3_GENRE:                                      
   tmp = strlen(id3->genre);
   if(tmp != 0)
   got_field = TRUE;
   strncpy(&ans[length], id3->genre, tmp);
   break;
   case FILE_NAME:
   strncpy(&ans[length], base, tmp = baselen);
   got_field = TRUE;
   break;
   case FILE_PATH:
   strncpy(&ans[length], path, tmp = pathlen);
   got_field = TRUE;
   break;
   case FILE_EXT:
   strncpy(&ans[length], ext, tmp = extlen);
   got_field = TRUE;
   break;
   default:
   ans[length] = c;
   break;
   }
   } */
}

void
change_entrys (GtkWidget * clist, gint row, gint column, GdkEventButton * event, gpointer data)
{
  gchar          *text;
  gtk_clist_get_text (GTK_CLIST (clist), row, column, &text);
  clist_column = column;
  clist_row = row;

  if (event->type == GDK_2BUTTON_PRESS) {
    gtk_widget_set_sensitive (get_widget (create_certdesign, "format_string_entry"), TRUE);
    switch (column) {
    case 0:
      gtk_widget_set_sensitive (get_widget (create_certdesign, "format_string_entry"), FALSE);
      break;
    case 3:
      gtk_entry_set_text (GTK_ENTRY (get_widget (create_certdesign, "format_string_entry")), text);
      gtk_widget_grab_focus (get_widget (create_certdesign, "format_string_entry"));
      eval_format_string (NULL, NULL);
      break;
    case 5:
      if (g_strcasecmp (text, _("No")) != 0)
	gtk_clist_set_text (GTK_CLIST (clist), row, column, _("No"));
      else
	gtk_clist_set_text (GTK_CLIST (clist), row, column, _("Yes"));
      break;
    default:
      gtk_entry_set_text (GTK_ENTRY (get_widget (create_certdesign, "format_string_entry")), text);
      gtk_widget_grab_focus (get_widget (create_certdesign, "format_string_entry"));
      gtk_label_set_text (GTK_LABEL (get_widget (create_certdesign, "entry_desc_label")), "(RETURN)");
    }
  } else {
    gtk_widget_set_sensitive (get_widget (create_certdesign, "format_string_entry"), FALSE);
  }
  return;
}

void
create_certification_menu (void)
{
  GtkWidget      *cert_option_entry;
  GtkWidget      *cert_option_menu;
  gchar         **certification_names = NULL;
  gint            i = 0;

  /* Memory free ??? */
  gtk_option_menu_remove_menu (GTK_OPTION_MENU (get_widget (create_certdesign, "cert_option")));
  cert_option_menu = gtk_menu_new ();

  if (cfg.certificates)
    certification_names = g_strsplit (cfg.certificates, ";", 0);
  while (certification_names[i] != NULL) {
    /* Here we must get the entrys from the config file */
    cert_option_entry = gtk_menu_item_new_with_label (certification_names[i]);
    gtk_signal_connect_object (GTK_OBJECT (cert_option_entry), "activate", GTK_SIGNAL_FUNC (change_cert_entry), (gpointer)
			       g_strdup (certification_names[i]));

    gtk_widget_show (cert_option_entry);
    gtk_menu_append (GTK_MENU (cert_option_menu), cert_option_entry);
    i++;
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (get_widget (create_certdesign, "cert_option")), cert_option_menu);


  if (certification_names)
    g_strfreev (certification_names);
}

void
create_design_dialog (void)
{
  /* Work with double click events and dialog boxes on the clist !! */

  GtkWidget      *dialog_vbox1;
  GtkWidget      *hbox1;
  GtkWidget      *size_frame;
  GtkWidget      *vbox2;
  GtkWidget      *scrolledwindow1;
  GtkWidget      *data_sheet_clist;
  GtkWidget      *vbox1;
  GtkWidget      *frame3;
  GtkWidget      *hbox2;
  GtkWidget      *entry_desc_label;
  GtkWidget      *cert_option;

  GtkWidget      *hbuttonbox2;
  GtkWidget      *new;
  GtkWidget      *paper_frame;
  GtkWidget      *paper_size_vbox;
  GtkWidget      *combo_box;
  GList          *combo_box_items = NULL;

  GtkWidget      *combo_entry;
  GtkWidget      *height_alignment;
  GtkWidget      *paper_size_label;
  GtkWidget      *margin_frame;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *format_string_entry;
  /* BUTTONS */
  GtkWidget      *hbuttonbox1;
  GtkWidget      *ok_button;
  GtkWidget      *save_button;
  GtkWidget      *delete_button;
  GtkWidget      *print_button;
  GtkWidget      *cancel_button;

  /* Misc */
  gint            i = 0;

  gchar          *clist_titles[] = {
    N_("Type"),
    N_("Y-Value"),
    N_("Align"),
    N_("String value"),
    N_("Font size"),
    N_("Print")
  };


  create_certdesign = gtk_dialog_new ();

  gtk_widget_set_name (create_certdesign, "create_certdesign");


  gtk_object_set_data (GTK_OBJECT (create_certdesign), "create_certdesign", create_certdesign);
  gtk_window_set_title (GTK_WINDOW (create_certdesign), _("Create Certificate"));
  GTK_WINDOW (create_certdesign)->type = GTK_WINDOW_DIALOG;


  gtk_widget_set_usize (create_certdesign, WINDOW_WIDTH - 140, WINDOW_HEIGHT - 160);

  dialog_vbox1 = GTK_DIALOG (create_certdesign)->vbox;
  gtk_widget_set_name (dialog_vbox1, "dialog_vbox1");


  gtk_object_set_data (GTK_OBJECT (create_certdesign), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox1, "hbox1");
  gtk_widget_ref (hbox1);


  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "hbox1", hbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);

  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);

  /*
   * Certification Name
   * Paper Size
   */

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox1, "vbox1");
  gtk_widget_ref (vbox1);


  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "vbox1", vbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox1, FALSE, TRUE, 0);

  frame3 = gtk_frame_new (_("Certification Name"));
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox1), frame3, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 3);

  hbuttonbox2 = gtk_hbutton_box_new ();
  gtk_widget_set_name (hbuttonbox2, "hbuttonbox2");
  gtk_widget_ref (hbuttonbox2);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "hbuttonbox2", hbuttonbox2, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox2);
  gtk_container_set_border_width (GTK_CONTAINER (hbuttonbox2), 3);
  /*gtk_box_pack_start(GTK_BOX(hbox2), hbuttonbox2, TRUE, TRUE, 0); */
  gtk_container_add (GTK_CONTAINER (frame3), hbuttonbox2);

  cert_option = gtk_option_menu_new ();
  gtk_widget_ref (cert_option);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "cert_option", cert_option, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cert_option);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), cert_option);

  create_certification_menu ();

  new = gtk_button_new_with_label (_("New"));
  gtk_widget_ref (new);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "new", new, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (new);
  gtk_signal_connect (GTK_OBJECT (new), "clicked", GTK_SIGNAL_FUNC (create_cert_name_dialog), NULL);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), new);
  //GTK_WIDGET_SET_FLAGS(new, GTK_CAN_DEFAULT);

  paper_frame = gtk_frame_new (_("Paper Size"));
  gtk_widget_ref (paper_frame);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "paper_frame", paper_frame, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (paper_frame);
  gtk_box_pack_start (GTK_BOX (vbox1), paper_frame, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (paper_frame), 3);

  paper_size_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (paper_size_vbox), 3);
  gtk_container_add (GTK_CONTAINER (paper_frame), paper_size_vbox);
  gtk_widget_show (paper_size_vbox);
  /*
     paper_table = gtk_table_new(3, 3, FALSE);
     gtk_widget_set_name(paper_table, "paper_table");
     gtk_widget_ref(paper_table);
     gtk_object_set_data_full(GTK_OBJECT(create_certdesign), "paper_table", paper_table,
     (GtkDestroyNotify) gtk_widget_unref);
     gtk_widget_show(paper_table);
     gtk_container_add(GTK_CONTAINER(paper_frame), paper_table);
     gtk_container_set_border_width(GTK_CONTAINER(paper_table), 3);
     gtk_table_set_row_spacings(GTK_TABLE(paper_table), 3);
     gtk_table_set_col_spacings(GTK_TABLE(paper_table), 3);
   */
  combo_box = gtk_combo_new ();
  gtk_widget_set_name (combo_box, "combo_box");
  gtk_widget_ref (combo_box);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "combo_box", combo_box, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_box);

  gtk_box_pack_start (GTK_BOX (paper_size_vbox), combo_box, FALSE, FALSE, 0);
  for (i = 1; i <= paper_sizes_n; i++) {
    combo_box_items = g_list_append (combo_box_items, get_paper_format (i));
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_box), combo_box_items);
  g_list_free (combo_box_items);

  combo_entry = GTK_COMBO (combo_box)->entry;
  gtk_widget_set_name (combo_entry, "combo_entry");
  gtk_widget_ref (combo_entry);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "combo_entry", combo_entry, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry);
  gtk_entry_set_editable (GTK_ENTRY (combo_entry), FALSE);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo_box)->entry), "changed", GTK_SIGNAL_FUNC (set_paper_size), NULL);
  // gtk_entry_set_text(GTK_ENTRY(combo_entry), _("None\t"));


  height_alignment = gtk_alignment_new (0, 0.5, 1, 1);
  gtk_widget_set_name (height_alignment, "height_alignment");
  gtk_widget_ref (height_alignment);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "height_alignment", height_alignment, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (height_alignment);
  gtk_box_pack_end (GTK_BOX (paper_size_vbox), height_alignment, FALSE, FALSE, 0);

  paper_size_label = gtk_label_new ("0 x 0");
  gtk_widget_set_name (paper_size_label, "paper_size_label");
  gtk_widget_ref (paper_size_label);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "paper_size_label", paper_size_label, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (paper_size_label);
  gtk_box_pack_end (GTK_BOX (paper_size_vbox), paper_size_label, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (paper_size_label), 0.5, 0);

  /* Margins */
  margin_frame = gtk_frame_new (_("Margins"));
  gtk_widget_set_name (margin_frame, "margin_frame");
  gtk_widget_ref (margin_frame);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "margin_frame", margin_frame, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (margin_frame);
  gtk_box_pack_end (GTK_BOX (vbox1), margin_frame, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (margin_frame), 3);
  /*
   * Default Text Size Clist 
   * Pixmap
   */

  size_frame = gtk_frame_new (_("Default Text Size and Location"));
  gtk_widget_set_name (size_frame, "size_frame");
  gtk_widget_ref (size_frame);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "size_frame", size_frame, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (size_frame);
  gtk_box_pack_end (GTK_BOX (hbox1), size_frame, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (size_frame), 3);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox2, "vbox2");
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "vbox2", vbox2, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (size_frame), vbox2);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrolledwindow1, "scrolledwindow1");
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "scrolledwindow1", scrolledwindow1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow1), 3);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  data_sheet_clist = gtk_clist_new_with_titles (CLIST_SHEET_COLUMNS, clist_titles);
  gtk_widget_set_name (data_sheet_clist, "data_sheet_clist");
  gtk_widget_ref (data_sheet_clist);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "data_sheet_clist", data_sheet_clist, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (data_sheet_clist);
  // gtk_clist_column_titles_passive(GTK_CLIST(data_sheet_clist));
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), data_sheet_clist);
  for (i = 0; i < CLIST_SHEET_COLUMNS; i++)
    gtk_clist_set_column_auto_resize (GTK_CLIST (data_sheet_clist), i, TRUE);


  gtk_clist_column_titles_show (GTK_CLIST (data_sheet_clist));
  gtk_signal_connect (GTK_OBJECT (data_sheet_clist), "select_row", GTK_SIGNAL_FUNC (change_entrys), NULL);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "hbox2", hbox2, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_container_set_border_width (GTK_CONTAINER (hbox2), 2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, TRUE, 0);

  format_string_entry = gtk_entry_new_with_max_length (50);
  gtk_object_set_data (GTK_OBJECT (create_certdesign), "format_string_entry", format_string_entry);
  gtk_widget_show (format_string_entry);
  gtk_signal_connect (GTK_OBJECT (format_string_entry), "activate", GTK_SIGNAL_FUNC (eval_format_string), NULL);
  gtk_box_pack_start (GTK_BOX (hbox2), format_string_entry, TRUE, TRUE, 2);
  gtk_widget_set_sensitive (format_string_entry, FALSE);

  entry_desc_label = gtk_label_new ("      HALLO        ");
  gtk_object_set_data (GTK_OBJECT (create_certdesign), "entry_desc_label", entry_desc_label);
  gtk_widget_show (entry_desc_label);
  gtk_box_pack_end (GTK_BOX (hbox2), entry_desc_label, FALSE, TRUE, 0);



  dialog_action_area1 = GTK_DIALOG (create_certdesign)->action_area;
  gtk_widget_set_name (dialog_action_area1, "dialog_action_area1");
  gtk_object_set_data (GTK_OBJECT (create_certdesign), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_set_name (hbuttonbox1, "hbuttonbox1");
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "hbuttonbox1", hbuttonbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);

  ok_button = gtk_button_new_with_label (_("Ok"));
  gtk_widget_set_name (ok_button, "ok_button");
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "ok_button", ok_button, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ok_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), ok_button);

  save_button = gtk_button_new_with_label (_("Save"));
  gtk_signal_connect (GTK_OBJECT (save_button), "clicked", GTK_SIGNAL_FUNC (save_certdesign), NULL);
  gtk_widget_show (save_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), save_button);

  delete_button = gtk_button_new_with_label (_("Delete"));
  gtk_widget_show (delete_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), delete_button);
  gtk_signal_connect_object (GTK_OBJECT (delete_button), "clicked", GTK_SIGNAL_FUNC (delete_certdesign), NULL);


  print_button = gtk_button_new_with_label (_("Print"));
  gtk_widget_set_name (print_button, "print_button");
  gtk_widget_ref (print_button);
  gtk_object_set_data_full (GTK_OBJECT (create_certdesign), "print_button", print_button, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (print_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), print_button);

  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_set_name (cancel_button, "cancel_button");
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (create_certdesign));
  gtk_widget_show (cancel_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel_button);

  gtk_widget_show (create_certdesign);
}

void
add_new_certification (GtkWidget * widget, gpointer data)
{
  gchar          *certification_names = NULL;
  certification_names = g_strdup_printf ("%s;%s", cfg.certificates, gtk_entry_get_text (GTK_ENTRY (widget)));
  if (cfg.certificates)
    g_free (cfg.certificates);
  cfg.certificates = g_strdup (certification_names);
  create_certification (gtk_entry_get_text (GTK_ENTRY (widget)));
  if (certification_names)
    g_free (certification_names);
  gtk_widget_destroy (cert_name_dialog);
  cert_name_dialog = NULL;
  save_config ();
  create_certification_menu ();
  /* In Menue eintragen und in clist eintragen !! */
}

static void
create_cert_name_dialog (void)
{
  GtkWidget      *dialog_vbox1;
  GtkWidget      *vbox1;
  GtkWidget      *head_label;
  GtkWidget      *cert_name_entry;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox1;
  GtkWidget      *ok_button;
  GtkWidget      *cancel_button;

  cert_name_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (cert_name_dialog), "cert_name_dialog", cert_name_dialog);
  gtk_window_set_title (GTK_WINDOW (cert_name_dialog), _("Enter a name"));
  gtk_window_set_position (GTK_WINDOW (cert_name_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (cert_name_dialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (cert_name_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (cert_name_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (cert_name_dialog), "vbox1", vbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 3);

  head_label = gtk_label_new (_("Enter a new name please"));
  gtk_widget_ref (head_label);
  gtk_object_set_data_full (GTK_OBJECT (cert_name_dialog), "head_label", head_label, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (head_label);
  gtk_box_pack_start (GTK_BOX (vbox1), head_label, FALSE, FALSE, 0);

  cert_name_entry = gtk_entry_new_with_max_length (50);
  gtk_widget_ref (cert_name_entry);
  gtk_object_set_data_full (GTK_OBJECT (cert_name_dialog), "cert_name_entry", cert_name_entry, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cert_name_entry);
  gtk_box_pack_end (GTK_BOX (vbox1), cert_name_entry, FALSE, FALSE, 0);

  dialog_action_area1 = GTK_DIALOG (cert_name_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (cert_name_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (cert_name_dialog), "hbuttonbox1", hbuttonbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);

  ok_button = gtk_button_new_with_label (_("Ok"));
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (cert_name_dialog), "ok_button", ok_button, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ok_button);
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (add_new_certification), GTK_OBJECT (cert_name_entry));
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), ok_button);

  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_ref (cancel_button);
  gtk_object_set_data_full (GTK_OBJECT (cert_name_dialog), "cancel_button", cancel_button, (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (cert_name_dialog));
  gtk_widget_show (cancel_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel_button);
  gtk_widget_grab_focus (cert_name_entry);
  gtk_widget_show (cert_name_dialog);
}
