/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include "krunenv.h"
#include "printing.h"
#include "preferences.h"
#include "classteamprint.h"
#include "database_modules.h"


GtkWidget      *classtresult_dialog;

extern PrintObject *print_object;

GList          *teamcount_find (GList * list, gint number);
GList          *teamcount_find_reverse (GList * list, gint number);
KTeamCount     *create_team_counter (gint number, gchar * teamname, gint teamnumber);
KClassCount    *create_class_counter (gint number, guint32 time, gchar * name, gchar * vorname, gchar * verein, gint geb_jahr);
static void     menuitem_response (GtkWidget * widget, gpointer number);
static void     init_classteamresult (GtkWidget * widget, gpointer selection);
void            clear_team_counter (KTeamCount * teamptr);
void            clear_class_counter (KClassCount * classcountptr);
void            calculate_team_result (gint sex);
void            calculate_class_result (gint sex);
static gint     time_compare_func (gconstpointer a, gconstpointer b);
static gint     class_compare_func (gconstpointer a, gconstpointer b);
/*static gint run_number = -1; */

void
create_classtresult_dialog ()
{
  GtkWidget      *dialog_vbox1;
  GtkWidget      *table2;
  GtkWidget      *scrolledwindow1;
  GtkWidget      *frame1;

  GtkWidget      *event_menue;

  GtkWidget      *vbox;
  GtkWidget      *button;
  GtkWidget      *hseparator;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox1;
  GtkWidget      *print_button;
  GtkWidget      *cancel_button;

  print_object = (PrintObject *) g_malloc0 (sizeof (PrintObject));
  classtresult_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "classtresult_dialog", classtresult_dialog);
  gtk_window_set_title (GTK_WINDOW (classtresult_dialog), _("Class & Teamresults"));
  gtk_window_set_policy (GTK_WINDOW (classtresult_dialog), TRUE, TRUE, FALSE);
  gtk_widget_set_usize (classtresult_dialog, 640, 480);

  print_object->parent_window = classtresult_dialog;	/* To set it modal while saving .. */

  dialog_vbox1 = GTK_DIALOG (classtresult_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);
  gtk_container_border_width (GTK_CONTAINER (dialog_vbox1), BORDER_WIDTH);

  table2 = gtk_table_new (3, 2, FALSE);
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "table2", table2);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), table2, TRUE, TRUE, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "scrolledwindow1", scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show (scrolledwindow1);
  gtk_table_attach (GTK_TABLE (table2), scrolledwindow1, 1, 2, 1, 2, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

  print_object->main_text = GTK_TEXT (gtk_text_new (NULL, NULL));
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "textwidget", print_object->main_text);
  gtk_widget_show (GTK_WIDGET (print_object->main_text));
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), GTK_WIDGET (print_object->main_text));
  gtk_widget_realize (GTK_WIDGET (print_object->main_text));

  event_menue = create_event_menu ((void (*)(GtkWidget *, gpointer)) menuitem_response, FALSE);
  gtk_table_attach (GTK_TABLE (table2), event_menue, 0, 1, 0, 1, (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_widget_show (event_menue);


  frame1 = gtk_frame_new (_("Select something :"));
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "frame1", frame1);
  gtk_widget_show (frame1);
  gtk_table_attach (GTK_TABLE (table2), frame1, 0, 1, 1, 2, (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);

  vbox = gtk_vbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (frame1), vbox);

  button = gtk_radio_button_new_with_label (NULL, _("Classes"));
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "ClassButton", button);
  gtk_signal_connect (GTK_OBJECT (button), "pressed", GTK_SIGNAL_FUNC (init_classteamresult), (gpointer) CLASSES);
  /*
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
     GTK_SIGNAL_FUNC (init_classteamresult), (gpointer) CLASSES);
   */
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
  gtk_widget_show (button);
  hseparator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (vbox), hseparator, TRUE, TRUE, 0);
  gtk_widget_show (hseparator);
  button = gtk_radio_button_new_with_label (gtk_radio_button_group (GTK_RADIO_BUTTON (button)), _("Teams"));

  gtk_signal_connect (GTK_OBJECT (button), "pressed", GTK_SIGNAL_FUNC (init_classteamresult), (gpointer) TEAMS);
  /*
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
     GTK_SIGNAL_FUNC (init_classteamresult), (gpointer) TEAMS);                                      
   */
  gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  dialog_action_area1 = GTK_DIALOG (classtresult_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "hbuttonbox1", hbuttonbox1);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);

  print_button = gtk_button_new_with_label (_("Print"));
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "print_button", print_button);
  gtk_signal_connect (GTK_OBJECT (print_button), "clicked", GTK_SIGNAL_FUNC (print_clist), print_object);
  gtk_widget_show (print_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), print_button);

  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (classtresult_dialog), "cancel_button", cancel_button);
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (classtresult_dialog));
  gtk_widget_show (cancel_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel_button);

  init_classteamresult (NULL, CLASSES);
  /*print_object->listtype = CLASSLIST; */

  gtk_widget_show (classtresult_dialog);
}

static void
init_classteamresult (GtkWidget * widget, gpointer selection)
{
  if (GPOINTER_TO_INT (selection) == GPOINTER_TO_INT (CLASSES)) {
    /* All lines from the result text */
    print_object->lines = 0;
    print_object->listtype = CLASSLIST;
    calculate_class_result (MALE);
    calculate_class_result (FEMALE);
  } else {
    print_object->lines = 0;
    print_object->listtype = TEAMRESULTLIST;
    calculate_team_result (MALE);
    calculate_team_result (FEMALE);
  }
}

static void
menuitem_response (GtkWidget * widget, gpointer number)
{
  print_object->event_number = GPOINTER_TO_INT (number);
  if (GTK_TOGGLE_BUTTON (get_widget (classtresult_dialog, "ClassButton"))->active) {
    print_object->listtype = CLASSLIST;
    init_classteamresult (NULL, GINT_TO_POINTER (CLASSES));
  } else {
    print_object->listtype = TEAMRESULTLIST;
    init_classteamresult (NULL, GINT_TO_POINTER (TEAMS));
  }
}

void
calculate_class_result (gint sex)
{
  KClassCount    *classcountptr = NULL;
  GList          *classlist = NULL;
  GList          *firstptr = NULL;	/* Points the first list element */
  gchar          *class = NULL;
  gchar          *text = NULL;
  guint32         seconds;
  gint            time[3];
  gint            a, place;
  GdkFont        *font;

  gtk_text_freeze (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")));
  if (sex == MALE) {
    gtk_text_backward_delete (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), gtk_text_get_length (GTK_TEXT (get_widget (classtresult_dialog, "textwidget"))));
  }
  font = gdk_font_load ("-adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*");
  if (sex == MALE) {
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, _("Men\n"), -1);
  } else {
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, _("Women\n"), -1);
  }
  if (cfg.print_team_separator_lines)
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), NULL, NULL, NULL, "---------------------------------\n", -1);
  else
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), NULL, NULL, NULL, "\n", -1);
  print_object->lines += 2;

  res = execSQL ("select runner.number, name,vorname, vereine.v_name,geb_jahr,sex,result_seconds "
		 "from runner,times, vereine " "where runner.sex = %d and runner.lauf_nr = %d " "and runner.verein = vereine.number " "and runner.number = times.runner_number and runner.number > 0 " "order by geb_jahr DESC", sex, KRunning_event_number.event_number);
  for (a = 0; a < PQntuples (res); a++) {
    classcountptr = create_class_counter (atoi (PQgetvalue (res, a, 0)), atoi (PQgetvalue (res, a, 6)), PQgetvalue (res, a, 1), PQgetvalue (res, a, 2), PQgetvalue (res, a, 3), atoi (PQgetvalue (res, a, 4)));

    if (a == 0) {
      /* The first class is stored in class and printed */
      class = g_strdup_printf ("%s", show_klasse (atoi (PQgetvalue (res, a, 4)), atoi (PQgetvalue (res, a, 5))));
      text = g_strdup_printf ("%s\n", class);
      gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, text, -1);

    }
    if (g_strcasecmp (class, show_klasse (atoi (PQgetvalue (res, a, 4)), atoi (PQgetvalue (res, a, 5)))) != 0) {
      /* If we have a new class go along the classlist and print all runners found */
      classlist = g_list_first (classlist);
      place = 0;
      while (classlist) {
	place++;
	seconds = ((KClassCount *) classlist->data)->time;
	time[0] = seconds / 3600L;
	seconds -= time[0] * 3600L;
	time[1] = seconds / 60L;
	time[2] = seconds - time[1] * 60L;
	text = g_strdup_printf ("%3d %-10s, %-10s %-20s\t%02d:%02d:%02d %3d\n",
				((KClassCount *) classlist->data)->runner, cut_text (((KClassCount *) classlist->data)->name, 10), cut_text (((KClassCount *) classlist->data)->vorname, 10), cut_text (((KClassCount *) classlist->data)->verein, 20), time[0], time[1], time[2], place);
	gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, text, -1);
	print_object->lines++;
	classlist = classlist->next;
      }
      g_list_free (classlist);
      class = g_strdup (show_klasse (atoi (PQgetvalue (res, a, 4)), atoi (PQgetvalue (res, a, 5))));
      text = g_strdup_printf ("\n%s\n", class);
      gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, text, -1);
      print_object->lines++;
    }
    classlist = g_list_insert_sorted (classlist, classcountptr, class_compare_func);
    if (a == 0)
      firstptr = classlist;
  }
  /* Here comes the part for the last class in the list */
  classlist = firstptr;		/*g_list_first (classlist); */
  place = 0;
  while (classlist) {
    place++;
    seconds = ((KClassCount *) classlist->data)->time;
    time[0] = seconds / 3600L;
    seconds -= time[0] * 3600L;
    time[1] = seconds / 60L;
    time[2] = seconds - time[1] * 60L;
    text = g_strdup_printf ("%3d %-10s, %-10s %-20s\t%02d:%02d:%02d %3d\n",
			    ((KClassCount *) classlist->data)->runner, cut_text (((KClassCount *) classlist->data)->name, 10), cut_text (((KClassCount *) classlist->data)->vorname, 10), cut_text (((KClassCount *) classlist->data)->verein, 20), time[0], time[1], time[2], place);
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, text, -1);
    classlist = g_list_next (classlist);
    print_object->lines++;
  }
  gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), NULL, NULL, NULL, "\n", -1);
  print_object->lines++;
  gdk_font_unref (font);
  gtk_text_thaw (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")));
  classlist = firstptr;
  if (classlist) {
    g_list_foreach (classlist, (GFunc) clear_class_counter, classlist->data);
    g_list_free_1 (classlist);
  }
  PQclear (res);
  if (class)
    g_free (class);
  if (text)
    g_free (text);
}

void
calculate_team_result (gint sex)
{
  KTeamCount     *teamcountptr = NULL;
  KRunnerNames   *runnerptr = NULL;
  GList          *teamlist = NULL;
  GList          *teamlist_tmp = NULL;
  GList          *runnerlist = NULL;
  GList          *firstptr_r = NULL;	/* Points the first list element */
  GList          *firstptr_t = NULL;	/* Points the first list element */
  GList          *firstptr_tmp = NULL;	/* Points the first list element */
  gchar          *text;
  gint            time[3];
  guint32         seconds;
  gint            a, teamplace;
  GdkFont        *font;
  if (sex == MALE) {
    gtk_text_backward_delete (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), gtk_text_get_length (GTK_TEXT (get_widget (classtresult_dialog, "textwidget"))));
  }
  font = gdk_font_load ("-adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*");
  if (sex == MALE)
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, _("Men\n"), -1);
  else
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, _("Women\n"), -1);
  if (cfg.print_team_separator_lines)
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), NULL, NULL, NULL, "---------------------------------\n", -1);
  else
    gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), NULL, NULL, NULL, "\n", -1);
  print_object->lines += 2;
  /* Collect all team in a list, their name,and the number 1 */
  res = execSQL ("select number,v_name from vereine where number > 0 order by number");
  for (a = 0; a < PQntuples (res); a++) {
    teamcountptr = create_team_counter (atoi (PQgetvalue (res, a, 0)), PQgetvalue (res, a, 1), 1);
    teamlist = g_list_append (teamlist, teamcountptr);
    if (a == 0)
      firstptr_t = teamlist;
  }

  PQclear (res);

  res = execSQL ("select runner.number, name,vorname,result_seconds " "from runner,times " "where lauf_nr = %d and runner.number = times.runner_number", KRunning_event_number.event_number);
  for (a = 0; a < PQntuples (res); a++) {
    runnerptr = create_runner (atoi (PQgetvalue (res, a, 0)), PQgetvalue (res, a, 1), PQgetvalue (res, a, 2), 0, atoi (PQgetvalue (res, a, 3)));
    runnerlist = g_list_append (runnerlist, runnerptr);
    if (a == 0)
      firstptr_r = runnerlist;
  }
  PQclear (res);

  res = execSQL ("select verein,result_seconds,runner.number " "from runner,times " "where lauf_nr = %d and runner.sex = %d and " "runner.number = times.runner_number and verein > 0 " "order by times.result_seconds ", KRunning_event_number.event_number, sex);
  if (PQresultStatus (res) == 0) {
    g_error ("Select didn't return tuples properly\n");
    exit (2);
  }				/* Some changes appear in here */
  teamlist_tmp = g_list_copy (teamlist);
  firstptr_tmp = teamlist_tmp;
  for (a = 0; a < PQntuples (res); a++) {
    teamlist_tmp = firstptr_tmp;
    teamlist_tmp = teamcount_find (teamlist_tmp, atoi (PQgetvalue (res, a, 0)));

    if (teamlist_tmp != NULL) {
      teamlist = firstptr_t;	/* teamlist -> First (List) */
      teamlist = teamcount_find (teamlist, atoi (PQgetvalue (res, a, 0)));
      ((KTeamCount *) teamlist->data)->runner[((KTeamCount *) teamlist->data)->counter] = atoi (PQgetvalue (res, a, 2));
      ((KTeamCount *) teamlist->data)->counter++;


      ((KTeamCount *) teamlist->data)->time += atoi (PQgetvalue (res, a, 1));
      teamlist = firstptr_t;	/* teamlist -> First (List) */
      teamlist_tmp = g_list_copy (teamlist);
    } else if (teamlist_tmp == NULL) {
      /* Here we come if there are more than 3 members from a team there */
      teamlist = g_list_last (teamlist);
      teamlist = teamcount_find_reverse (teamlist, atoi (PQgetvalue (res, a, 0)));
      teamcountptr = create_team_counter (((KTeamCount *) teamlist->data)->team, ((KTeamCount *) teamlist->data)->teamname, ((KTeamCount *) teamlist->data)->teamnumber + 1);
      teamcountptr->runner[0] = atoi (PQgetvalue (res, a, 2));
      teamcountptr->time = atoi (PQgetvalue (res, a, 1));
      teamcountptr->counter++;
      teamlist = g_list_append (teamlist, teamcountptr);
      teamlist = firstptr_t;	/* teamlist -> First (List) */
      teamlist_tmp = g_list_copy (teamlist);
    }
  }
  PQclear (res);

  teamlist = firstptr_t;	/* teamlist -> First (List) */
  teamlist = g_list_sort (teamlist, time_compare_func);
  teamlist = firstptr_t;	/* teamlist -> First (List) */







  /* The Place inside the teamresult */
  teamplace = 0;
  gtk_text_freeze (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")));
  while (teamlist) {
    if (((KTeamCount *) teamlist->data)->counter == TEAMSIZE) {
      teamplace++;
      seconds = ((KTeamCount *) teamlist->data)->time;
      time[0] = seconds / 3600L;
      seconds -= time[0] * 3600L;
      time[1] = seconds / 60L;
      time[2] = seconds - time[1] * 60L;
      /* Zeiten weiter rausschreiben */
      text = g_strdup_printf ("%d. %-24s (%d) \t\t%02d:%02d:%02d\n", teamplace, cut_text (((KTeamCount *) teamlist->data)->teamname, 24), ((KTeamCount *) teamlist->data)->teamnumber, time[0], time[1], time[2]);
      gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, text, -1);
      g_free (text);
      print_object->lines++;
      for (a = 0; a < TEAMSIZE; a++) {
	runnerlist = runner_find (runnerlist, ((KTeamCount *) teamlist->data)->runner[a]);
	seconds = ((KRunnerNames *) runnerlist->data)->time_seconds;
	time[0] = seconds / 3600L;
	seconds -= time[0] * 3600L;
	time[1] = seconds / 60L;
	time[2] = seconds - time[1] * 60L;
	text = g_strdup_printf (" %3d %-10s, %-10s %02d:%02d:%02d\n", ((KRunnerNames *) runnerlist->data)->number, cut_text (((KRunnerNames *) runnerlist->data)->last_name, 10), cut_text (((KRunnerNames *) runnerlist->data)->first_name, 10), time[0], time[1], time[2]);
	gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), font, NULL, NULL, text, -1);
	g_free (text);
	print_object->lines++;
      }
      if (cfg.print_team_separator_lines)
	gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), NULL, NULL, NULL, "---------------------------------\n", -1);
      else
	gtk_text_insert (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")), NULL, NULL, NULL, "\n", -1);
      print_object->lines++;
    }
    teamlist = teamlist->next;
  }
  gdk_font_unref (font);
  gtk_text_thaw (GTK_TEXT (get_widget (classtresult_dialog, "textwidget")));

  teamlist = firstptr_t;	/* teamlist -> First (List) */
  teamlist_tmp = firstptr_tmp;
  runnerlist = firstptr_r;
  if (runnerlist) {
    g_list_foreach (runnerlist, (GFunc) clear_runner, runnerlist->data);
    g_list_free_1 (runnerlist);
  }
  if (teamlist) {
    g_list_foreach (teamlist, (GFunc) clear_team_counter, teamlist->data);
    g_list_free_1 (teamlist);
  }
  if (teamlist_tmp) {
    g_list_free_1 (teamlist_tmp);
  }
}

KClassCount    *
create_class_counter (gint number, guint32 time, gchar * name, gchar * vorname, gchar * verein, gint geb_jahr)
{
  KClassCount    *classcountptr = NULL;
  classcountptr = (KClassCount *) g_malloc0 (sizeof (KClassCount));
  classcountptr->runner = number;
  classcountptr->time = time;
  classcountptr->name = g_strdup (name);
  classcountptr->vorname = g_strdup (vorname);
  classcountptr->verein = g_strdup (verein);
  classcountptr->geb_jahr = geb_jahr;
  return classcountptr;
}

void
clear_class_counter (KClassCount * classcountptr)
{
  if (classcountptr) {
    g_free (classcountptr->name);
    g_free (classcountptr->vorname);
    g_free (classcountptr->verein);
    g_free (classcountptr);
  }
}

KTeamCount     *
create_team_counter (gint number, gchar * teamname, gint teamnumber)
{
  KTeamCount     *teamptr = NULL;
  teamptr = (KTeamCount *) g_malloc0 (sizeof (KTeamCount));
  teamptr->team = number;
  teamptr->teamname = g_strdup (teamname);
  teamptr->teamnumber = teamnumber;
  teamptr->time = 0;
  teamptr->counter = 0;		/* On Init it is always 0 */
  return teamptr;
}

void
clear_team_counter (KTeamCount * teamptr)
{
  if (teamptr) {
    g_free (teamptr->teamname);
    g_free (teamptr);
  }
}

KRunnerNames   *
create_runner (gint number, gchar * last_name, gchar * first_name, gint birthday, guint32 seconds)
{
  KRunnerNames   *runnerptr = NULL;
  runnerptr = (KRunnerNames *) g_malloc (sizeof (KRunnerNames));
  runnerptr->number = number;
  runnerptr->first_name = g_strdup (first_name);
  runnerptr->last_name = g_strdup (last_name);
  runnerptr->birthday = birthday;
  runnerptr->time_seconds = seconds;
  return runnerptr;
}

void
clear_runner (KRunnerNames * runner)
{
  if (runner) {
    g_free (runner->first_name);
    g_free (runner->last_name);
    g_free (runner);
  }
}

GList          *
runner_find (GList * list, gint number)
{
  if (list)
    list = g_list_first (list);
  while (list) {
    if (((KRunnerNames *) list->data)->number == number)
      break;
    list = g_list_next (list);
  }
  return list;
}

GList          *
teamcount_find (GList * list, gint number)
{
  while (list) {
    if (((KTeamCount *) list->data)->team == number && ((KTeamCount *) list->data)->counter < TEAMSIZE)
      break;
    list = g_list_next (list);
  }
  return list;
}

GList          *
teamcount_find_reverse (GList * list, gint number)
{
  while (list) {
    if (((KTeamCount *) list->data)->team == number && ((KTeamCount *) list->data)->counter == TEAMSIZE)
      break;
    list = g_list_previous (list);
  }
  return list;
}

static          gint
class_compare_func (gconstpointer a, gconstpointer b)
{
  guint32         one = ((KClassCount *) a)->time;
  guint32         two = ((KClassCount *) b)->time;
  if (one < two)
    return -1;
  else
    return 1;
}

static          gint
time_compare_func (gconstpointer a, gconstpointer b)
{
  guint32         one = ((KTeamCount *) a)->time;
  guint32         two = ((KTeamCount *) b)->time;
  if (one < two)
    return -1;
  else
    return 1;
}
