/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "krunning_time.h"
#include "database_modules.h"
#include "krunenv.h"
#include "preferences.h"
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>

#ifdef HAVE_XML
#include <parser.h>
#include <tree.h>
#include <debugXML.h>
#endif
/* Dialogs */
GtkWidget      *connection_dialog;

extern char     pixdir[];
#ifdef HAVE_XML
void            save_xml (GtkWidget * widget, GtkWidget * fs);
void            load_xml (GtkWidget * widget, GtkWidget * fs);
static gchar    *xmlGetValue (xmlNodePtr node, const char *name);
static xmlNodePtr xmlSearchChild (xmlNodePtr node, const char *name);
#endif

/*----------------------------------------------------------------------------------------------------
Global variables for postgres connection
----------------------------------------------------------------------------------------------------*/

gchar           *pghost, *pgport, *pgoptions, *pgtty;
gchar           *dbName;
PGconn         *conn;
PGresult       *res;
/* Global declarations for edvereindlg.c and editrundlg.c */
GList          *events;
GList          *vereine;

/*---------------------------------------------
	Wrapper function for SQL statements
----------------------------------------------*/


PGresult       *
execSQL (gchar * format, ...)
{
  va_list         args;
  gchar          *sql_command;
  PGresult       *retval;
  time_t          t;
  int             nTuples;
  t = time (NULL);
  va_start (args, format);
  sql_command = g_strdup_vprintf (format, args);
#ifdef DEBUG
  g_message ("%s\t- '%s'\n", ctime (&t), sql_command);
#endif
  retval = PQexec (conn, sql_command);

  if (!retval) {
#ifdef DEBUG
    g_warning ("A database error has occured - check log file !\n");
#endif
    g_free (sql_command);
    return NULL;
  }
  if (!memcmp (sql_command, "select", 6) && (PQresultStatus (retval) != PGRES_TUPLES_OK)) {
#ifdef DEBUG
    g_warning ("(%d)  \n", PQresultStatus (retval));
    g_warning ("A database error has occured - check log file !\n");
#endif
  } else if (memcmp (sql_command, "select", 6)) {
#ifdef DEBUG
    nTuples = PQresultStatus (retval);
    g_message ("\t  (%d updates)\n\n", nTuples);
#endif
  } else {
    nTuples = PQntuples (retval);
#ifdef DEBUG
    g_message ("\t  (%d Rows)\n\n", nTuples);
#endif /* DEBUG */
  }
  g_free (sql_command);
  va_end (args);
  return retval;
}

void
test_database_again (GtkWidget * widget, gpointer data)
{
  gint            database_ret_code = -1;
  gtk_widget_destroy (widget);
  database_ret_code = test_database ();
  if (database_ret_code != 0) {
    create_question_dialog (DATABASE_ERROR_TEXT, database_ret_code);
  } else
    return;
}



gint
create_database_tables (GtkWidget * widget, gint number)
{

#ifdef DEBUG
  printf ("Database table number %d would be created\n", number);
#endif

  if (number == (DATABASE_TABLES + 1)) {
    /* DESTROY ALL TABLES */
  }
  switch (number) {
  case 1:
    res = execSQL ("drop table vereine");
    res = execSQL ("create table vereine (number int2,v_name text)");
    res = execSQL ("create unique index verein_number on vereine (number)");
    res = execSQL ("create unique index verein_name on vereine (v_name)");
    res = execSQL ("grant ALL on vereine to public");
    break;
  case 2:
    res = execSQL ("drop table runner");
    res = execSQL ("create table runner (number int2, name text," "vorname text,geb_jahr int2,sex int2," "lauf_nr int2,verein int2,strasse text,wohnort text,plz int4, " "chip_no text, " "aktiv int2,place_total int2,place_class int2, " "place_sex int2, place_team int2,place_nation int2)");

    res = execSQL ("create unique index runner_number on runner (number)");
    res = execSQL ("grant ALL on runner to public");
    break;
  case 3:
    res = execSQL ("drop table times");
    res = execSQL ("drop sequence time_sequence");
    res = execSQL ("create table times (number int2,run_number int2, " "runner_number int2,seconds_0 int4,seconds_1 int4, " "seconds_2 int4,seconds_3 int4,seconds_4 int4, " "result_seconds int4 )");
    res = execSQL ("create index time_number on times (number)");
    res = execSQL ("create sequence time_sequence increment 1 start 1 minvalue 1 cycle");
    res = execSQL ("grant ALL on times to public");
    res = execSQL ("grant ALL on time_sequence to public");
    break;
  case 4:
    res = execSQL ("drop table running_events");
    res = execSQL ("create table running_events (number int2,event_name text," "min_number int2,max_number int2,starttime_0 int4, " "starttime_1 int4,starttime_2 int4,starttime_3 int4, " "starttime_4 int4,distance int2)");
    res = execSQL ("create unique index event_number on running_events (number)");
    res = execSQL ("grant ALL on running_events to public");
    break;
  case 5:
    res = execSQL ("drop view statistics");
    res = execSQL ("create view statistics as select COUNT(name) from runner");
    res = execSQL ("grant ALL on statistics to public");
  }
  PQclear (res);
  return 0;
}

GList          *
clear_list (GList * glist)
{
/* We have to clear the list here */
  if (glist != NULL) {
    g_list_free (glist);
    glist = NULL;
  }
  return glist;
}

gint
test_database (void)
{
  /* Really slow and inefficent but ... */
  res = execSQL ("select * from vereine");
  if (PQresultStatus (res) != PGRES_TUPLES_OK) {
#ifdef DEBUG
    printf ("Database error Number %d\n", PQresultStatus (res));
#endif
    PQclear (res);
    return 1;			/* Sould use KRunning Table Types here */
  }
  PQclear (res);
  res = execSQL ("select * from runner");
  if (PQresultStatus (res) != PGRES_TUPLES_OK) {
#ifdef DEBUG
    printf ("Database error Number %d\n", PQresultStatus (res));
#endif
    PQclear (res);
    return 2;
  }
  PQclear (res);
  res = execSQL ("select * from times");
  if (PQresultStatus (res) != PGRES_TUPLES_OK) {
#ifdef DEBUG
    printf ("Database error Number %d\n", PQresultStatus (res));
#endif
    PQclear (res);
    return 3;
  }
  PQclear (res);
  res = execSQL ("select * from running_events");
  if (PQresultStatus (res) != PGRES_TUPLES_OK) {
#ifdef DEBUG
    printf ("Database error Number %d\n", PQresultStatus (res));
#endif
    PQclear (res);
    return 4;
  }
  PQclear (res);

  return 0;
}

void
db_connect (GtkWidget * widget, gpointer data)
{
/* We must syncronise with the preferences first */
  if (cfg.connection_ok == TRUE) {
    cfg.connection_ok = FALSE;
    PQfinish (conn);
    gtk_entry_set_text (GTK_ENTRY (get_widget (connection_dialog, "entry1")), "");
    gtk_entry_set_text (GTK_ENTRY (get_widget (connection_dialog, "entry2")), "");
    gtk_entry_set_text (GTK_ENTRY (get_widget (connection_dialog, "entry3")), "");
  } else {
    conn = PQsetdb (gtk_entry_get_text (GTK_ENTRY (get_widget (connection_dialog, "entry3"))), gtk_entry_get_text (GTK_ENTRY (get_widget (connection_dialog, "entry2"))), NULL, NULL, gtk_entry_get_text (GTK_ENTRY (get_widget (connection_dialog, "entry1"))));
    if (PQstatus (conn) == CONNECTION_BAD) {
      g_warning ("Connection to database '%s' failed.\n", gtk_entry_get_text (GTK_ENTRY (get_widget (connection_dialog, "entry1"))));
      g_warning ("%s", PQerrorMessage (conn));
      cfg.connection_ok = FALSE;
    } else {
      cfg.connection_ok = TRUE;
      strcpy (cfg.database_name, gtk_entry_get_text (GTK_ENTRY (get_widget (connection_dialog, "entry1"))));
      strcpy (cfg.database_host, gtk_entry_get_text (GTK_ENTRY (get_widget (connection_dialog, "entry3"))));
      strcpy (cfg.database_port, gtk_entry_get_text (GTK_ENTRY (get_widget (connection_dialog, "entry2"))));
    }
  }
}

void 
pg_get_connection () 
{
 /* Connect to postgres */
#ifdef DEBUG
  FILE           *debug_file;
#endif

  pgoptions = NULL;		/* special options to start up the backend
				 * server */
  pgtty = NULL;			/* debugging tty for the backend server */

  /* make a connection to the database */
  conn = PQsetdb (cfg.database_host, cfg.database_port, pgoptions, pgtty, cfg.database_name);


  /* check to see that the backend connection was successfully made */
  if (PQstatus (conn) == CONNECTION_BAD) {
    g_warning ("Connection to database '%s' failed.\n", cfg.database_name);
    g_warning ("%s", PQerrorMessage (conn));
    cfg.connection_ok = FALSE;
  } else
    cfg.connection_ok = TRUE;

#ifdef DEBUG
  printf ("Database Name :%s \n", PQdb (conn));
  printf ("Host Name :%s \n", PQhost (conn));
  printf ("Port : %s \n", PQport (conn));
  printf ("Options : %s \n", PQoptions (conn));
  / Trace all commands to a file */
  debug_file = fopen ("./KRunning_debug.sql", "w");
  PQtrace (conn, debug_file);
#endif
}

void
pg_close_connection()
{
 PQfinish (conn);
}

void
create_connection_dialog ()
{
  GtkWidget      *dialog_vbox1;
  GtkWidget      *fixed;
  GtkWidget      *entry1;
  GtkWidget      *entry2;
  GtkWidget      *entry3;
  GtkWidget      *label2;
  GtkWidget      *label3;
  GtkWidget      *label1;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox1;
  GtkWidget      *button;
  GtkWidget      *cancel_button;

  connection_dialog = gtk_dialog_new ();
  /*gtk_widget_ref (connection_dialog); */
  gtk_object_set_data (GTK_OBJECT (connection_dialog), "connection_dialog", connection_dialog);
  GTK_WINDOW (connection_dialog)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (connection_dialog), _("Connect / Disconnect"));
  gtk_window_set_position (GTK_WINDOW (connection_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (connection_dialog), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (connection_dialog)->vbox;
  gtk_widget_ref (dialog_vbox1);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "dialog_vbox1", dialog_vbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dialog_vbox1);

  fixed = gtk_fixed_new ();
  gtk_widget_ref (fixed);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "fixed", fixed, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fixed);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), fixed, TRUE, TRUE, 0);

  entry1 = gtk_entry_new ();
  /*gtk_widget_ref (entry1); */
  gtk_object_set_data (GTK_OBJECT (connection_dialog), "entry1", entry1);
  gtk_widget_show (entry1);
  gtk_fixed_put (GTK_FIXED (fixed), entry1, 152, 48);
  gtk_widget_set_uposition (entry1, 142, 35);
  gtk_widget_set_usize (entry1, 158, 22);

  entry2 = gtk_entry_new_with_max_length (4);
  /*gtk_widget_ref (entry2); */
  gtk_object_set_data (GTK_OBJECT (connection_dialog), "entry2", entry2);
  gtk_widget_show (entry2);
  gtk_fixed_put (GTK_FIXED (fixed), entry2, 152, 88);
  gtk_widget_set_uposition (entry2, 142, 75);
  gtk_widget_set_usize (entry2, 80, 22);

  entry3 = gtk_entry_new ();
  /*gtk_widget_ref (entry3); */
  gtk_object_set_data (GTK_OBJECT (connection_dialog), "entry3", entry3);
  gtk_widget_show (entry3);
  gtk_fixed_put (GTK_FIXED (fixed), entry3, 152, 128);
  gtk_widget_set_uposition (entry3, 142, 115);
  gtk_widget_set_usize (entry3, 158, 22);

  label1 = gtk_label_new (_("Database Name"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "label1", label1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_fixed_put (GTK_FIXED (fixed), label1, 16, 48);
  gtk_widget_set_uposition (label1, 16, 36);

  label2 = gtk_label_new (_("Database Port"));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "label2", label2, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_fixed_put (GTK_FIXED (fixed), label2, 16, 96);
  gtk_widget_set_uposition (label2, 16, 76);

  label3 = gtk_label_new (_("Database Host"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "label3", label3, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_fixed_put (GTK_FIXED (fixed), label3, 24, 136);
  gtk_widget_set_uposition (label3, 16, 116);

  dialog_action_area1 = GTK_DIALOG (connection_dialog)->action_area;
  gtk_widget_ref (dialog_action_area1);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "dialog_action_area1", dialog_action_area1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dialog_action_area1);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "hbuttonbox1", hbuttonbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);

  if (cfg.connection_ok == TRUE) {
    button = gtk_button_new_with_label (_("Disconnect"));
  } else {
    button = gtk_button_new_with_label (_("Connect"));
  }
  gtk_widget_ref (button);
  gtk_object_set_data (GTK_OBJECT (connection_dialog), "button", button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (db_connect), NULL);
  gtk_widget_show (button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button);

  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_ref (cancel_button);
  gtk_object_set_data_full (GTK_OBJECT (connection_dialog), "cancel_button", cancel_button, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cancel_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel_button);

  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (connection_dialog));
  if (cfg.connection_ok == TRUE) {
    gtk_entry_set_text (GTK_ENTRY (entry1), PQdb (conn));
    gtk_entry_set_text (GTK_ENTRY (entry2), PQport (conn));
    gtk_entry_set_text (GTK_ENTRY (entry3), PQhost (conn));
  } else {
    gtk_entry_set_text (GTK_ENTRY (entry1), cfg.database_name);
    gtk_entry_set_text (GTK_ENTRY (entry2), cfg.database_port);
    gtk_entry_set_text (GTK_ENTRY (entry3), cfg.database_host);
  }
  gtk_widget_show (connection_dialog);
}


#ifdef HAVE_XML
/* Input & Output functions */
void
load_xml_dialog ()
{
  GtkWidget      *fs;
  fs = gtk_file_selection_new (_("Load ..."));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".xml");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked", (GtkSignalFunc) load_xml, fs);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (fs));
  gtk_widget_show (fs);
}

void
load_xml (GtkWidget * widget, GtkWidget * fs)
{
  xmlDocPtr       doc;
  xmlNodePtr      node, child;
  gchar          *filename;
  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
  doc = xmlParseFile (filename);
  if (doc == NULL)
    return;
  if (doc->root == NULL) {
    g_error ("KRunning save file empty.");
    xmlFreeDoc (doc);
    return;
  }
  node = doc->root->childs->next;;	/* node points to the team node */
  child = node->childs;		/* points to number */

  while (child != NULL) {
    printf ("-1>%s\n", child->childs->content);
    child = child->next;
    printf ("-2>%s\n", child->childs->content);
    child = child->next;
  }
  node = node->next;
  child = node->childs;		/* points to number (of runners) */
  while (child != NULL) {

    if (child->childs != NULL) {
      printf ("-N>%s\n", child->name);
      printf ("-X>%s\n", child->childs->content);
    } else
      printf ("---> %s\n", child->name);
    child = child->next;
  }

  xmlFreeDoc (doc);
  gtk_widget_destroy (fs);
}

void
save_xml_dialog ()
{
  GtkWidget      *fs;
  fs = gtk_file_selection_new (_("Save as ..."));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".xml");
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked", (GtkSignalFunc) save_xml, fs);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (fs));
  gtk_widget_show (fs);
}

void
save_xml (GtkWidget * widget, GtkWidget * fs)
{
  time_t          zeit;
  xmlDocPtr       doc;
  xmlNodePtr      tree, subtree;
  gchar          *filename;
  gchar          *datum;
  gint            a;
  zeit = time (NULL);
  datum = ctime (&zeit);
  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
  doc = xmlNewDoc ("1.0");
  doc->root = xmlNewDocNode (doc, NULL, "KRunning", NULL);
  xmlSetProp (doc->root, "version", KRUNNING_VERSION);
  xmlSetProp (doc->root, "DATE", datum);
  tree = xmlNewChild (doc->root, NULL, "head", NULL);
  subtree = xmlNewChild (tree, NULL, "title", "KRunning Database savings");
  tree = xmlNewChild (doc->root, NULL, "teams", NULL);
  res = execSQL ("select * from vereine");
  for (a = 0; a < PQntuples (res); a++) {
    subtree = xmlNewChild (tree, NULL, "number", PQgetvalue (res, a, 0));
    subtree = xmlNewChild (tree, NULL, "name", PQgetvalue (res, a, 1));
  }
  PQclear (res);
  tree = xmlNewChild (doc->root, NULL, "runners", NULL);
  res = execSQL ("select * from runner");
  for (a = 0; a < PQntuples (res); a++) {
    subtree = xmlNewChild (tree, NULL, "number", PQgetvalue (res, a, 0));
    subtree = xmlNewChild (tree, NULL, "lastname", PQgetvalue (res, a, 1));
    subtree = xmlNewChild (tree, NULL, "firstname", PQgetvalue (res, a, 2));
    subtree = xmlNewChild (tree, NULL, "birthday", PQgetvalue (res, a, 3));
    subtree = xmlNewChild (tree, NULL, "sex", PQgetvalue (res, a, 4));
    subtree = xmlNewChild (tree, NULL, "run_number", PQgetvalue (res, a, 5));
    subtree = xmlNewChild (tree, NULL, "team_number", PQgetvalue (res, a, 6));
    subtree = xmlNewChild (tree, NULL, "street", PQgetvalue (res, a, 7));
    subtree = xmlNewChild (tree, NULL, "village", PQgetvalue (res, a, 8));
    subtree = xmlNewChild (tree, NULL, "zipcode", PQgetvalue (res, a, 9));
  }
  PQclear (res);
  tree = xmlNewChild (doc->root, NULL, "events", NULL);
  res = execSQL ("select * from running_events");
  for (a = 0; a < PQntuples (res); a++) {
    subtree = xmlNewChild (tree, NULL, "number", PQgetvalue (res, a, 0));
    subtree = xmlNewChild (tree, NULL, "eventname", PQgetvalue (res, a, 1));
    subtree = xmlNewChild (tree, NULL, "min", PQgetvalue (res, a, 2));
    subtree = xmlNewChild (tree, NULL, "max", PQgetvalue (res, a, 3));
    subtree = xmlNewChild (tree, NULL, "starttime_0", PQgetvalue (res, a, 4));
  }
  PQclear (res);
  tree = xmlNewChild (doc->root, NULL, "times", NULL);
  res = execSQL ("select * from times");
  for (a = 0; a < PQntuples (res); a++) {
    subtree = xmlNewChild (tree, NULL, "number", PQgetvalue (res, a, 0));
    subtree = xmlNewChild (tree, NULL, "run_number", PQgetvalue (res, a, 1));
    subtree = xmlNewChild (tree, NULL, "runner_number", PQgetvalue (res, a, 2));
    subtree = xmlNewChild (tree, NULL, "result_seconds", PQgetvalue (res, a, 8));
  }
  PQclear (res);
  xmlSaveFile (filename, doc);
  xmlFreeDoc (doc);

  gtk_widget_destroy (fs);
}

/*
 * Search a child by name, if needed go down the tree to find it. 
 */
static          xmlNodePtr
xmlSearchChild (xmlNodePtr node, const char *name)
{
  xmlNodePtr      ret;
  xmlNodePtr      child;

  child = node->childs;
  while (child != NULL) {
    if (!strcmp (child->name, name))
      return child;
    child = child->next;
  }
  child = node->childs;
  while (child != NULL) {
    ret = xmlSearchChild (child, name);
    if (ret != NULL)
      return ret;
    child = child->next;
  }
  return NULL;
}

/*
 * Get a value for a node either carried as an attibute or as
 * the content of a child.
 */

static char    *
xmlGetValue (xmlNodePtr node, const char *name)
{
  char           *ret;
  xmlNodePtr      child;

  ret = (char *) xmlGetProp (node, name);
  if (ret != NULL)
    return strdup (ret);
  child = node->childs;

  while (child != NULL) {
    if (!strcmp (child->name, name)) {
      /*
       * !!! Inefficient, but ...
       */
      ret = xmlNodeGetContent (child);
      if (ret != NULL)
	return (ret);
    }
    child = child->next;
  }

  return NULL;
}
#endif /* HAVE_XML */
