/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <string.h>
#include "krunning_time.h"
#include "main.h"
#include "menubar.h"
#include "editrundlg.h"
#include "krunenv.h"
#include "printing.h"
#include "database_modules.h"
/* Database Name :
 "RUNNERS"
 Database Format :
 |runner_number(int)|
*/

/* Global declarations for edvereindlg.c and editrundlg.c */
GList          *events = NULL;;
GList          *vereine = NULL;

int             runners_sex = NOT_DEFINED;	/* 0 == MALE && 1 == FEMALE */

GtkWidget      *combo_box_teams = NULL;
GtkWidget      *combo_box_events = NULL;

GtkWidget      *change_button;

static GtkWidget *s_nummer;
static GtkWidget *Klasse;
static GtkWidget *name_entry;
static GtkWidget *prename_entry;
static GtkWidget *geburtsjahr_entry;
static GtkWidget *street_entry;
static GtkWidget *village_entry;
static GtkWidget *zipcode_entry;
static GtkWidget *sex_entry;

GtkWidget      *edit_runner_dlg;

gint            visible = FALSE;

/* change dialog */
static GtkWidget *change_number_dialog;
static GtkWidget *pre_name_label;
static GtkWidget *name_label_2;
static GtkWidget *new_number_entry;
static GtkWidget *old_number_entry;

/* Functions */
static gint     team_combo_box_key_press (GtkWidget * widget, GdkEventKey * event, gpointer data);
static void     team_combo_box_populate (GtkCombo * combo, gchar * text, gboolean complete);

static          gint
key_press_event (GtkWidget * widget, GdkEventKey * event)
{
  switch (event->keyval) {
  case 0xffbe:
#ifdef DEBUG
    g_message ("F1 key pressed ...\n");
#endif
    add_new_runner (NULL, NULL);
    break;

  default:

  }
  return TRUE;
}

static void
focus_change (GtkWidget * widget, gpointer data)
{
  switch (GPOINTER_TO_INT (data)) {
  case 1:
    gtk_widget_grab_focus (GTK_WIDGET (prename_entry));
    break;
  case 2:
    gtk_widget_grab_focus (GTK_WIDGET (geburtsjahr_entry));
    break;
  case 3:
    gtk_widget_grab_focus (GTK_WIDGET (sex_entry));
    break;
  case 4:
    gtk_widget_grab_focus (GTK_WIDGET (GTK_COMBO (combo_box_teams)->entry));
    break;
  default:
    gtk_widget_grab_focus (GTK_WIDGET (name_entry));
  }
}

gint
check_entrys (void)
{				/* This function is to check all entrys */
#ifdef DEBUG
  printf ("Value in entry : %d\n", GTK_ENTRY (name_entry)->text_length);
#endif
  if (GTK_ENTRY (name_entry)->text_length == 0) {
    message_box (WARNING, _("Enter a valid name first !"));
    focus_change (NULL, GINT_TO_POINTER (100));
    return 1;
  }
  if (GTK_ENTRY (prename_entry)->text_length == 0) {
    message_box (WARNING, _("Enter a valid prename first !"));
    focus_change (NULL, GINT_TO_POINTER (1));
    return 1;
  }
  if (atoi (gtk_entry_get_text (GTK_ENTRY (geburtsjahr_entry))) == 19) {
    message_box (WARNING, _("Enter a valid birthdate first !"));
    focus_change (NULL, GINT_TO_POINTER (2));
    return 1;
  }
  if (GTK_ENTRY (sex_entry)->text_length == 0) {
    message_box (WARNING, _("Enter m/f for the runners sex"));
    focus_change (NULL, GINT_TO_POINTER (3));
    return 1;
  }

  return 0;
}

gint
get_runnumber (void)
{
  gint            run_number = 0;

  res = execSQL ("select number from running_events where event_name = '%s'", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_box_events)->entry)));
  run_number = atoi (PQgetvalue (res, 0, 0));
  PQclear (res);
  return run_number;
}

gint
get_teamnumber (void)
{

  gint            teamnumber = 0;
  res = execSQL ("select number from vereine where v_name = '%s'", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_box_teams)->entry)));
  if (PQntuples (res) != 0)
    teamnumber = atoi (PQgetvalue (res, 0, 0));

  PQclear (res);
  return teamnumber;
}

void
clear_entrys (void)
{
  gtk_entry_set_text (GTK_ENTRY (name_entry), "");
  gtk_entry_set_text (GTK_ENTRY (prename_entry), "");
  gtk_entry_set_text (GTK_ENTRY (geburtsjahr_entry), "19");
  gtk_entry_set_text (GTK_ENTRY (sex_entry), "");

  runners_sex = NOT_DEFINED;
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo_box_teams)->entry), _("no team"));
  gtk_label_set (GTK_LABEL (Klasse), "XXX");
  gtk_entry_set_text (GTK_ENTRY (street_entry), "");
  gtk_entry_set_text (GTK_ENTRY (village_entry), "");
  gtk_entry_set_text (GTK_ENTRY (zipcode_entry), "");
  /* Some guys wish to have the focus on the startnumber after clearing everithing */
  gtk_widget_grab_focus (GTK_WIDGET (s_nummer));
}

void
enter_startnumber (GtkWidget * widget, GtkWidget * entry)
{
  static int      vereinsnummer = 0;
  static int      event_nummer = 0;
  static int      in_database = FALSE;
  gchar          *entry_string;
  static int      startnumber;
  entry_string = gtk_entry_get_text (GTK_ENTRY (entry));
  startnumber = atoi (entry_string);
  res = execSQL ("select number from runner where number = %d", startnumber);
#ifdef DEBUG
  printf ("Tuples %d\n", PQntuples (res));
  printf ("Fields %d\n", PQnfields (res));
#endif
  if (PQntuples (res) != 0) {
    /* Is Runner already in Database ?? */
    in_database = TRUE;
  }
  PQclear (res);

  if (in_database == TRUE) {

    res = execSQL ("select name,vorname,geb_jahr,sex,lauf_nr,strasse,plz,wohnort from runner where number = %d", startnumber);

    gtk_entry_set_text (GTK_ENTRY (name_entry), PQgetvalue (res, 0, 0));
    gtk_entry_set_text (GTK_ENTRY (prename_entry), PQgetvalue (res, 0, 1));
    gtk_entry_set_text (GTK_ENTRY (geburtsjahr_entry), PQgetvalue (res, 0, 2));
    gtk_entry_set_text (GTK_ENTRY (street_entry), PQgetvalue (res, 0, 5));
    gtk_entry_set_text (GTK_ENTRY (zipcode_entry), PQgetvalue (res, 0, 6));
    gtk_entry_set_text (GTK_ENTRY (village_entry), PQgetvalue (res, 0, 7));


    if (atoi (PQgetvalue (res, 0, 3)) == MALE) {
      gtk_entry_set_text (GTK_ENTRY (sex_entry), _("m"));
      runners_sex = MALE;
    } else {
      gtk_entry_set_text (GTK_ENTRY (sex_entry), _("f"));
      runners_sex = FEMALE;
    }
    gtk_label_set (GTK_LABEL (Klasse), show_klasse (atoi (PQgetvalue (res, 0, 2)), runners_sex));
    PQclear (res);
    res = execSQL ("select verein,lauf_nr from runner where number = %d", startnumber);
    vereinsnummer = atoi (PQgetvalue (res, 0, 0));
    event_nummer = atoi (PQgetvalue (res, 0, 1));
    PQclear (res);
    res = execSQL ("select event_name from running_events where number = %d", event_nummer);
    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo_box_events)->entry), PQgetvalue (res, 0, 0));
    PQclear (res);
    res = execSQL ("select v_name from vereine where number = %d", vereinsnummer);
    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo_box_teams)->entry), PQgetvalue (res, 0, 0));
    PQclear (res);
  } else {
    clear_entrys ();
  }

  in_database = FALSE;
  gtk_widget_grab_focus (GTK_WIDGET (name_entry));
}

void
enter_callback (GtkWidget * widget, GtkWidget * entry)
{
  /* Default value at begin is NOT_DEFINED !!! */
  gint            year_of_birth;
  gchar          *entry_text;
  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  year_of_birth = atoi (entry_text);
  gtk_label_set (GTK_LABEL (Klasse), show_klasse (year_of_birth, runners_sex));
}



void
enable_menu_again (GtkWidget * widget, gpointer data)
{
  menus_set_sensitive (_("/Edit/Edit Runners"), TRUE);
  visible = FALSE;
  gtk_widget_destroy (edit_runner_dlg);
  return;
}

void
create_Edit_Runners ()
{

  GtkWidget      *dialog_vbox;

  /* Entrys */
  GtkWidget      *chip_no_entry;

  /* For the buttons */
  GtkWidget      *hbuttonbox;
  GtkWidget      *Neu;
  GtkWidget      *abbrechen;
  GtkWidget      *change_button;
  GtkWidget      *delete_button;
  GtkWidget      *print_runner_button;

  /* For labels only */
  GtkWidget      *nothing;

  /* Misc stuff */
  GtkWidget      *dialog_action_area;
  GtkWidget      *fixed;
  GtkWidget      *frame1;

/*  GSList *sex_group = NULL;*/


  /* Thats because there can only be one of this window open !! */
  menus_set_sensitive (_("/Edit/Edit Runners"), FALSE);


  edit_runner_dlg = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "edit_runner_dlg", edit_runner_dlg);
  /*gtk_widget_set_usize (edit_runner_dlg, 309, 350); */
  GTK_WINDOW (edit_runner_dlg)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (edit_runner_dlg), _("Edit - Runners"));
  gtk_window_position (GTK_WINDOW (edit_runner_dlg), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (edit_runner_dlg), FALSE, FALSE, FALSE);
  gtk_signal_connect (GTK_OBJECT (edit_runner_dlg), "delete_event", GTK_SIGNAL_FUNC (enable_menu_again), NULL);
  gtk_signal_connect (GTK_OBJECT (edit_runner_dlg), "destroy", GTK_SIGNAL_FUNC (enable_menu_again), NULL);
  gtk_signal_connect (GTK_OBJECT (edit_runner_dlg), "key_press_event", GTK_SIGNAL_FUNC (key_press_event), NULL);

  dialog_vbox = GTK_DIALOG (edit_runner_dlg)->vbox;
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "dialog_vbox", dialog_vbox);
  gtk_widget_show (dialog_vbox);

  frame1 = gtk_frame_new (NULL);
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "frame1", frame1);
  gtk_widget_show (frame1);
  /*gtk_container_add (GTK_CONTAINER (edit_runner_dlg), frame1); */
  gtk_box_pack_start (GTK_BOX (dialog_vbox), frame1, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame1), 5);

  fixed = gtk_fixed_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "fixed", fixed);
  gtk_widget_show (fixed);
  gtk_container_add (GTK_CONTAINER (frame1), fixed);

  nothing = gtk_label_new (_("Startnumber"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 5);

  s_nummer = gtk_entry_new_with_max_length (6);
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "s_nummer", s_nummer);
  gtk_widget_show (s_nummer);
  gtk_fixed_put (GTK_FIXED (fixed), s_nummer, 5, 20);
  gtk_signal_connect (GTK_OBJECT (s_nummer), "activate", GTK_SIGNAL_FUNC (enter_startnumber), s_nummer);
  gtk_widget_set_usize (s_nummer, 40, 22);


  nothing = gtk_label_new (_("Last name"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 40);

  name_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "name_entry", name_entry);
  gtk_signal_connect (GTK_OBJECT (name_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (1));
  gtk_widget_show (name_entry);
  gtk_fixed_put (GTK_FIXED (fixed), name_entry, 5, 55);

  nothing = gtk_label_new (_("First name"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 75);


  prename_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "prename_entry", prename_entry);
  gtk_signal_connect (GTK_OBJECT (prename_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (2));
  gtk_widget_show (prename_entry);
  gtk_fixed_put (GTK_FIXED (fixed), prename_entry, 5, 90);

  nothing = gtk_label_new (_("Birthday"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 110);

  geburtsjahr_entry = gtk_entry_new_with_max_length (4);
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "geburtsjahr_entry", geburtsjahr_entry);
  gtk_signal_connect (GTK_OBJECT (geburtsjahr_entry), "activate", GTK_SIGNAL_FUNC (enter_callback), geburtsjahr_entry);
  gtk_signal_connect (GTK_OBJECT (geburtsjahr_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (3));
  gtk_entry_set_text (GTK_ENTRY (geburtsjahr_entry), "19");
  gtk_widget_show (geburtsjahr_entry);
  gtk_widget_set_usize (geburtsjahr_entry, 40, 22);
  gtk_fixed_put (GTK_FIXED (fixed), geburtsjahr_entry, 5, 125);

  /* Adress fields */
  nothing = gtk_label_new (_("Street"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 200, 40);

  street_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "street_entry", street_entry);
  gtk_signal_connect (GTK_OBJECT (street_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (1));
  gtk_widget_show (street_entry);
  gtk_fixed_put (GTK_FIXED (fixed), street_entry, 200, 55);


  nothing = gtk_label_new (_("Zipcode"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 200, 75);

  zipcode_entry = gtk_entry_new_with_max_length (5);
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "zipcode_entry", zipcode_entry);
  gtk_signal_connect (GTK_OBJECT (zipcode_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (1));
  gtk_widget_set_usize (zipcode_entry, 50, 22);
  gtk_widget_show (zipcode_entry);
  gtk_fixed_put (GTK_FIXED (fixed), zipcode_entry, 200, 90);


  nothing = gtk_label_new (_("Village"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 200, 110);

  village_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "village_entry", village_entry);
  gtk_signal_connect (GTK_OBJECT (village_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (1));
  gtk_widget_show (village_entry);
  gtk_fixed_put (GTK_FIXED (fixed), village_entry, 200, 125);

  nothing = gtk_label_new (_("Sex"));
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 146);

  sex_entry = gtk_entry_new_with_max_length (1);
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "sex_entry", sex_entry);
  gtk_signal_connect (GTK_OBJECT (sex_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (4));
  gtk_signal_connect (GTK_OBJECT (sex_entry), "activate", GTK_SIGNAL_FUNC (sex_selection), NULL);

  gtk_widget_show (sex_entry);
  gtk_widget_set_usize (sex_entry, 30, 22);
  gtk_fixed_put (GTK_FIXED (fixed), sex_entry, 5, 161);


  nothing = gtk_label_new (_("Class : "));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "nothing", nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 80, 165);
  gtk_widget_show (nothing);

  Klasse = gtk_label_new (_("XXX"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "Klasse", Klasse);
  gtk_fixed_put (GTK_FIXED (fixed), Klasse, 130, 165);
  gtk_widget_show (Klasse);

  nothing = gtk_label_new (_("Team"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "nothing", nothing);
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 210);

  nothing = gtk_label_new (_("Chip Number"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "nothing", nothing);
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 200, 160);
  gtk_widget_set_sensitive (nothing, FALSE);

  chip_no_entry = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "chip_no_entry", chip_no_entry);
  gtk_signal_connect (GTK_OBJECT (chip_no_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (1));
  gtk_widget_show (chip_no_entry);
  gtk_entry_set_text (GTK_ENTRY (chip_no_entry), _("not implemented yet"));
  gtk_widget_set_sensitive (chip_no_entry, FALSE);
  gtk_fixed_put (GTK_FIXED (fixed), chip_no_entry, 200, 175);

  visible = TRUE;

  /* Here we wrote the Team List */
  combo_box_teams = gtk_combo_new ();
  add_vereine ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "combo_box_teams", combo_box_teams);
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo_box_teams)->entry), "kein Verein");

  /*
     gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo_box_teams)->entry), FALSE);
   */
  gtk_editable_select_region (GTK_EDITABLE (GTK_COMBO (combo_box_teams)->entry), 0, -1);
  /* gtk_combo_set_use_arrows(GTK_COMBO(combo_box_teams),1); */
  gtk_widget_show (combo_box_teams);
  gtk_signal_connect (GTK_OBJECT (GTK_COMBO (combo_box_teams)->entry), "key_press_event", GTK_SIGNAL_FUNC (team_combo_box_key_press), combo_box_teams);
  gtk_widget_set_usize (combo_box_teams, 200, -1);
  gtk_fixed_put (GTK_FIXED (fixed), combo_box_teams, 5, 225);

  nothing = gtk_label_new (_("Event"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "nothing", nothing);
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 250);


  add_events ();
  combo_box_events = gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo_box_events), events);

  g_list_free (events);
  events = NULL;

  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "combo_box_events", combo_box_events);
  /*
     gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo_box_events)->entry),
     "kein Verein"); 
   */
  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (combo_box_events)->entry), FALSE);

  gtk_editable_select_region (GTK_EDITABLE (GTK_COMBO (combo_box_events)->entry), 0, -1);
  /* gtk_combo_set_use_arrows(GTK_COMBO(combo_box_events),1); */
  gtk_widget_show (combo_box_events);

  gtk_widget_set_usize (combo_box_events, 200, -1);
  gtk_fixed_put (GTK_FIXED (fixed), combo_box_events, 5, 265);

  nothing = gtk_label_new ("");
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "nothing", nothing);
  gtk_widget_show (nothing);
  gtk_fixed_put (GTK_FIXED (fixed), nothing, 5, 285);

  /* Here are the buttons on that dialog */

  dialog_action_area = GTK_DIALOG (edit_runner_dlg)->action_area;
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "dialog_action_area", dialog_action_area);
  gtk_widget_show (dialog_action_area);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area), 10);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  /*gtk_container_add (GTK_CONTAINER (edit_runner_dlg), hbuttonbox); */
  gtk_box_pack_start (GTK_BOX (dialog_action_area), hbuttonbox, FALSE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbuttonbox), 2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_EDGE);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 10);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox), 2, 0);


  Neu = gtk_button_new_with_label (_("New (F1)"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "Neu", Neu);
  gtk_signal_connect (GTK_OBJECT (Neu), "clicked", GTK_SIGNAL_FUNC (add_new_runner), (gpointer) NULL);
  gtk_widget_show (Neu);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), Neu);

  print_runner_button = gtk_button_new_with_label (_("Print"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "print_runner_button", print_runner_button);
  gtk_signal_connect (GTK_OBJECT (print_runner_button), "clicked", GTK_SIGNAL_FUNC (print_runner_sign), (gpointer) edit_runner_dlg);
  gtk_widget_show (print_runner_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), print_runner_button);


  delete_button = gtk_button_new_with_label (_("Delete"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "delete_button", delete_button);
  gtk_widget_show (delete_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), delete_button);
  gtk_signal_connect_object (GTK_OBJECT (delete_button), "clicked", GTK_SIGNAL_FUNC (delete_db_entry), NULL);

  change_button = gtk_button_new_with_label ("Change Number");
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "change_button", change_button);
  gtk_widget_show (change_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), change_button);
  gtk_signal_connect_object (GTK_OBJECT (change_button), "clicked", GTK_SIGNAL_FUNC (create_change_number_dialog), NULL);

  abbrechen = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (edit_runner_dlg), "abbrechen", abbrechen);
  gtk_widget_show (abbrechen);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), abbrechen);
  gtk_signal_connect_object (GTK_OBJECT (abbrechen), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (edit_runner_dlg));

  gtk_widget_grab_focus (GTK_WIDGET (name_entry));
  gtk_widget_show (edit_runner_dlg);
  return;
}

static          gint
team_combo_box_key_press (GtkWidget * widget, GdkEventKey * event, gpointer data)
{
  GtkCombo       *cb;
  gchar          *text = NULL;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->keyval == GDK_Tab) {
    cb = GTK_COMBO (data);
    text = g_strdup (gtk_entry_get_text (GTK_ENTRY (cb->entry)));
    team_combo_box_populate (cb, text, TRUE);
    if (text);
    g_free (text);

    gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");

    return TRUE;
  }
  return FALSE;
}

static void
team_combo_box_populate (GtkCombo * combo, gchar * text, gboolean complete)
{
  GCompletion    *cmp;
  GList          *result;

  gchar          *new_text = NULL;
  cmp = g_completion_new (FALSE);

  g_completion_add_items (cmp, vereine);

  result = g_completion_complete (cmp, text, &new_text);

  /*g_list_foreach(result,(GFunc) printf ,NULL); */

  if (new_text) {
    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo)->entry), new_text);
    g_free (new_text);
    new_text = NULL;
  }
  g_completion_clear_items (cmp);
  g_completion_free (cmp);
}

void
delete_db_entry (void)
{
  gint            startnumber;
  if (strlen (gtk_entry_get_text (GTK_ENTRY (s_nummer))) > 0) {
    startnumber = atoi (gtk_entry_get_text (GTK_ENTRY (s_nummer)));
    if (startnumber >= 0) {	/* To protect the -1 Numbers !!! */
      res = execSQL ("delete from runner where number = %d", startnumber);
      PQclear (res);
      gtk_entry_set_text (GTK_ENTRY (s_nummer), "");
      clear_entrys ();
      gtk_widget_grab_focus (GTK_WIDGET (name_entry));
    }
  }
}

void
sex_selection (GtkWidget * widget, gpointer data)
{
  /* Changed the memory allocation 20.03.2000 
     Changed the whole thing 16.06.2000 */

  gchar          *sex_string;
  gint            year_of_birth;

  year_of_birth = atoi (gtk_entry_get_text (GTK_ENTRY (geburtsjahr_entry)));
  sex_string = gtk_entry_get_text (GTK_ENTRY (sex_entry));

  if (g_strcasecmp (sex_string, _("m")) == 0) {
    runners_sex = MALE;
    gtk_label_set (GTK_LABEL (Klasse), show_klasse (year_of_birth, runners_sex));
  } else if (g_strcasecmp (sex_string, _("f")) == 0) {	/* FEMALE */
    runners_sex = FEMALE;
    gtk_label_set (GTK_LABEL (Klasse), show_klasse (year_of_birth, runners_sex));
  } else {
    runners_sex = NOT_DEFINED;
    gtk_label_set (GTK_LABEL (Klasse), show_klasse (year_of_birth, runners_sex));
  }
}

void
add_new_runner (GtkWidget * widget, gpointer data)
{
/* What we have to do here is to add a new runner to the database
 * but first we have to check that no runners are twice ....
 * and clean the whole entrys again
 */
  gint            a;
  static int      in_database = FALSE;
  static int      startnumber = 0;
  static int      teamnumber = 0;
  int             run_number = 0;
  static char    *buffer = NULL;
  gchar          *entry_string;
  entry_string = gtk_entry_get_text (GTK_ENTRY (s_nummer));
  startnumber = atoi (entry_string);
  teamnumber = get_teamnumber ();
  run_number = get_runnumber ();
#ifdef DEBUG
  g_print ("Startnummer : %d\n", startnumber);
  g_print ("Teamnumber : %d\n", teamnumber);
  g_print ("Runnumber : %d\n", run_number);
#endif
  if (check_entrys () != 0)
    return;

  res = execSQL ("select number from runner");
  for (a = 0; a < PQntuples (res); a++) {
    if (startnumber == atoi (PQgetvalue (res, a, 0))) {	/* Is Runner already in Database ?? */
      in_database = TRUE;
    }
  }
  PQclear (res);

  if (in_database == TRUE) {	/* Read every entry and update Database */

    res = execSQL ("update runner set name = '%s',vorname = '%s',  geb_jahr = '%d',  "
		   "strasse = '%s', wohnort = '%s', plz = '%d' , sex = '%d', lauf_nr = '%d', verein = '%d', aktiv = '%d' where number = %d ",
		   gtk_entry_get_text (GTK_ENTRY (name_entry)),
		   gtk_entry_get_text (GTK_ENTRY (prename_entry)),
		   atoi (gtk_entry_get_text (GTK_ENTRY (geburtsjahr_entry))), gtk_entry_get_text (GTK_ENTRY (street_entry)), gtk_entry_get_text (GTK_ENTRY (village_entry)), atoi (gtk_entry_get_text (GTK_ENTRY (zipcode_entry))), runners_sex, run_number, teamnumber, 0, startnumber);
    PQclear (res);
    /* After that clean all entrys and increase th number ! */
    buffer = (gchar *) g_malloc0 (sizeof (guint) + 1);
    g_snprintf (buffer, sizeof (guint) + 1, "%d", startnumber + 1);
    gtk_entry_set_text (GTK_ENTRY (s_nummer), buffer);

    g_free (buffer);

    clear_entrys ();
  } else
    /* Add a new runner and calculate the new number */
  {
    if (startnumber == 0) {
      /* We have to calculate a new number here */
#ifdef DEBUG
      g_print ("Laufnummer : %d\n", get_runnumber ());
#endif
      startnumber = get_runnumber ();
      startnumber *= 100;
      res = execSQL ("select number from runner");
      for (a = 0; a < PQntuples (res); a++) {
	if (startnumber == atoi (PQgetvalue (res, a, 0))) {
	  startnumber++;
	}
      }
      PQclear (res);
      if (startnumber == 0) {
	startnumber = 1;
      }
      buffer = (char *) g_malloc0 (5);
      g_snprintf (buffer, 5, "%d", startnumber);
      gtk_entry_set_text (GTK_ENTRY (s_nummer), buffer);
      g_free (buffer);
    }
    res = execSQL ("insert into runner (number,name,vorname,geb_jahr, "
		   " strasse,wohnort,plz, sex,lauf_nr,verein,aktiv,place_class) "
		   "values ('%d','%s','%s','%d', '%s', '%s', '%d', '%d','%d','%d','%d','%d') ",
		   atoi (gtk_entry_get_text (GTK_ENTRY (s_nummer))),
		   gtk_entry_get_text (GTK_ENTRY (name_entry)),
		   gtk_entry_get_text (GTK_ENTRY (prename_entry)),
		   atoi (gtk_entry_get_text (GTK_ENTRY (geburtsjahr_entry))), gtk_entry_get_text (GTK_ENTRY (street_entry)), gtk_entry_get_text (GTK_ENTRY (village_entry)), atoi (gtk_entry_get_text (GTK_ENTRY (zipcode_entry))), runners_sex, run_number, teamnumber, 0, 0);
    PQclear (res);
    buffer = (char *) g_malloc0 (sizeof (guint));
    sprintf (buffer, "%d", startnumber + 1);
    gtk_entry_set_text (GTK_ENTRY (s_nummer), buffer);
    g_free (buffer);
    clear_entrys ();
  }
/* clear */
  in_database = FALSE;
}

void
add_events (void)
{
  gint            a;
  if (events) {
    g_list_free (events);
    events = NULL;
  }
  res = execSQL ("select event_name from running_events");
  for (a = 0; a < PQntuples (res); a++) {
    events = g_list_append (events, g_strdup (PQgetvalue (res, a, 0)));
  }
  PQclear (res);
}

GList          *
add_vereine (void)
{
  gint            a;
  gchar          *teamname = NULL;

  if (vereine) {
    g_list_free (vereine);
    vereine = NULL;
  }
  res = execSQL ("select v_name from vereine order by v_name");
  if (PQntuples (res) != 0) {
    for (a = 0; a < PQntuples (res); a++) {
      vereine = g_list_append (vereine, g_strdup (PQgetvalue (res, a, 0)));
    }
    PQclear (res);
  } else {
    res = execSQL ("insert into vereine (number,v_name) values ('%s','%s');", "0", NO_TEAM);
    vereine = g_list_append (vereine, g_strdup (NO_TEAM));
    PQclear (res);
  }

  if (visible == TRUE) {
    teamname = g_strdup_printf ("%s", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo_box_teams)->entry)));
    gtk_combo_set_popdown_strings (GTK_COMBO (combo_box_teams), vereine);
    gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo_box_teams)->entry), teamname);
  }
  /*g_list_free(vereine);
     vereine = NULL; */
  if (teamname)
    g_free (teamname);
  return vereine;
}

void
create_change_number_dialog ()
{

  GtkWidget      *dialog_vbox1;
  GtkWidget      *frame1;
  GtkWidget      *fixed;
  GtkWidget      *label;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox1;
  GtkWidget      *cancel_button_;
  /* GtkWidget *ok_button; */


  if (change_number_dialog != NULL) {
    gdk_window_raise (change_number_dialog->window);
    return;
  }
  change_number_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "change_number_dialog", change_number_dialog);
  GTK_WINDOW (change_number_dialog)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (change_number_dialog), _("Change Startnumber"));
  gtk_window_position (GTK_WINDOW (change_number_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (change_number_dialog), FALSE, FALSE, FALSE);

  dialog_vbox1 = GTK_DIALOG (change_number_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  frame1 = gtk_frame_new (NULL);
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "frame1", frame1);
  gtk_widget_show (frame1);
  gtk_widget_set_usize (frame1, 320, 160);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame1), 5);

  fixed = gtk_fixed_new ();
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "fixed", fixed);
  gtk_widget_show (fixed);
  gtk_container_add (GTK_CONTAINER (frame1), fixed);

  label = gtk_label_new (_("Enter Old Number :"));
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "label", label);
  gtk_widget_show (label);
  gtk_fixed_put (GTK_FIXED (fixed), label, 5, 7);
  /* gtk_widget_set_usize (label, 34, 16); */

  old_number_entry = gtk_entry_new_with_max_length (6);
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "old_number_entry", old_number_entry);
  gtk_widget_show (old_number_entry);
  gtk_fixed_put (GTK_FIXED (fixed), old_number_entry, 120, 5);
  gtk_widget_set_usize (old_number_entry, 50, 20);
  gtk_signal_connect (GTK_OBJECT (old_number_entry), "activate", GTK_SIGNAL_FUNC (enter_number), old_number_entry);

  gtk_widget_grab_focus (GTK_WIDGET (old_number_entry));


  label = gtk_label_new (_("Name :"));
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "label", label);
  gtk_widget_show (label);
  gtk_fixed_put (GTK_FIXED (fixed), label, 5, 40);

  name_label_2 = gtk_label_new ("");
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "name_label_2", name_label_2);
  gtk_widget_show (name_label_2);
  gtk_fixed_put (GTK_FIXED (fixed), name_label_2, 65, 40);

  label = gtk_label_new (_("Vorname :"));
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "label", label);
  gtk_widget_show (label);
  gtk_fixed_put (GTK_FIXED (fixed), label, 5, 65);

  pre_name_label = gtk_label_new ("");
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "pre_name_label", pre_name_label);
  gtk_widget_show (pre_name_label);
  gtk_fixed_put (GTK_FIXED (fixed), pre_name_label, 65, 65);



  label = gtk_label_new (_("Enter New Number :"));
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "label", label);
  gtk_widget_show (label);
  gtk_fixed_put (GTK_FIXED (fixed), label, 5, 107);
  /* gtk_widget_set_usize (label, 34, 16); */

  new_number_entry = gtk_entry_new_with_max_length (6);
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "new_number_entry", new_number_entry);
  gtk_widget_show (new_number_entry);
  gtk_fixed_put (GTK_FIXED (fixed), new_number_entry, 120, 105);
  gtk_widget_set_usize (new_number_entry, 50, 20);
  gtk_signal_connect (GTK_OBJECT (new_number_entry), "activate", GTK_SIGNAL_FUNC (enter_new_number), new_number_entry);


  dialog_action_area1 = GTK_DIALOG (change_number_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "hbuttonbox1", hbuttonbox1);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);
/*
   ok_button = gtk_button_new_with_label ("Ok");
   gtk_object_set_data (GTK_OBJECT (change_number_dialog), "ok_button", ok_button);
   gtk_widget_show (ok_button);
   gtk_container_add (GTK_CONTAINER (hbuttonbox1), ok_button);
   gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked",
   GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (change_number_dialog));

 */
  cancel_button_ = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (change_number_dialog), "cancel_button_", cancel_button_);
  gtk_widget_show (cancel_button_);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel_button_);
  gtk_signal_connect (GTK_OBJECT (cancel_button_), "clicked", GTK_SIGNAL_FUNC (chancel_dialog), NULL);

  gtk_widget_show (change_number_dialog);
  return;
}

void
chancel_dialog (void)
{
  gtk_widget_destroy (change_number_dialog);
  change_number_dialog = NULL;
}

void
enter_number (GtkWidget * widget, GtkWidget * entry)
{
  res = execSQL ("select name,vorname from runner where number = %d", atoi (gtk_entry_get_text (GTK_ENTRY (entry))));
  if (PQntuples (res) > 0) {
    gtk_label_set (GTK_LABEL (name_label_2), PQgetvalue (res, 0, 0));
    gtk_label_set (GTK_LABEL (pre_name_label), PQgetvalue (res, 0, 1));
    gtk_widget_grab_focus (GTK_WIDGET (new_number_entry));
  } else {
    gtk_label_set (GTK_LABEL (name_label_2), "");
    gtk_label_set (GTK_LABEL (pre_name_label), "");
  }
  PQclear (res);
}

void
enter_new_number (GtkWidget * widget, GtkWidget * entry)
{
  /* First we have to test if the new number already exists */
  res = execSQL ("select number from runner where number = %d", atoi (gtk_entry_get_text (GTK_ENTRY (entry))));
  if (PQntuples (res) == 0) {
    PQclear (res);
    res = execSQL ("update runner set number = '%d' where number = %d", atoi (gtk_entry_get_text (GTK_ENTRY (entry))), atoi (gtk_entry_get_text (GTK_ENTRY (old_number_entry))));
    PQclear (res);
    /* Safety comes first ;-) ! */
    res = execSQL ("update times set runner_number = '%d' where runner_number =%d", atoi (gtk_entry_get_text (GTK_ENTRY (entry))), atoi (gtk_entry_get_text (GTK_ENTRY (old_number_entry))));
    PQclear (res);
    gtk_label_set (GTK_LABEL (name_label_2), "");
    gtk_label_set (GTK_LABEL (pre_name_label), "");
    gtk_entry_set_text (GTK_ENTRY (old_number_entry), "");
    gtk_entry_set_text (GTK_ENTRY (new_number_entry), "");
    gtk_entry_set_text (GTK_ENTRY (s_nummer), "");
    gtk_widget_grab_focus (GTK_WIDGET (old_number_entry));
  } else {
    gtk_widget_grab_focus (GTK_WIDGET (entry));
  }
}
