/*  KRunning
 *  Copyright (C) 1998-2000 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>


#include "edvereindlg.h"
#include "editrundlg.h"
#include "main.h"
#include "show_runners.h"
#include "printing.h"
#include "preferences.h"
#include "krunenv.h"
#include "database_modules.h"
#define VEREINSLISTE_CLIST_COLUMNS 2	// Yes, we change to a clist
#define WINDOW_WIDTH 470
#define WINDOW_HEIGHT 480

GtkWidget      *ev_window;
gint            teamnumber = -1;
static void     clean_up (GtkWidget * widget, gpointer data);
static void     list_remove (GtkWidget * widget, gpointer data);
static void     init_vlist (GtkWidget * list);

static GtkWidget *vereinentry;
/*static GtkWidget *vereinsliste_clist; */
extern PrintObject *print_object;

static gint     vereinsliste_rows = 0;
static gint     vereinsliste_selected_row = 0;

static gboolean edit_name_state = FALSE;

void
select_vereinsliste (GtkWidget * widget, gint row, gint column, GdkEventButton * bevent)
{
  gint            i;
  gchar          *text;
#ifdef DEBUG
  g_print ("GtkCList Selection: row %d column %d button %d\n", row, column, bevent ? bevent->button : 0);
#endif
  for (i = 0; i < VEREINSLISTE_CLIST_COLUMNS; i++) {
    switch (gtk_clist_get_cell_type (GTK_CLIST (widget), row, i)) {
    case GTK_CELL_TEXT:
      if (i == 0) {
	gtk_clist_get_text (GTK_CLIST (widget), row, i, &text);
	teamnumber = atoi (text);
      }
      gtk_clist_get_text (GTK_CLIST (widget), row, i, &text);
      gtk_entry_set_text (GTK_ENTRY (vereinentry), (char *) text);
      edit_name_state = TRUE;
      break;
    default:
      break;
    }
  }
  vereinsliste_selected_row = row;
}


void
unselect_vereinsliste (GtkWidget * widget, gint row, gint column, GdkEventButton * bevent)
{
  gint            i;
  gchar          *text;
#ifdef DEBUG
  g_print ("GtkCList Unselection: row %d column %d button %d\n", row, column, bevent ? bevent->button : 0);
#endif
  for (i = 0; i < VEREINSLISTE_CLIST_COLUMNS; i++) {
    switch (gtk_clist_get_cell_type (GTK_CLIST (widget), row, i)) {
    case GTK_CELL_TEXT:
      gtk_clist_get_text (GTK_CLIST (widget), row, i, &text);
      gtk_entry_set_text (GTK_ENTRY (vereinentry), "");
      edit_name_state = FALSE;
      break;
    default:
      break;
    }
  }
  teamnumber = -1;
  vereinsliste_selected_row = row;
}


void
create_ev_window ()
{
  static char    *titles[] = {
    N_("Number"),
    N_("Teamname"),
  };


  GtkWidget      *Frame, *vbox1, *hbuttonbox, *newentry, *edit, *fixed, *text_label, *close, *delete_item, *print_teams, *scrolled_window;
  GtkWidget      *dialog_action_area2;
  if (ev_window != NULL) {
    gdk_window_raise (ev_window->window);
    return;
  }
  print_object = (PrintObject *) g_malloc (sizeof (PrintObject));

  ev_window = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (ev_window), "ev_window", ev_window);
  GTK_WINDOW (ev_window)->type = GTK_WINDOW_DIALOG;
  gtk_widget_set_usize (ev_window, WINDOW_WIDTH, WINDOW_HEIGHT);
  gtk_window_set_title (GTK_WINDOW (ev_window), _("Edit - Teams"));
  /*gtk_window_set_policy (GTK_WINDOW (ev_window), FALSE, FALSE, FALSE); */
  gtk_container_border_width (GTK_CONTAINER (ev_window), 5);


  print_object->parent_window = ev_window;	/* To set it modal while saving .. */

  vbox1 = GTK_DIALOG (ev_window)->vbox;
  gtk_object_set_data (GTK_OBJECT (ev_window), "vbox1", vbox1);
  gtk_widget_show (vbox1);

  Frame = gtk_frame_new (_("Teams"));
  gtk_object_set_data (GTK_OBJECT (ev_window), "Frame", Frame);
  gtk_widget_show (Frame);
  gtk_container_add (GTK_CONTAINER (vbox1), Frame);
  gtk_frame_set_label_align (GTK_FRAME (Frame), 0, 0.5);


  fixed = gtk_fixed_new ();
  gtk_object_set_data (GTK_OBJECT (ev_window), "fixed", fixed);
  gtk_widget_show (fixed);
  gtk_container_add (GTK_CONTAINER (Frame), fixed);

  text_label = gtk_label_new (_("Teamname"));
  gtk_object_set_data (GTK_OBJECT (ev_window), "text_label", text_label);
  gtk_widget_show (text_label);
  gtk_fixed_put (GTK_FIXED (fixed), text_label, 5, 6);

  vereinentry = gtk_entry_new_with_max_length (MAX_TEAMNAME_LENGTH);
  gtk_object_set_data (GTK_OBJECT (ev_window), "vereinentry", vereinentry);
  gtk_editable_select_region (GTK_EDITABLE (vereinentry), 0, -1);
  gtk_widget_set_usize (vereinentry, 200, 20);
  gtk_widget_show (vereinentry);
  gtk_fixed_put (GTK_FIXED (fixed), vereinentry, 90, 5);
  GTK_WIDGET_SET_FLAGS (vereinentry, GTK_CAN_FOCUS);
  gtk_widget_grab_focus (vereinentry);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_fixed_put (GTK_FIXED (fixed), scrolled_window, 5, 35);
  gtk_widget_set_usize (scrolled_window, WINDOW_WIDTH - 20, WINDOW_HEIGHT - 110);
  gtk_widget_show (scrolled_window);
  /* Configure the print_object->main_clist */
  print_object->main_clist = GTK_CLIST (gtk_clist_new_with_titles (VEREINSLISTE_CLIST_COLUMNS, titles));
  gtk_clist_set_row_height (GTK_CLIST (print_object->main_clist), 20);
  gtk_signal_connect (GTK_OBJECT (print_object->main_clist), "select_row", (GtkSignalFunc) select_vereinsliste, NULL);

  gtk_signal_connect (GTK_OBJECT (print_object->main_clist), "unselect_row", (GtkSignalFunc) unselect_vereinsliste, NULL);

  /* There are only 2 columes yet */
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 0, 50);
  gtk_clist_set_column_auto_resize (GTK_CLIST (print_object->main_clist), 1, TRUE);
  gtk_clist_set_selection_mode (GTK_CLIST (print_object->main_clist), GTK_SELECTION_SINGLE);


  gtk_clist_set_sort_column (GTK_CLIST (print_object->main_clist), cfg.sort_column_teams);
  gtk_clist_set_sort_type (GTK_CLIST (print_object->main_clist), GTK_SORT_ASCENDING);
  init_vlist (GTK_WIDGET (print_object->main_clist));
  /* Signal for the entry field to copy the clist */
  gtk_signal_connect (GTK_OBJECT (vereinentry), "activate", GTK_SIGNAL_FUNC (list_add), print_object->main_clist);
  gtk_clist_set_shadow_type (GTK_CLIST (print_object->main_clist), GTK_SHADOW_IN);

  gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (print_object->main_clist));
  gtk_widget_show (GTK_WIDGET (print_object->main_clist));


  /* For my mouse */
  gtk_widget_realize (GTK_WIDGET (print_object->main_clist));
  gdk_window_set_cursor (GTK_WIDGET (print_object->main_clist)->window, gdk_cursor_new (GDK_HAND1));


  dialog_action_area2 = GTK_DIALOG (ev_window)->action_area;
  gtk_object_set_data (GTK_OBJECT (ev_window), "dialog_action_area2", dialog_action_area2);
  gtk_widget_show (dialog_action_area2);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10);



  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (ev_window), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 5);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox), 80, 25);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), hbuttonbox, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);

  newentry = gtk_button_new_with_label (_("New"));
  gtk_object_set_data (GTK_OBJECT (ev_window), "newentry", newentry);
  gtk_widget_show (newentry);
  gtk_signal_connect (GTK_OBJECT (newentry), "clicked", GTK_SIGNAL_FUNC (list_add), print_object->main_clist);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), newentry);


  edit = gtk_button_new_with_label (_("Edit"));
  gtk_object_set_data (GTK_OBJECT (ev_window), "edit", edit);
  gtk_widget_show (edit);
  gtk_signal_connect (GTK_OBJECT (edit), "clicked", GTK_SIGNAL_FUNC (edit_entry), print_object->main_clist);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), edit);


  delete_item = gtk_button_new_with_label (_("Delete"));
  gtk_object_set_data (GTK_OBJECT (ev_window), "delete_item", delete_item);
  gtk_signal_connect (GTK_OBJECT (delete_item), "clicked", (GtkSignalFunc) list_remove, (gpointer) print_object->main_clist);
  gtk_widget_show (delete_item);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), delete_item);

  print_object->event_number = -1;
  print_object->listtype = TEAMLIST;
  print_object->main_clist = GTK_CLIST (print_object->main_clist);
  print_teams = gtk_button_new_with_label (_("Print"));
  gtk_object_set_data (GTK_OBJECT (ev_window), "print_teams", print_teams);
  gtk_signal_connect (GTK_OBJECT (print_teams), "clicked", (GtkSignalFunc) print_clist, print_object);
  gtk_widget_show (print_teams);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), print_teams);


  close = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (ev_window), "close", close);
  gtk_signal_connect (GTK_OBJECT (close), "clicked", GTK_SIGNAL_FUNC (clean_up), print_object->main_clist);
  gtk_widget_show (close);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), close);

  gtk_widget_show (ev_window);
  /*g_free(print_object); */
}

/*
 * GtkList - The Implementation for the print_object->main_clist
 */


static void
init_vlist (GtkWidget * list)
{
  gint            a;
  gchar           text[VEREINSLISTE_CLIST_COLUMNS][50];
  gchar          *texts[VEREINSLISTE_CLIST_COLUMNS];
  //KRunningTable *team_table;
  texts[0] = text[0];
  texts[1] = text[1];
  gtk_clist_freeze (print_object->main_clist);
  res = execSQL ("select number,v_name from vereine where number > 0 order by number");

  for (a = 0; a < PQntuples (res); a++) {
    sprintf (text[0], "%d", atoi (PQgetvalue (res, a, 0)));
    sprintf (text[1], "%s", PQgetvalue (res, a, 1));
    gtk_clist_insert (GTK_CLIST (print_object->main_clist), a, texts);
    vereinsliste_rows++;
  }
  gtk_clist_sort (print_object->main_clist);
  gtk_clist_thaw (print_object->main_clist);
}

void
edit_entry (GtkWidget * widget, gpointer data)
{
  if (edit_name_state == FALSE)
    return;
  else {
    res = execSQL ("update vereine set v_name= '%s' where number = %d", gtk_entry_get_text (GTK_ENTRY (vereinentry)), teamnumber);
    gtk_clist_set_text (print_object->main_clist, vereinsliste_selected_row, 1, gtk_entry_get_text (GTK_ENTRY (vereinentry)));
    add_vereine ();
    return;
  }
}

void
list_add (GtkWidget * widget, gpointer data)
{
  int             a;
  int             new_number = 1;
  static char     text[VEREINSLISTE_CLIST_COLUMNS][50];
  static char    *texts[VEREINSLISTE_CLIST_COLUMNS];
  for (a = 0; a < VEREINSLISTE_CLIST_COLUMNS; a++) {
    texts[a] = text[a];
    sprintf (text[a], _("empty"));
  }
  if (vereinsliste_rows == 0) {

    res = execSQL ("insert into vereine (number,v_name) values ('%s','%s');", "0", NO_TEAM);
    PQclear (res);


    /* new_number = db_insert_string (database, "TEAMS",NO_TEAM);; */
  } else {
    res = execSQL ("select number from vereine where number > 0 order by number");
    for (a = 0; a < PQntuples (res); a++) {
#ifdef DEBUG
      printf ("Numbers : %s\n", PQgetvalue (res, a, 0));
#endif
      if (atoi (PQgetvalue (res, a, 0)) > new_number && atoi (PQgetvalue (res, a, 0)) > 1) {
	new_number = atoi (PQgetvalue (res, a, 0)) - 1;
	break;
      } else
	new_number = atoi (PQgetvalue (res, a, 0)) + 1;
    }
    PQclear (res);

    if (strlen (gtk_entry_get_text (GTK_ENTRY (vereinentry))) < 1) {
      message_box (ERROR, _("You must enter something !!"));
      return;
    }

  }

  sprintf (texts[0], "%d", new_number);
  sprintf (texts[1], "%s", gtk_entry_get_text GTK_ENTRY (vereinentry));

  res = execSQL ("insert into vereine (number,v_name) values ('%s','%s');", texts[0], text[1]);
  printf ("%d\n", PQresultStatus (res));
  if (PQresultStatus (res) == 0) {
    g_warning ("\nInsert command failed !\n");
    PQclear (res);
  } else {
    PQclear (res);
    gtk_clist_freeze (GTK_CLIST (data));
    gtk_clist_append (GTK_CLIST (data), texts);
    gtk_clist_sort (GTK_CLIST (data));
    gtk_clist_thaw (GTK_CLIST (data));
    vereinsliste_rows++;
#ifdef DEBUG
    printf ("Vereinsname : '%s'\n", gtk_entry_get_text (GTK_ENTRY (vereinentry)));
#endif
    gtk_entry_set_text (GTK_ENTRY (vereinentry), "");
    add_vereine ();
  }
}


static void
list_remove (GtkWidget * widget, gpointer data)
{
  static gchar   *number;
  gtk_clist_get_text (GTK_CLIST (data), vereinsliste_selected_row, 0, &number);
#ifdef DEBUG
  printf ("selected row : %d\n", vereinsliste_selected_row);
  printf ("Number %s\n", number);
#endif


  res = execSQL ("delete from vereine where number = %s", number);
  gtk_clist_remove (GTK_CLIST (data), vereinsliste_selected_row);

  gtk_entry_set_text (GTK_ENTRY (vereinentry), "");
  vereinsliste_rows--;
  add_vereine ();
}

static void
clean_up (GtkWidget * widget, gpointer data)
{
  gtk_widget_destroy (ev_window);
  ev_window = NULL;
  if (vereinsliste_rows <= 0) {
    return;
  }
  vereinsliste_rows = 0;
  add_vereine ();
}
