/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "krunenv.h" 
#include "evaluate.h"
#include "krunning_time.h"
#include "database_modules.h"

#define WINDOW_WIDTH 320
#define WINDOW_HEIGHT 240


static gint     run_number;
GtkWidget      *runners_entry;
GtkWidget      *times_entry;
GtkWidget      *summary_entry;
GtkWidget      *message_label;
GtkWidget      *evalbutton;
GtkWidget      *progressbar;
GtkAdjustment  *adj;

static void
menuitem_response (GtkWidget * widget, gpointer number)
{
  guint           runners;
  guint           times;
  char           *numberptr;
  numberptr = (char *) g_malloc (sizeof (int));
  run_number = GPOINTER_TO_INT (number);
#ifdef DEBUG
  printf ("nummer %d\n", run_number);
#endif
  res = execSQL ("select * from runner where lauf_nr = %d and number > -1", run_number);
  runners = PQntuples (res);
  g_snprintf (numberptr, sizeof (int), "%d", runners);
  PQclear (res);
  gtk_entry_set_text (GTK_ENTRY (runners_entry), numberptr);
  res = execSQL ("select * from times where run_number = %d", run_number);
  times = PQntuples (res);
  g_snprintf (numberptr, sizeof (int), "%d", times);
  PQclear (res);
  gtk_entry_set_text (GTK_ENTRY (times_entry), numberptr);
  g_free (numberptr);
  if (runners - times != 0 || runners == 0) {
    gtk_label_set (GTK_LABEL (message_label), _("Warning"));
    gtk_widget_set_sensitive (evalbutton, FALSE);
  } else {
    gtk_label_set (GTK_LABEL (message_label), _("Ok"));
    gtk_widget_set_sensitive (evalbutton, TRUE);
  }
}

void
create_evaluate_dialog ()
{
  GtkWidget      *evaluate_dialog;
  GtkWidget      *dialog_vbox1;
  GtkWidget      *fixed2;

  GtkWidget      *label;
  GtkWidget      *event_menue;
  GtkWidget      *frame2;
  GtkWidget      *label2;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox;

  GtkWidget      *cancelbutton;
  GtkStyle       *style;

  evaluate_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "evaluate_dialog", evaluate_dialog);
  GTK_WINDOW (evaluate_dialog)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (evaluate_dialog), _("Evaluate"));
  gtk_window_position (GTK_WINDOW (evaluate_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (evaluate_dialog), TRUE, TRUE, FALSE);
  gtk_widget_set_usize (evaluate_dialog, WINDOW_WIDTH, WINDOW_HEIGHT);

  dialog_vbox1 = GTK_DIALOG (evaluate_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  event_menue = create_event_menu ((void (*)(GtkWidget *, gpointer)) menuitem_response, FALSE), gtk_box_pack_start (GTK_BOX (dialog_vbox1), event_menue, TRUE, TRUE, 0);
  gtk_widget_show (event_menue);

  /*

     label = gtk_label_new (_("Event"));
     gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "label", label);
     gtk_widget_show (label);
     gtk_fixed_put (GTK_FIXED (fixed1), label, 5, 5);
   */
  frame2 = gtk_frame_new (_("Statistics"));
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "frame2", frame2);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), frame2, TRUE, TRUE, 0);
/*  gtk_fixed_put (GTK_FIXED (fixed1), frame2, 5, 50);*/
  gtk_widget_set_usize (frame2, WINDOW_WIDTH - 100, 120);

  fixed2 = gtk_fixed_new ();
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "fixed2", fixed2);
  gtk_widget_show (fixed2);
  gtk_container_add (GTK_CONTAINER (frame2), fixed2);

  label = gtk_label_new (_("Number of runners :"));
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "label", label);
  gtk_widget_show (label);
  gtk_fixed_put (GTK_FIXED (fixed2), label, 5, 5);

  runners_entry = gtk_entry_new ();
  gtk_widget_show (runners_entry);
  gtk_fixed_put (GTK_FIXED (fixed2), runners_entry, 150, 5);
  gtk_widget_set_usize (runners_entry, 70, 15);

  label = gtk_label_new (_("Number of times :"));
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "label", label);
  gtk_widget_show (label);
  gtk_fixed_put (GTK_FIXED (fixed2), label, 5, 25);

  times_entry = gtk_entry_new ();
  gtk_widget_show (times_entry);
  gtk_fixed_put (GTK_FIXED (fixed2), times_entry, 150, 25);
  gtk_widget_set_usize (times_entry, 70, 15);

  message_label = gtk_label_new ("");
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "message_label", message_label);
  gtk_widget_show (message_label);
  gtk_fixed_put (GTK_FIXED (fixed2), message_label, 5, 50);
  style = gtk_style_new ();
  style->fg[GTK_STATE_NORMAL] = LRED;
  gtk_widget_set_style (message_label, style);

  adj = (GtkAdjustment *) gtk_adjustment_new (0, 1, 100, 0, 0, 0);

  progressbar = gtk_progress_bar_new_with_adjustment (adj);
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "progressbar", progressbar);
  /* gtk_progress_set_format_string (GTK_PROGRESS (progressbar),
     "%v from [%l-%u] (=%p%%)");
     gtk_progress_set_show_text (GTK_PROGRESS (progressbar),
     TRUE); */
  gtk_widget_show (progressbar);
  gtk_fixed_put (GTK_FIXED (fixed2), progressbar, 5, 70);
  gtk_widget_set_usize (progressbar, 150, 20);


  label2 = gtk_label_new (_(" "));
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "label2", label2);
  gtk_widget_show (label2);
  gtk_fixed_put (GTK_FIXED (fixed2), label2, 24, 96);
  gtk_widget_set_usize (label2, 34, 16);

  dialog_action_area1 = GTK_DIALOG (evaluate_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox, TRUE, TRUE, 0);

  evalbutton = gtk_button_new_with_label (_("Evaluate"));
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "evalbutton", evalbutton);
  gtk_widget_show (evalbutton);
  gtk_signal_connect_object (GTK_OBJECT (evalbutton), "clicked", GTK_SIGNAL_FUNC (evaluate_all_entrys), NULL);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), evalbutton);
  gtk_widget_set_sensitive (evalbutton, FALSE);

  cancelbutton = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (evaluate_dialog), "cancelbutton", cancelbutton);
  gtk_widget_show (cancelbutton);
  gtk_signal_connect_object (GTK_OBJECT (cancelbutton), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (evaluate_dialog));
  gtk_container_add (GTK_CONTAINER (hbuttonbox), cancelbutton);

  menuitem_response (NULL, GINT_TO_POINTER (run_number));

  gtk_widget_show (evaluate_dialog);
}

void
evaluate_all_entrys ()
{
  gfloat          progress_adj = 0.0;
  guint           a = 0;
  guint           total_place = 1;
  gint            women_count = 1;
  gint            man_count = 1;
  classcount     *classdataptr = NULL;
  teamcount      *teamdataptr = NULL;
  evaluate_entrys *evaluate_entryptr = NULL;
  GList          *classcountlist = NULL;
  GList          *classcount_tmp = NULL;
  GList          *teamcountlist = NULL;
  GList          *teamcount_tmp = NULL;
  GList          *evaluatelist = NULL;
  GList          *eval_firstptr = NULL;	/* Points the first list element */
  gtk_widget_set_sensitive (evalbutton, FALSE);
  /* Set progressbar */
  gtk_progress_bar_update (GTK_PROGRESS_BAR (progressbar), 0.0);
  /* Read from database */
  res = execSQL ("select runner.number,geb_jahr,runner.sex,times.result_seconds,verein " "from runner, times " "where runner.lauf_nr = '%d' and runner_number > -1 and " "runner.number = times.runner_number order by times.result_seconds ", run_number);
  if (PQntuples (res) != 0) {
    adj->upper = (PQntuples (res) * 2);
    gtk_adjustment_changed (adj);
  } else
    adj->upper = 1;
  gtk_progress_set_adjustment (GTK_PROGRESS (progressbar), adj);

  for (a = 0; a < PQntuples (res); a++) {

    evaluate_entryptr = (evaluate_entrys *) g_malloc0 (sizeof (evaluate_entrys));

    classdataptr = (classcount *) g_malloc0 (sizeof (classcount));
    /* Copy the classname into classdata struct */
    classdataptr->klasse = g_strdup (show_klasse (atoi (PQgetvalue (res, a, 1)), atoi (PQgetvalue (res, a, 2))));
    /* And set the value to 1 */
    classdataptr->age_count = 1;
    /* The same for the teams */
    teamdataptr = (teamcount *) g_malloc0 (sizeof (teamcount));
    teamdataptr->teamnumber = atoi (PQgetvalue (res, a, 4));
    teamdataptr->team_count = 1;


    /* Write at first the 1 into total place than increase it by 1 */
    evaluate_entryptr->place_total = total_place++;
    if (get_classlist_length (teamcountlist) != 0) {
      teamcount_tmp = teamlist_find (teamcountlist, teamdataptr);
    }
    if (teamcount_tmp != NULL) {
      ((teamcount *) teamcount_tmp->data)->team_count++;
      evaluate_entryptr->place_team = ((teamcount *) teamcount_tmp->data)->team_count;
      teamcount_tmp = g_list_first (teamcount_tmp);
      teamcountlist = g_list_copy (teamcount_tmp);
    } else {
      evaluate_entryptr->place_team = 1;
      teamcountlist = g_list_append (teamcountlist, teamdataptr);
    }

    /* The list should have a length -> search for the class */
    if (get_classlist_length (classcountlist) != 0) {
#ifdef DEBUG
      g_message ("Laenge = %d\n", get_classlist_length (classcountlist));
#endif
      classcount_tmp = classlist_find (classcountlist, classdataptr);
    }
    /* If the classname was already stored -> increase the counter */

    if (classcount_tmp != NULL) {
      ((classcount *) classcount_tmp->data)->age_count++;
      evaluate_entryptr->place_class = ((classcount *) classcount_tmp->data)->age_count;
      classcount_tmp = g_list_first (classcount_tmp);
      classcountlist = g_list_copy (classcount_tmp);
    } else {
      evaluate_entryptr->place_class = 1;
      classcountlist = g_list_append (classcountlist, classdataptr);
    }
    evaluate_entryptr->runner_number = atoi (PQgetvalue (res, a, 0));

    if (atoi (PQgetvalue (res, a, 2)) == MALE) {
      evaluate_entryptr->place_sex = man_count++;
    } else {
      evaluate_entryptr->place_sex = women_count++;
    }

    evaluatelist = g_list_append (evaluatelist, evaluate_entryptr);
    if (a == 0)
      eval_firstptr = evaluatelist;

    progress_adj = gtk_progress_get_value (GTK_PROGRESS (progressbar)) + 1;
    gtk_progress_set_value (GTK_PROGRESS (progressbar), progress_adj);
    while (gtk_events_pending ())
      gtk_main_iteration ();
  }
  PQclear (res);
  evaluatelist = eval_firstptr;
  while (evaluatelist) {
    res = execSQL ("update runner set place_total = '%d' , place_class = '%d' , place_sex = '%d', place_team = '%d' where number = %d",
		   ((evaluate_entrys *) evaluatelist->data)->place_total, ((evaluate_entrys *) evaluatelist->data)->place_class, ((evaluate_entrys *) evaluatelist->data)->place_sex, ((evaluate_entrys *) evaluatelist->data)->place_team, ((evaluate_entrys *) evaluatelist->data)->runner_number);
    evaluatelist = evaluatelist->next;
    progress_adj = gtk_progress_get_value (GTK_PROGRESS (progressbar)) + 1;
    gtk_progress_set_value (GTK_PROGRESS (progressbar), progress_adj);
    while (gtk_events_pending ())
      gtk_main_iteration ();
    PQclear (res);
  }

  evaluatelist = eval_firstptr;

  if (evaluatelist) {
    g_list_free_1 (evaluatelist);
  }
/*
   #fixme : Noch zu ndern :
 */
  g_list_free (classcountlist);
  g_list_free (classcount_tmp);
  g_list_free (teamcountlist);
  g_list_free (teamcount_tmp);
/*if (classcountlist)
   {
   g_list_foreach (classcountlist, (GFunc) classcount_clear, classcountlist->data);
   g_list_free_1 (classcountlist);
   }
   if (classcountlist_tmp)
   {
   g_list_foreach (classcountlist_tmp, (GFunc) classcount_clear, classcountlist_tmp->data);
   g_list_free_1 (classcountlist_tmp);
   } */
  teamcountlist = NULL;
  teamcount_tmp = NULL;
}

void
classcount_clear (classcount * count)
{
  if (count) {
    g_free (count->klasse);
    g_free (count);
  }
}

gint
get_classlist_length (GList * list)
{
  if (!list)
    return 0;
  return g_list_length (list);
}

GList          *
classlist_find (GList * list, gpointer data)
{
  if (list)
    list = g_list_first (list);
  while (list) {
#ifdef DEBUG
    g_message ("Fund function data -> %s and data %s\n", ((classcount *) list->data)->klasse, ((classcount *) data)->klasse);
#endif
    if (g_strcasecmp (((classcount *) list->data)->klasse, ((classcount *) data)->klasse) == 0)
      break;
    list = list->next;
  }
  return list;
}

GList          *
teamlist_find (GList * list, gpointer data)
{
  if (list)
    list = g_list_first (list);
  while (list) {
    if (((teamcount *) list->data)->teamnumber == ((teamcount *) data)->teamnumber)
      break;
    list = list->next;
  }
  return list;
}
