/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include "krunenv.h"
#include "gui.h"
#include "database_modules.h"
#include "preferences.h"
#include "../pixmaps/pixmap_numbers.h"

#include "../pixmaps/import.xpm"
#include "../pixmaps/gnome-info.xpm"
#include "../pixmaps/gnome-error.xpm"
#include "../pixmaps/gnome-warning.xpm"
#include "../pixmaps/gnome-default.xpm"
#include "../pixmaps/calendar.xpm"
KRunningPixmaps KRunningxpm;

ActualRunningEvent KRunning_event_number;

#define N_BUCKETS 2
#define LAST_BUCKET (N_BUCKETS)	/* bucket number 0 is current color */
#define BUCKET_SIZE 32

static GtkWidget *color_preview[N_BUCKETS];
static gdouble  bucket[N_BUCKETS + 1][4];
static gint     preview_id = 0;

GtkWidget      *
get_widget (GtkWidget * widget, gchar * widget_name)
{
  GtkWidget      *parent, *found_widget;

  for (;;) {
    if (GTK_IS_MENU (widget))
      parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
    else
      parent = widget->parent;
    if (parent == NULL)
      break;
    widget = parent;
  }

  found_widget = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (widget), widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

static void
change_event_number (GtkWidget * widget, gpointer event_number)
{
  KRunning_event_number.event_number = GPOINTER_TO_INT (event_number);
}

static void
reset_event_semaphor (GtkWidget * widget, gpointer data)
{
  KRunning_event_number.semaphor = 1;
}

GtkWidget      *
create_event_menu (void (*event_menue_response) (GtkWidget *, gpointer), gboolean show_all)
{
  GtkWidget      *frame;
  GtkWidget      *hbox;
  GtkWidget      *label = NULL;
  GtkWidget      *event_optionmenu_;
  GtkWidget      *event_optionmenu__menu;
  GtkWidget      *glade_menuitem;
  gint            a;

  frame = gtk_frame_new (_("Select event :"));
  gtk_widget_show (frame);

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox);
  gtk_object_set_data (GTK_OBJECT (frame), "event_hbox", hbox);
  gtk_container_border_width (GTK_CONTAINER (hbox), BORDER_WIDTH);
  gtk_container_add (GTK_CONTAINER (frame), hbox);

  /*  label = gtk_label_new("");
     gtk_widget_show(label);
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0); */
  /* Not ready yet !! */

  KRunning_event_number.semaphor = 1;
  if (KRunning_event_number.semaphor == 1) {
    KRunning_event_number.semaphor = 0;

    event_optionmenu_ = gtk_option_menu_new ();
    gtk_widget_show (event_optionmenu_);
    gtk_object_set_data (GTK_OBJECT (frame), "event_optionmenu", event_optionmenu_);
    gtk_box_pack_start (GTK_BOX (hbox), event_optionmenu_, FALSE, FALSE, 0);
    gtk_container_border_width (GTK_CONTAINER (event_optionmenu_), 1);
    gtk_signal_connect (GTK_OBJECT (event_optionmenu_), "destroy", GTK_SIGNAL_FUNC (reset_event_semaphor), NULL);
    event_optionmenu__menu = gtk_menu_new ();
    gtk_object_set_data (GTK_OBJECT (frame), "event_menu", event_optionmenu__menu);

    res = execSQL ("select number,event_name from running_events");
    for (a = 0; a < PQntuples (res); a++) {
      if (a == 0) {
	KRunning_event_number.event_number = atoi (PQgetvalue (res, a, 0));
      }
      glade_menuitem = gtk_menu_item_new_with_label (PQgetvalue (res, a, 1));
      /* activate-item */
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (change_event_number), GINT_TO_POINTER (atoi (PQgetvalue (res, a, 0))));
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (event_menue_response), GINT_TO_POINTER (atoi (PQgetvalue (res, a, 0))));


      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (event_optionmenu__menu), glade_menuitem);

    }
    PQclear (res);
    /* for the show runners dialog */
    if (show_all) {
      glade_menuitem = gtk_menu_item_new_with_label (_("All"));
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (event_menue_response), (gpointer) ALL_RUNNERS);
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (event_optionmenu__menu), glade_menuitem);
    }
    gtk_option_menu_set_menu (GTK_OPTION_MENU (event_optionmenu_), event_optionmenu__menu);
    gtk_option_menu_set_history (GTK_OPTION_MENU (event_optionmenu_), 0);
  } else {
    /* Text anzeigen wenn schon ein Event Men geffnet worden ist */
    label = gtk_label_new (_("Sorry , close all Event selections before opening a new one ! "));
    gtk_widget_show (label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
  }
  return frame;
}

void
AppendTreeNode (GtkCTree * ctree, gchar * treeLabel, GtkCTreeNode * parent, GtkCTreeNode ** newNode, gint pageIndex)
{
  gchar          *titles[1];
  titles[0] = treeLabel;
  *newNode = gtk_ctree_insert_node (ctree, parent, NULL, titles, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
  gtk_ctree_node_set_row_data (ctree, *newNode, (gpointer) pageIndex);
}

GtkWidget      *
KRunningAppendPage (GtkNotebook * notebook, gchar * title_label, GtkCTree * ctree, gchar * treeLabel, GtkCTreeNode * parent, GtkCTreeNode ** newNode, gint pageIndex)
{
  GtkWidget      *out_vbox;
  GtkWidget      *vbox;
  GtkWidget      *frame;
  GtkWidget      *label;
  gchar          *titles[1];

  out_vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (out_vbox);

  frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
  gtk_box_pack_start (GTK_BOX (out_vbox), frame, FALSE, TRUE, 0);
  gtk_widget_show (frame);

  label = gtk_label_new (title_label);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label), 2, 1);
  gtk_container_add (GTK_CONTAINER (frame), label);
  gtk_widget_show (label);

  vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
  gtk_container_add (GTK_CONTAINER (out_vbox), vbox);
  gtk_widget_show (vbox);

  titles[0] = treeLabel;
  *newNode = gtk_ctree_insert_node (ctree, parent, NULL, titles, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
  gtk_ctree_node_set_row_data (ctree, *newNode, (gpointer) pageIndex);
  gtk_notebook_append_page (notebook, out_vbox, NULL);

  return vbox;
}

void
KRunningCtreeSelect (GtkWidget * widget, GtkCTreeNode * node)
{
  GtkNotebook    *notebook;
  gint            page;

  if (GTK_CLIST (widget)->selection) {
    notebook = (GtkNotebook *) gtk_object_get_user_data (GTK_OBJECT (widget));
    page = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (widget), node);

    gtk_notebook_set_page (notebook, page);
  }
}

void
message_box (KRunningMessageType type, const gchar * format, ...)
{
  GtkWidget      *dialog_window = NULL;
  GtkWidget      *label;
  GtkWidget      *hbox;
  GtkWidget      *hbuttonbox;
  GtkWidget      *button;
  GtkWidget      *dialog_action_area2;
  GtkWidget      *dialog_type_pixmap = NULL;
  gchar          *string;
  va_list         args;

  dialog_window = gtk_dialog_new ();

  gtk_window_set_title (GTK_WINDOW (dialog_window), _("Message"));
  gtk_window_set_policy (GTK_WINDOW (dialog_window), FALSE, FALSE, FALSE);
  gtk_window_set_position (GTK_WINDOW (dialog_window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialog_window), TRUE);
  gtk_container_border_width (GTK_CONTAINER (dialog_window), 0);

  dialog_action_area2 = GTK_DIALOG (dialog_window)->action_area;
  gtk_widget_show (dialog_action_area2);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), hbuttonbox, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbuttonbox), 2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 10);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox), 2, 0);

  button = gtk_button_new_with_label (_("Ok"));
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), button);
  gtk_widget_grab_default (button);
  gtk_widget_show (button);
  gtk_signal_connect_object (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (dialog_window));

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);

  switch (type) {
  case WARNING:
    dialog_type_pixmap = gtk_pixmap_new (KRunningxpm.gnome_warning, KRunningxpm.gnome_warning_mask);
    break;
  case ERROR:
    dialog_type_pixmap = gtk_pixmap_new (KRunningxpm.gnome_error, KRunningxpm.gnome_error_mask);
    break;
  case INFO:
    dialog_type_pixmap = gtk_pixmap_new (KRunningxpm.gnome_info, KRunningxpm.gnome_info_mask);
    break;
  case DEFAULT:
    dialog_type_pixmap = gtk_pixmap_new (KRunningxpm.gnome_default, KRunningxpm.gnome_default_mask);
    break;
  }
  gtk_widget_show (dialog_type_pixmap);
  gtk_box_pack_start (GTK_BOX (hbox), dialog_type_pixmap, FALSE, FALSE, 0);

  va_start (args, format);
  string = g_strdup_vprintf (format, args);
  label = gtk_label_new (string);
  gtk_misc_set_padding (GTK_MISC (label), 10, 10);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);
  va_end (args);

  if (!GTK_WIDGET_VISIBLE (dialog_window))
    gtk_widget_show (dialog_window);
  else
    gtk_widget_destroy (dialog_window);
  g_free (string);
}


GtkWidget      *
clist_title_with_arrow (GtkWidget * clist, char col, char *label_text)
{
  GtkWidget      *hbox = gtk_hbox_new (FALSE, 0);
  GtkWidget      *arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_IN);
  GtkWidget      *label = gtk_label_new (label_text);

  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
  gtk_box_pack_end (GTK_BOX (hbox), arrow, FALSE, TRUE, 0);
  gtk_widget_show (label);

  gtk_widget_show (hbox);
  gtk_clist_set_column_widget (GTK_CLIST (clist), col, hbox);
  return arrow;
}

void
clist_title_with_arrow_set_up_dow (GtkWidget * arrow, char IS_UP)
{
  gtk_arrow_set (GTK_ARROW (arrow), IS_UP ? GTK_ARROW_UP : GTK_ARROW_DOWN, GTK_SHADOW_IN);
}

static void
gtk_kill_widget (GtkWidget * widget, GtkWidget * victim)
{
  gtk_widget_destroy (victim);
}

void
hex_to_color (char *buf, gdouble color[4])
{
  int             r, g, b;
  sscanf (buf, "#%2x%2x%2x", &r, &g, &b);

  color[0] = (double) r / 255.0;
  color[1] = (double) g / 255.0;
  color[2] = (double) b / 255.0;
  color[3] = 0.0;
}

void
hex_to_gdkcolor (char *buf, GdkColor * c)
{
  int             r, g, b;
  sscanf (buf, "#%2x%2x%2x", &r, &g, &b);
  c->red = r * 256;
  c->green = g * 256;
  c->blue = b * 256;
}

void
color_to_hex (gdouble color[4], char *buf)
{
  char            hextable[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
  int             i;

  buf[0] = '#';
  buf[7] = '\0';
  for (i = 0; i < 3; i++) {
    int             val = 255 * color[i];
    buf[i * 2 + 1] = hextable[val / 16];
    buf[i * 2 + 2] = hextable[val % 16];
  }
}
static void
draw_color_preview (gint id, char *button_color)
{
  gulong          i, a;
  guchar          c[3];
  guchar          color[3 * 32];

  if (sscanf (button_color, "#%lx", &i)) {
    c[0] = (i & 0xFF0000) >> 16;
    c[1] = (i & 0x00FF00) >> 8;
    c[2] = (i & 0x0000FF);
  }
  for (i = 0, a = 0; i <= 32; i++) {
    color[a + 0] = c[0];
    color[a + 1] = c[1];
    color[a + 2] = c[2];
    a += 3;
  }

  gtk_preview_reset ();
  color_preview[id] = gtk_preview_new (GTK_PREVIEW_COLOR);
  gtk_preview_size (GTK_PREVIEW (color_preview[id]), BUCKET_SIZE, BUCKET_SIZE);
  for (i = 0; i <= BUCKET_SIZE; i++) {
    gtk_preview_draw_row (GTK_PREVIEW (color_preview[id]), color, 0, i, BUCKET_SIZE);
  }
  gtk_widget_draw (color_preview[id], NULL);

}

static void
color_selected (GtkWidget * ok, GtkWidget * color_dialog)
{
  gdouble         color[3];
  gdouble         old_color[4];
  gchar           hex_color[8];
  gboolean        table_color = FALSE;
  gint            i;

  hex_to_color (cfg.html_table_color, old_color);
  gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_dialog)->colorsel), color);
  color_to_hex (color, hex_color);

  for (i = 0; i <= 3; i++) {
    if (bucket[0][i] != old_color[i]) {
      table_color = TRUE;
    }
  }
  if (table_color == TRUE) {
    if (cfg.html_background_color)
      g_free (cfg.html_background_color);
    cfg.html_background_color = g_strdup (hex_color);
/* CHANGE !!! */
    gtk_widget_hide (color_preview[1]);
    draw_color_preview (1, hex_color);
    gtk_widget_show_now (color_preview[1]);
  } else {
    if (cfg.html_table_color)
      g_free (cfg.html_table_color);
    cfg.html_table_color = g_strdup (hex_color);
    gtk_widget_hide (color_preview[0]);
    draw_color_preview (0, hex_color);
    gtk_widget_show_now (color_preview[0]);
  }

  gtk_widget_destroy (color_dialog);
}

/* Show a colordialog with the given color selected
   get the color from the savefile and save them after closing ...
 */
void
show_color_dialog (GtkWidget * widget, gpointer data)
{
  GtkWidget      *color_selection;
  gdouble         color[4];
  gint            i;
  hex_to_color ((char *) data, color);

  for (i = 0; i <= 3; i++) {
    bucket[0][i] = color[i];
  }

  color_selection = gtk_color_selection_dialog_new (_("Choose a color"));

  gtk_color_selection_set_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_selection)->colorsel), color);

  gtk_widget_hide (GTK_COLOR_SELECTION_DIALOG (color_selection)->help_button);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (color_selection)->ok_button), "clicked", GTK_SIGNAL_FUNC (color_selected), color_selection);
  gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (color_selection)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_kill_widget), color_selection);
  /*
     gtk_signal_connect(GTK_OBJECT(color_selection), "color_changed", GTK_SIGNAL_FUNC(color_changed), NULL);
   */
  gtk_widget_show (color_selection);
}

GtkWidget      *
create_color_button (gchar * button_color, gchar * button_text)
{

  GtkWidget      *button;
  GtkWidget      *hbox;
  GtkWidget      *label;

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);

  label = gtk_label_new (button_text);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  button = gtk_button_new ();
  gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (show_color_dialog), (gpointer) button_color);
  gtk_widget_show (button);
  gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);

  draw_color_preview (preview_id, button_color);
  gtk_widget_show (color_preview[preview_id]);
  gtk_container_add (GTK_CONTAINER (button), color_preview[preview_id]);

  if (preview_id == 0)
    preview_id++;

  return hbox;
}

gboolean
create_question_dialog (gchar * string, gint state)
{
  GtkWidget      *question_window;
  GtkWidget      *yes_button;
  GtkWidget      *no_button;
  GtkWidget      *hbuttonbox;
  GtkWidget      *question_label;
  GtkWidget      *dialog_vbox1;
  GtkWidget      *dialog_action_area1;

  question_window = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (question_window), "question_window", question_window);
  GTK_WINDOW (question_window)->type = GTK_WINDOW_DIALOG;
  gtk_widget_set_usize (question_window, 300, 140);
  gtk_container_border_width (GTK_CONTAINER (question_window), 3);
  gtk_window_set_title (GTK_WINDOW (question_window), _("Question"));
  gtk_window_position (GTK_WINDOW (question_window), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (question_window), TRUE, FALSE, FALSE);
  gtk_signal_connect (GTK_OBJECT (question_window), "delete_event", GTK_SIGNAL_FUNC (test_database_again), NULL);
  gtk_signal_connect (GTK_OBJECT (question_window), "destroy", GTK_SIGNAL_FUNC (test_database_again), NULL);

  dialog_vbox1 = GTK_DIALOG (question_window)->vbox;
  gtk_object_set_data (GTK_OBJECT (question_window), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  question_label = gtk_label_new (string);
  gtk_object_set_data (GTK_OBJECT (question_window), "question_label", question_label);
  gtk_widget_show (question_label);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), question_label, TRUE, TRUE, 0);

  dialog_action_area1 = GTK_DIALOG (question_window)->action_area;
  gtk_object_set_data (GTK_OBJECT (question_window), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 5);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (question_window), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox, TRUE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_EDGE);

  yes_button = gtk_button_new_with_label (_("Yes"));
  gtk_object_set_data (GTK_OBJECT (question_window), "yes_button", yes_button);
  gtk_widget_show (yes_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), yes_button);
  gtk_signal_connect (GTK_OBJECT (yes_button), "clicked", (GtkSignalFunc) create_database_tables, (gpointer) state);
  gtk_signal_connect_object (GTK_OBJECT (yes_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (question_window));
  gtk_widget_grab_focus (yes_button);

  no_button = gtk_button_new_with_label (_("No"));
  gtk_object_set_data (GTK_OBJECT (question_window), "no_button", no_button);
  gtk_widget_show (no_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), no_button);
  gtk_signal_connect_object (GTK_OBJECT (no_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (question_window));
  gtk_widget_show (question_window);

  return TRUE;
}


gint
init_pixmaps (GtkWidget * window)
{
  GdkColor       *background = &window->style->bg[GTK_STATE_NORMAL];
  KRunningxpm.gray_header = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.gray_header_mask, background, gray_header_xpm);
  KRunningxpm.quad = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.quad_mask, background, quad_xpm);
  KRunningxpm.null = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.null_mask, background, n0_xpm);
  KRunningxpm.one = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.one_mask, background, n1_xpm);
  KRunningxpm.two = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.two_mask, background, n2_xpm);
  KRunningxpm.three = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.three_mask, background, n3_xpm);
  KRunningxpm.four = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.four_mask, background, n4_xpm);
  KRunningxpm.five = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.five_mask, background, n5_xpm);
  KRunningxpm.six = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.six_mask, background, n6_xpm);
  KRunningxpm.seven = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.seven_mask, background, n7_xpm);
  KRunningxpm.eight = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.eight_mask, background, n8_xpm);
  KRunningxpm.nine = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.nine_mask, background, n9_xpm);
  KRunningxpm.separator = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.separator_mask, background, colon_xpm);
  KRunningxpm.dot = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.dot_mask, background, dot_xpm);
  KRunningxpm.space = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.space_mask, background, space_xpm);
  KRunningxpm.crossbox = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.crossbox_mask, background, crossbox);
  KRunningxpm.checkedbox = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.checkedbox_mask, background, checkedbox);
  KRunningxpm.import = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.import_mask, background, import_xpm);
  KRunningxpm.gnome_info = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.gnome_info_mask, background, gnome_info_xpm);
  KRunningxpm.gnome_error = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.gnome_error_mask, background, gnome_error_xpm);
  KRunningxpm.gnome_warning = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.gnome_warning_mask, background, gnome_warning_xpm);
  KRunningxpm.gnome_default = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.gnome_default_mask, background, gnome_default_xpm);
  KRunningxpm.calendar = gdk_pixmap_create_from_xpm_d (window->window, &KRunningxpm.calendar_mask, background, calendar_xpm);
/*
      if(led_pixmap) gdk_pixmap_unref(led_pixmap);
      if(led_mask) gdk_pixmap_unref(led_mask);
      led_pixmap = NULL;
      if(led_xpm_fn[0])
        led_pixmap = gdk_pixmap_create_from_xpm( window->window, &led_mask,
					         &style->bg[GTK_STATE_NORMAL],
					         led_xpm_fn );
      if(!led_pixmap)
	led_pixmap = gdk_pixmap_create_from_xpm_d(window->window, &led_mask,
					&style->bg[GTK_STATE_NORMAL], led_xpm);

      if(!led_pixmap) fail2load(led_xpm_fn);
      led_elem_width = ((GdkWindowPrivate *)led_pixmap)->width /15;
      led_elem_height= ((GdkWindowPrivate *)led_pixmap)->height;
    
*/
  return 1;
}
