/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "krunenv.h"
#include "krunning_time.h"
#include "preferences.h"
#include "io.h"
#include "database_modules.h"

#define IS_BLANK(c) (((c) == 0x20) || ((c) == 0x09) || ((c) == 0xa) ||	\
                     ((c) == 0x0D))

static void     save_html (gchar * filename);
static void     load_html (GtkWidget * widget, GtkWidget * fs);
static void     create_select_event (GtkWidget * widget, GtkWidget * fs);
static void     import_save_file (GtkWidget * widget, KImportData * data);

static GtkWidget *fs;

static gint     fileformat = -1;

static void
fileformat_response (GtkWidget * widget, gpointer data)
{
  switch (GPOINTER_TO_INT (data)) {
  case RTL:
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".sav");
    fileformat = RTL;
    break;
#ifdef PGDUMP_INSTALLED
  case POSTGRES:
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".psql");
    fileformat = POSTGRES;
    break;
#endif
  default:
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".html");
    fileformat = HTML;
  }
}

void
import_times (void)
{
  GtkWidget      *optionmenu;
  GtkWidget      *optionmenu_menu;
  GtkWidget      *glade_menuitem;
  GtkWidget      *label;
  fs = gtk_file_selection_new (_("Import Times ..."));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".html");

  label = gtk_label_new (_("File Type :"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), label, FALSE, FALSE, 0);
  optionmenu = gtk_option_menu_new ();
  gtk_widget_show (optionmenu);
  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), optionmenu, FALSE, FALSE, 0);
  optionmenu_menu = gtk_menu_new ();

  glade_menuitem = gtk_menu_item_new_with_label (_("KRunning Times"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (TIMES));
  gtk_widget_show (glade_menuitem);


  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("RTL Save File"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (RTL));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), optionmenu_menu);

  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked", (GtkSignalFunc) create_select_event, fs);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (fs));
  gtk_widget_show (fs);

}


static void
create_select_event (GtkWidget * widget, GtkWidget * fs)
{
  GtkWidget      *select_event;
  GtkWidget      *dialog_vbox1;
  GtkWidget      *fixed1;
  GtkWidget      *textlabel;
  GtkWidget      *filename_label;
  GtkWidget      *event_menue;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox1;
  GtkWidget      *ok_button;
  GtkWidget      *cancel_button;
  /* Temp Datas */

  gchar          *filename;
  KImportData    *fileinfo;
  /* Pixmap */
  GtkWidget      *import_pixmap = NULL;

  fileinfo = (KImportData *) g_malloc (sizeof (KImportData));
  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
  fileinfo->filename = (gchar *) g_malloc (strlen (filename) + 1);
  strcpy (fileinfo->filename, filename);

  select_event = gtk_dialog_new ();
  gtk_widget_ref (select_event);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "select_event", select_event, (GtkDestroyNotify) gtk_widget_unref);
  GTK_WINDOW (select_event)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (select_event), _("Select event"));
  gtk_window_set_position (GTK_WINDOW (select_event), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (select_event), TRUE, FALSE, FALSE);

  dialog_vbox1 = GTK_DIALOG (select_event)->vbox;
  gtk_widget_ref (dialog_vbox1);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "dialog_vbox1", dialog_vbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dialog_vbox1);

  fixed1 = gtk_fixed_new ();
  gtk_widget_ref (fixed1);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "fixed1", fixed1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fixed1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), fixed1, TRUE, TRUE, 0);

  textlabel = gtk_label_new (_("Import times from :"));
  gtk_widget_ref (textlabel);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "textlabel", textlabel, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (textlabel);
  gtk_fixed_put (GTK_FIXED (fixed1), textlabel, 0, 8);
  gtk_widget_set_uposition (textlabel, 5, 8);

  filename_label = gtk_label_new (g_basename (filename));
  gtk_widget_show (filename_label);
  gtk_fixed_put (GTK_FIXED (fixed1), filename_label, 16, 32);
  gtk_widget_set_uposition (filename_label, 5, 32);
  gtk_label_set_justify (GTK_LABEL (filename_label), GTK_JUSTIFY_LEFT);

  event_menue = create_event_menu ((void (*)(GtkWidget *, gpointer)) NULL, FALSE);
  gtk_fixed_put (GTK_FIXED (fixed1), event_menue, 200, 5);
  gtk_widget_show (event_menue);

  import_pixmap = gtk_pixmap_new (KRunningxpm.import, KRunningxpm.import_mask);
  gtk_widget_ref (import_pixmap);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "import_pixmap", import_pixmap, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (import_pixmap);
  gtk_fixed_put (GTK_FIXED (fixed1), import_pixmap, 160, 16);
  gtk_widget_set_uposition (import_pixmap, 160, 16);
  gtk_widget_set_usize (import_pixmap, 24, 24);

  dialog_action_area1 = GTK_DIALOG (select_event)->action_area;
  gtk_widget_ref (dialog_action_area1);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "dialog_action_area1", dialog_action_area1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "hbuttonbox1", hbuttonbox1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox1, TRUE, TRUE, 0);

  ok_button = gtk_button_new_with_label (_("Ok"));
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "ok_button", ok_button, (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", (GtkSignalFunc) import_save_file, fileinfo);
  gtk_widget_show (ok_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), ok_button);


  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_widget_ref (cancel_button);
  gtk_object_set_data_full (GTK_OBJECT (select_event), "cancel_button", cancel_button, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cancel_button);
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (select_event));
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), cancel_button);

  gtk_widget_show (select_event);
  gtk_widget_destroy (fs);
}

/* Importing RTL Time files 
 * and since Version 0.3.0 it could import its own times ..
 */
static void
import_save_file (GtkWidget * widget, KImportData * data)
{
  FILE           *filehandler;
  gchar           line[513];
  gchar         **string;
  guint32         seconds;
  guint32         time_number;
  filehandler = fopen (data->filename, "r");
  if (filehandler != NULL) {
    while (fgets (line, 512, filehandler) != NULL) {
      string = g_strsplit (line, " ", 0);
      if (atoi (string[0]) != 0) {
	seconds = (atoi (string[1]) / 100) % 100;
	seconds = seconds + (((atoi (string[1]) / 10000) % 60) * 60);
	seconds = seconds + (((atoi (string[1]) / 10000) / 60) * 3600);
	res = execSQL ("select nextval ('time_sequence')");
	time_number = atoi (PQgetvalue (res, 0, 0));
	PQclear (res);
	res = execSQL ("insert into times (number,run_number,runner_number,result_seconds) " "values ('%d','%d','%d','%d')", time_number, KRunning_event_number.event_number,	/* Dummy value */
		       -1, seconds);
	PQclear (res);
      }
      if (string)
	g_strfreev (string);
    }
  }
  fclose (filehandler);
}

void
load_html_dialog ()
{
  GtkWidget      *optionmenu;
  GtkWidget      *optionmenu_menu;
  GtkWidget      *glade_menuitem;
  GtkWidget      *label;
  fs = gtk_file_selection_new (_("Load ..."));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".html");

  label = gtk_label_new (_("File Type :"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), label, FALSE, FALSE, 0);
  optionmenu = gtk_option_menu_new ();
  gtk_widget_show (optionmenu);
  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), optionmenu, FALSE, FALSE, 0);
  optionmenu_menu = gtk_menu_new ();

  glade_menuitem = gtk_menu_item_new_with_label (_("KRunning Save File"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (HTML));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("Postgres Dump File"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (POSTGRES));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("RTL Save File"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (RTL));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), optionmenu_menu);

  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked", (GtkSignalFunc) load_html, fs);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (fs));
  gtk_widget_show (fs);
}

void
save_html_dialog ()
{
  GtkWidget      *optionmenu;
  GtkWidget      *optionmenu_menu;
  GtkWidget      *glade_menuitem;
  GtkWidget      *label;
  fs = gtk_file_selection_new (_("Save as ..."));
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".html");
  fileformat = HTML;

  label = gtk_label_new (_("File Type :"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), label, FALSE, FALSE, 0);
  optionmenu = gtk_option_menu_new ();
  gtk_widget_show (optionmenu);
  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), optionmenu, FALSE, FALSE, 0);
  optionmenu_menu = gtk_menu_new ();

  glade_menuitem = gtk_menu_item_new_with_label (_("KRunning Save File"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (HTML));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);

#ifdef PGDUMP_INSTALLED
  glade_menuitem = gtk_menu_item_new_with_label (_("Postgres Dump File"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (POSTGRES));
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);
#endif
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), optionmenu_menu);


  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked", (GtkSignalFunc) save_data, fs);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (fs));
  gtk_widget_show (fs);
}


gboolean
find_index (GArray * array, guint number)
{
  guint           i;
  if (array && array->len > 0) {
    for (i = 0; i <= array->len; i++) {
      if (g_array_index (array, guint, i) == number)
	return TRUE;
    }
  }
  return FALSE;
}

gchar          *
string_strip_n (gchar * string, gint length)
{
  gint            stringsize = -1;
#ifdef DEBUG
  printf ("-->%s , %d\n", string, length);
#endif
  if (string) {
    stringsize = (int) strlen (string);
#ifdef DEBUG
    printf ("size = %d length = %d\n", stringsize, length);
#endif
/* NULL value stored */

    if ((int) stringsize <= (int) length) {
      return "0";		/* Here we must return 0 because atoi wants that */
    } else if (stringsize > length) {
      return g_strndup (string, stringsize - length);
    }
  }
  return NULL;
}

void
insert_teams (GArray * number_array, gchar * line)
{
  gint            a;
  gchar         **string;
  for (a = 0; sizeof (line); a++) {
    if (!IS_BLANK (line[a])) {
      if (line[a] == '<' && line[a + 1] == 'T' && line[a + 2] == 'D' && line[a + 3] == '>') {
	string = g_strsplit (line, "<TD>", 0);
	if (find_index (number_array, atoi (g_strndup (string[1], strlen (string[1]) - 5)))) {
	  g_warning ("Number %s found !!", g_strndup (string[1], strlen (string[1]) - 5));
	  /* No or update */
	  res = execSQL ("update vereine set v_name= '%s' where number = %d", string_strip_n (string[2], 6), atoi (string_strip_n (string[1], 5)));
	  PQclear (res);
	} else {
	  /* Insert new line */
	  res = execSQL ("insert into vereine (number,v_name) " "values ('%s','%s');", string_strip_n (string[1], 5), string_strip_n (string[2], 6));
	  PQclear (res);
	}
	if (string)
	  g_strfreev (string);
      } else
	return;
    }
  }
}



void
insert_runners (GArray * number_array, gchar * line)
{
  gint            a;
  gchar         **string;
  for (a = 0; sizeof (line); a++) {
    if (!IS_BLANK (line[a])) {
      if (line[a] == '<' && line[a + 1] == 'T' && line[a + 2] == 'D' && line[a + 3] == '>') {
	string = g_strsplit (line, "<TD>", 0);
#ifdef DEBUG
	for (a = 1; a <= 17; a++) {
	  printf ("%d.->%d\n", a, strlen (string[a]) - 5);
	}
#endif

	if (find_index (number_array, atoi (string_strip_n (string[1], 5)))) {
	  /* No or update */
	  res = execSQL ("update runner set name='%s',vorname='%s',geb_jah='%d', "
			 "sex='%d', lauf_nr='%d', verein='%d', strasse='%s', "
			 "wohnort='%s', plz='%d',chip_no='%s' ,aktiv='%d', place_total='%d', "
			 "place_class='%d', place_sex='%d',place_team='%d', "
			 "place_nation='%d' where number = '%d'",
			 string_strip_n (string[2], 5), string_strip_n (string[3],
									5),
			 atoi (string_strip_n (string[4], 5)),
			 atoi (string_strip_n (string[5], 5)),
			 atoi (string_strip_n (string[6], 5)),
			 atoi (string_strip_n (string[7], 5)),
			 string_strip_n (string[8], 5), string_strip_n (string[9],
									5),
			 atoi (string_strip_n (string[10], 5)),
			 string_strip_n (string[11], 5),
			 atoi (string_strip_n (string[12], 5)), atoi (string_strip_n (string[13], 5)), atoi (string_strip_n (string[14], 5)), atoi (string_strip_n (string[15], 5)), atoi (string_strip_n (string[16], 5)), atoi (string_strip_n (string[17], 6)), atoi (string_strip_n (string[1], 5)));
	  /* Problems seems to come if string too small !! */
	  PQclear (res);
	} else {
	  /* Insert new line */
	  res = execSQL ("insert into runner (number,name,vorname,geb_jahr, "
			 " sex,lauf_nr,verein,strasse,wohnort,plz,chip_no,aktiv,place_total,place_class,place_sex, "
			 " place_team,place_nation) "
			 "values ('%d','%s','%s','%d', '%d', '%d', '%d', '%s','%s','%d','%d','%d','%d', "
			 " '%d','%d','%d')",
			 atoi (g_strndup (string[1], strlen (string[1]) - 5)),
			 string_strip_n (string[2], 5), string_strip_n (string[3],
									5),
			 atoi (string_strip_n (string[4], 5)),
			 atoi (string_strip_n (string[5], 5)),
			 atoi (string_strip_n (string[6], 5)),
			 atoi (string_strip_n (string[7], 5)),
			 string_strip_n (string[8], 5), string_strip_n (string[9],
									5),
			 atoi (string_strip_n (string[10], 5)),
			 string_strip_n (string[11], 5), atoi (string_strip_n (string[12], 5)), atoi (string_strip_n (string[13], 5)), atoi (string_strip_n (string[14], 5)), atoi (string_strip_n (string[15], 5)), atoi (string_strip_n (string[16], 5)), atoi (string_strip_n (string[17], 6)));
	  PQclear (res);
	}
	if (string) {
	  g_strfreev (string);
	  string = NULL;
	}
      } else
	return;
    }
  }
}

void
insert_times (GHashTable * time_hash, gchar * line)
{
  gint            a, event = -1;
  gchar         **string;
  gchar         **timestring;
  guint32         seconds;
  for (a = 0; sizeof (line); a++) {
    if (!IS_BLANK (line[a])) {
      if (line[a] == '<' && line[a + 1] == 'T' && line[a + 2] == 'D' && line[a + 3] == '>') {
	string = g_strsplit (line, "<TD>", 0);
	event = GPOINTER_TO_INT (g_hash_table_lookup (time_hash, GINT_TO_POINTER (atoi (g_strndup (string[1], strlen (string[1]) - 5)))));
	if (event == atoi (string_strip_n (string[2], 5))) {
	  /* No or update */
	  timestring = g_strsplit (string_strip_n (string[9], 6), ":", 0);
	  seconds = atoi (timestring[0]) * 3600L;
	  seconds += atoi (timestring[1]) * 60L;
	  seconds += atoi (timestring[2]);
	  g_strfreev (timestring);	/* This should free timestring */
	  res = execSQL ("update times set run_number='%d',runner_number='%d', "
			 "seconds_0='%d',seconds_1='%d',seconds_2='%d',seconds_3='%d', "
			 "seconds_4='%d',result_seconds='%d' where number = '%d' ",
			 atoi (string_strip_n (string[2], 5)),
			 atoi (string_strip_n (string[3], 5)), atoi (string_strip_n (string[4], 5)), atoi (string_strip_n (string[5], 5)), atoi (string_strip_n (string[6], 5)), atoi (string_strip_n (string[7], 5)), atoi (string_strip_n (string[8], 5)), seconds, atoi (string_strip_n (string[1], 5)));
	  PQclear (res);
	} else {
	  /* Insert new line */
	  timestring = g_strsplit (string_strip_n (string[9], 6), ":", 0);
	  seconds = atoi (timestring[0]) * 3600L;
	  seconds += atoi (timestring[1]) * 60L;
	  seconds += atoi (timestring[2]);
	  g_strfreev (timestring);	/* This should free timestring */
	  res = execSQL ("insert into times (number,run_number,runner_number, "
			 "seconds_0, seconds_1, seconds_2, seconds_3,seconds_4,result_seconds) "
			 "values ('%d','%d','%d','%d','%d','%d','%d','%d','%d') ",
			 atoi (string_strip_n (string[1], 5)),
			 atoi (string_strip_n (string[2], 5)), atoi (string_strip_n (string[3], 5)), atoi (string_strip_n (string[4], 5)), atoi (string_strip_n (string[5], 5)), atoi (string_strip_n (string[6], 5)), atoi (string_strip_n (string[7], 5)), atoi (string_strip_n (string[8], 5)), seconds);
	  PQclear (res);
	}
	if (string)
	  g_strfreev (string);
	event = -1;
      } else
	return;
    }
  }

}


void
insert_events (GArray * number_array, gchar * line)
{
  gint            a;
  gchar         **string;
  for (a = 0; sizeof (line); a++) {
    if (!IS_BLANK (line[a])) {
      if (line[a] == '<' && line[a + 1] == 'T' && line[a + 2] == 'D' && line[a + 3] == '>') {
	string = g_strsplit (line, "<TD>", 0);
	if (find_index (number_array, atoi (g_strndup (string[1], strlen (string[1]) - 5)))) {
	  /* No or update */
	  res = execSQL ("update running_events set eventname='%s',min_number='%d', "
			 "max_number='%d',starttime_0='%d',starttime_1='%d',starttime_2='%d',starttime_3='%d',starttime_4='%d', "
			 "distance='%d' where number ='%d' ",
			 string_strip_n (string[2], 5),
			 atoi (string_strip_n (string[3], 5)),
			 atoi (string_strip_n (string[4], 5)),
			 atoi (string_strip_n (string[5], 5)), atoi (string_strip_n (string[6], 5)), atoi (string_strip_n (string[7], 5)), atoi (string_strip_n (string[8], 5)), atoi (string_strip_n (string[9], 5)), atoi (string_strip_n (string[10], 6)), atoi (string_strip_n (string[1], 5)));
	  PQclear (res);
	} else {
	  /* Insert new line */
	  res = execSQL ("insert into running_events (number,event_name, "
			 " min_number, max_number,starttime_0,starttime_1,starttime_2,starttime_3, "
			 "starttime_4,distance) values ('%d','%s','%d','%d','%d','%d','%d','%d', "
			 "'%d','%d') ",
			 atoi (string_strip_n (string[1], 5)),
			 string_strip_n (string[2], 5),
			 atoi (string_strip_n (string[3], 5)),
			 atoi (string_strip_n (string[4], 5)), atoi (string_strip_n (string[5], 5)), atoi (string_strip_n (string[6], 5)), atoi (string_strip_n (string[7], 5)), atoi (string_strip_n (string[8], 5)), atoi (string_strip_n (string[9], 5)), atoi (string_strip_n (string[10], 6)));
	  PQclear (res);
	}
	if (string)
	  g_strfreev (string);
      } else
	return;
    }
  }

}


static void
load_html (GtkWidget * widget, GtkWidget * fs)
{
  FILE           *filehandler;
  gchar          *filename;
  gchar           line[513];
  gint            a, value, pos = 0;
  gint            table = NO_TABLE;
  GArray         *team_array, *runner_array, *event_array;
  GHashTable     *time_hash;
  /* I use this arrays to keep all numbers of each table in them
     to check which numbers are already stored in a table */

  team_array = g_array_new (FALSE, TRUE, sizeof (guint));
  runner_array = g_array_new (FALSE, TRUE, sizeof (guint));
  event_array = g_array_new (FALSE, TRUE, sizeof (guint));
  /*time_array = g_array_new (FALSE, TRUE, sizeof (guint)); */

  time_hash = g_hash_table_new (NULL, NULL);

  res = execSQL ("select number from vereine");
  for (a = 0; a < PQntuples (res); a++) {
    value = atoi (PQgetvalue (res, a, 0));
    g_array_append_val (team_array, value);
  }
  PQclear (res);
  res = execSQL ("select number from runner");
  for (a = 0; a < PQntuples (res); a++) {
    value = atoi (PQgetvalue (res, a, 0));
    g_array_append_val (runner_array, value);
  }
  PQclear (res);
  /*
     Die Zeitnummer und das event zusammen ergeben ob eine Zeit
     schon vorhanden ist. Ist eine Nummer vorhanden aber das event
     stimmt nicht so wird die Zeit neu eingefgt. So das dem event eine
     neue Zeit hinzugefgt wird. Hash Table !!
   */
  res = execSQL ("select number,run_number from times");
  for (a = 0; a < PQntuples (res); a++) {
    /*value = atoi (PQgetvalue (res, a, 0));
       g_array_append_val (time_array, value); */
    /* Insert the number as index and the event as value into the hast table */
    g_hash_table_insert (time_hash, PQgetvalue (res, a, 0), PQgetvalue (res, a, 1));

  }
  PQclear (res);
  res = execSQL ("select number from running_events");
  for (a = 0; a < PQntuples (res); a++) {
    value = atoi (PQgetvalue (res, a, 0));
    g_array_append_val (event_array, value);
  }
  PQclear (res);


  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
  filehandler = fopen (filename, "r+");
  if (filehandler != NULL) {
    /* fgets liest bis zum Zeilentrenner und maximal 512 Zeichen ein */
    while (fgets (line, 512, filehandler) != NULL) {
      if (line != NULL && pos == 0) {
	if (g_strncasecmp (line, "<!-- KRunning Save File -->", 27) != 0) {
	  message_box (WARNING, "Not a KRunning file.");
	  break;
	}
	pos++;
      }
      for (a = 0; a < 255; a++) {
	if (line[a] == '<' && line[a + 1] == '!' && line[a + 2] == '-' && line[a + 3] == '-') {

	  /* Diese Routine mu sich merken in welchem Abschnitt wir uns befinden 
	     und dann an die jeweilige Routine den line string bergeben */
	  if (line[a + 5] == 'T' && line[a + 6] == 'e' && line[a + 7] == 'a' && line[a + 8] == 'm' && line[a + 9] == 's' && table != TEAMS_TABLE) {
	    printf ("Teams section\n");
	    table = TEAMS_TABLE;
	  } else if (line[a + 5] == 'R' && line[a + 6] == 'u' && line[a + 7] == 'n' && line[a + 8] == 'n' && line[a + 9] == 'e' && line[a + 10] == 'r' && line[a + 11] == 's' && table != RUNNERS_TABLE) {
	    printf ("Runners section\n");
	    table = RUNNERS_TABLE;
	  } else if (line[a + 5] == 'E' && line[a + 6] == 'v' && line[a + 7] == 'e' && line[a + 8] == 'n' && line[a + 9] == 't' && line[a + 10] == 's' && table != EVENTS_TABLE) {
	    printf ("Events section\n");
	    table = EVENTS_TABLE;
	  } else if (line[a + 5] == 'T' && line[a + 6] == 'i' && line[a + 7] == 'm' && line[a + 8] == 'e' && line[a + 9] == 's' && table != TIMES_TABLE) {
	    printf ("Times section\n");
	    table = TIMES_TABLE;
	  } else
	    table = NO_TABLE;
	}
      }
      switch (table) {
      case TEAMS_TABLE:
	insert_teams (team_array, line);
	break;
      case RUNNERS_TABLE:
	insert_runners (runner_array, line);
	break;
      case EVENTS_TABLE:
	insert_events (event_array, line);
	break;
      case TIMES_TABLE:
	insert_times (time_hash, line);
	break;
      default:
      }
    }
  }

  /*g_array_free (time_array, TRUE); */
  g_array_free (event_array, TRUE);
  g_array_free (team_array, TRUE);
  g_array_free (runner_array, TRUE);

  g_hash_table_destroy (time_hash);

  fclose (filehandler);
  gtk_widget_destroy (fs);
}


void
print_html_header (FILE * fd)
{
  time_ctrl       time;
  date_format     date;
  time = gettime (time);
  date = get_date ();
  fprintf (fd, "<!-- KRunning Save File -->\n");
  fprintf (fd, "<!-- generated by %s Version %s -->\n", KRUNNING_INFO, KRUNNING_VERSION);
  fprintf (fd, "<!-- Date : %d. %s %d Time : %02d:%02d:%02d -->\n", date.day, date.monthname, date.year, time.currenttime[0], time.currenttime[1], time.currenttime[2]);
  fprintf (fd, "<HTML>\n<HEAD>\n");
  fprintf (fd, "<META NAME=\"krunning_data\" CONTENT=\"KRunning HTML export file\">\n<TITLE>");
  fprintf (fd, "KRunning Saves : %d. %s %d </TITLE>\n", date.day, date.monthname, date.year);
  fprintf (fd, "</HEAD>\n<BODY BGCOLOR=\"%s\" >\n", cfg.html_background_color);
}

void
print_html_footer (FILE * fd)
{
  fprintf (fd, "<br>\n");
  fprintf (fd, "Diese Liste wurde generiert mit <A HREF=\"%s\">%s</A> Version %s\n", URL, KRUNNING_INFO, KRUNNING_VERSION);
  fprintf (fd, "</BODY>\n</HTML>");
}

void
print_page_header (FILE * fd, gchar * eventname)
{
  fprintf (fd, "<H1>%s</H1>\n", eventname);
}

void
print_table_header (FILE * fd)
{
  fprintf (fd, "<TABLE BORDER=\"1\" CELLSPACING=\"0\" CELLPADDING=\"0\" WIDTH=\"90%%\">\n");
  fprintf (fd, "<TR BGCOLOR=\"%s\">\n", cfg.html_table_color);
  fprintf (fd, "<TD>Nummer</TD><TD>Name</TD><TD>Vorname</TD><TD>Verein</TD>" "<TD>Geburtsjahr</TD><TD>Klasse</TD><TD>Zeit</TD><TD>Gesamt</TD><TD>Klasse</TD>" "<TD>Geschlecht</TD><TD>Verein</TD>\n");
  fprintf (fd, "</TR>\n");
}

void
print_table_footer (FILE * fd)
{
  fprintf (fd, "</TABLE>\n");
}

void
print_table_data (FILE * fd, gchar * number, gchar * last_name, gchar * first_name, gchar * team, gchar * birthday, gchar * class, gchar * time, gchar * place_total, gchar * place_class, gchar * place_sex, gchar * place_team)
{
  fprintf (fd, "<TR>\n");
  fprintf (fd,
	   " <TD ALIGN=RIGHT>%s</TD><TD>%s</TD><TD>%s</TD><TD>%s</TD><TD>%s</TD><TD>%s</TD><TD>%s</TD><TD ALIGN=RIGHT>%s</TD><TD ALIGN=RIGHT>%s</TD><TD ALIGN=RIGHT>%s</TD><TD ALIGN=RIGHT>%s</TD>\n",
	   number, last_name, first_name, team, birthday, class, time, place_total, place_class, place_sex, place_team);
  fprintf (fd, "</TR>\n");
}


void
save_data (GtkWidget * widget, GtkWidget * fs)
{
  FILE           *fd;
  gint            pfd[2], rc;
  gchar          *gs_call[3];
  gchar          *filename;
  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
  switch (fileformat) {
  case HTML:
    save_html (filename);
    break;
  case POSTGRES:
    rc = pipe (pfd);		/* Create a new pipe */
    rc = fork ();
    if (rc == -1) {
      g_error ("Cant fork ()");
    } else if (rc == 0) {
      close (fileno (stdin));	/* Close StdInput */
      dup (pfd[0]);
      close (pfd[0]);
      close (fileno (stdout));

      fd = fopen (filename, "w");	/* Open a pipe for writing */
      dup (fileno (fd));
      close (pfd[1]);
      gs_call[0] = "pg_dump";
      gs_call[1] = cfg.database_name;
      gs_call[2] = 0;
      execvp (gs_call[0], gs_call);
      fclose (fd);
      close (pfd[0]);
      close (pfd[1]);
    }

    gtk_widget_destroy (fs);
  default:
  }

}

static void
save_html (gchar * filename)
{
  FILE           *fd;
  gint            times[3];
  guint32         seconds = 0;
  gint            a;

  fd = fopen (filename, "w+");
  if (fd != NULL) {
    print_html_header (fd);
    fprintf (fd, "<!-- Teams begin -->\n");
    fprintf (fd, _("<H1><U>Teams</U></H1>\n"));
    fprintf (fd, "<TABLE>\n");
    res = execSQL ("select * from vereine");
    for (a = 0; a < PQntuples (res); a++) {
      fprintf (fd, " <TR>\n");
      fprintf (fd, "  <TD>%s</TD>", PQgetvalue (res, a, 0));
      fprintf (fd, "<TD>%s</TD>\n", PQgetvalue (res, a, 1));
      fprintf (fd, " </TR>\n");
    }
    fprintf (fd, "</TABLE>\n");
    fprintf (fd, "<!-- Teams end -->\n");
    PQclear (res);
    fprintf (fd, "<!-- Runners begin -->\n");
    fprintf (fd, _("<H1><U>Runners</U></H1>\n"));
    fprintf (fd, "<TABLE>\n");
    res = execSQL ("select * from runner");
    for (a = 0; a < PQntuples (res); a++) {
      fprintf (fd, " <TR>\n");
      fprintf (fd, "  <TD>%s</TD>", PQgetvalue (res, a, 0));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 1));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 2));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 3));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 4));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 5));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 6));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 7));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 8));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 9));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 10));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 11));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 12));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 13));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 14));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 15));
      fprintf (fd, "<TD>%s</TD>\n", PQgetvalue (res, a, 16));
      fprintf (fd, " </TR>\n");
    }
    fprintf (fd, "</TABLE>\n");
    fprintf (fd, "<!-- Runners end -->\n");
    PQclear (res);
    fprintf (fd, "<!-- Events begin -->\n");
    fprintf (fd, _("<H1><U>Events</U></H1>\n"));
    fprintf (fd, "<TABLE>\n");
    res = execSQL ("select * from running_events");
    for (a = 0; a < PQntuples (res); a++) {
      fprintf (fd, " <TR>\n");
      fprintf (fd, "  <TD>%s</TD>", PQgetvalue (res, a, 0));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 1));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 2));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 3));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 4));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 5));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 6));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 7));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 8));
      fprintf (fd, "<TD>%s</TD>\n", PQgetvalue (res, a, 9));
      fprintf (fd, " </TR>\n");
    }
    fprintf (fd, "</TABLE>\n");
    fprintf (fd, "<!-- Events end -->\n");
    PQclear (res);
    fprintf (fd, "<!-- Times begin -->\n");
    fprintf (fd, _("<H1><U>Times</U></H1>\n"));
    fprintf (fd, "<TABLE>\n");
    res = execSQL ("select * from times");
    for (a = 0; a < PQntuples (res); a++) {
      fprintf (fd, " <TR>\n");
      fprintf (fd, "  <TD>%s</TD>", PQgetvalue (res, a, 0));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 1));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 2));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 3));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 4));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 5));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 6));
      fprintf (fd, "<TD>%s</TD>", PQgetvalue (res, a, 7));
      seconds = atoi (PQgetvalue (res, a, 8));
      times[0] = seconds / 3600L;
      seconds -= times[0] * 3600L;
      times[1] = seconds / 60L;
      times[2] = seconds - times[1] * 60L;
      fprintf (fd, "<TD>%02d:%02d:%02d</TD>\n", times[0], times[1], times[2]);
      fprintf (fd, " </TR>\n");
    }
    fprintf (fd, "</TABLE>\n");
    fprintf (fd, "<!-- Times end -->\n");
    PQclear (res);
    print_html_footer (fd);
    fclose (fd);
  }
  gtk_widget_destroy (fs);
}
