/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  time.c v0.0.1 Initial code
 *  time.c v0.0.2 Added ndisplay_text function + textlabel functions
 *  time.c v0.0.3 ndisplay works now correctly 
 *  time.c v0.0.4 added sec_to_string function
 */

#include <signal.h>
#include "krunenv.h"
#include "krunning_time.h"
#include "gui.h"
#include <time.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#define TM_YEAR_BASE 1900
#define EPOCH_YEAR 1970

static gboolean show_label = FALSE;
static int      time_id;

gchar          *
sec_to_string (guint32 seconds)
{
  guint           time[3];
  gchar          *time_string;
  time[0] = seconds / 3600L;
  seconds -= time[0] * 3600L;
  time[1] = seconds / 60L;
  time[2] = seconds - time[1] * 60L;
  time_string = g_strdup_printf ("%02d:%02d:%02d", time[0], time[1], time[2]);
  return time_string;
}

gchar          *
show_klasse (gint year_of_birth, gint runners_sex)
{
  /* Diese Funktion ermittlet die Klasseneinteilung nach den Vorgaben des DLV */
  date_format     datevalue;
  gint            timespan;
  gchar           sex;
  gchar          *class_string = NULL;

  datevalue = get_date ();

  if (year_of_birth < TM_YEAR_BASE || year_of_birth >= datevalue.year) {
    message_box (WARNING, _("Bitte ein gltiges Datum \n grer 1900 eingeben !"));
    return ("XXX");
  }
  if (runners_sex == MALE)
    sex = 'M';
  else if (runners_sex == FEMALE)
    sex = 'W';
  else
    sex = 'X';

  timespan = datevalue.year - year_of_birth;
  if (timespan < 20) {
    if (timespan < 8) {
      class_string = g_strdup_printf ("%c08", sex);
    } else if (timespan < 16) {
      class_string = g_strdup_printf ("%c%02d", sex, timespan);
    }
    if (timespan == 16 || timespan == 17)
      class_string = g_strdup_printf ("%cJB", sex);
    if (timespan == 18 || timespan == 19)
      class_string = g_strdup_printf ("%cJA", sex);
  }
  if (timespan >= 20 && timespan < 30) {
    class_string = g_strdup_printf ("%cHK", sex);
  }
  if (timespan >= 30) {
    if (timespan % 10 < 5) {
      timespan = (timespan / 10) * 10;
      class_string = g_strdup_printf ("%c%02d", sex, timespan);
    } else {
      timespan = (timespan / 10) * 10 + 5;
      class_string = g_strdup_printf ("%c%02d", sex, timespan);
    }
  }
  return class_string;
}

time_ctrl
gettime (time_ctrl timeorga)
{
  struct tm      *tp;
  time_t          curr_time;
  curr_time = time (NULL);
  tp = localtime (&curr_time);
  timeorga.currenttime[0] = tp->tm_hour;
  timeorga.currenttime[1] = tp->tm_min;
  timeorga.currenttime[2] = tp->tm_sec;
  return timeorga;
}

date_format
get_date (void)
{
  struct tm      *tp;
  date_format     datevalue;
  time_t          curr_time;
  curr_time = time (NULL);
  tp = localtime (&curr_time);
  datevalue.day = tp->tm_mday;
  datevalue.month = tp->tm_mon + 1;
  datevalue.year = tp->tm_year + TM_YEAR_BASE;
  datevalue.monthname = month_name_de (datevalue.month);
  return datevalue;
}

textlabel      *
create_textlabel (GtkLabel * label, gchar * text)
{
  textlabel      *tl;
  tl = (textlabel *) g_malloc0 (sizeof (textlabel));
  tl->label = label;
  tl->text = g_strdup (text);
  return tl;
}

void
free_textlabel (textlabel * tl)
{
  if (tl->text)
    g_free (tl->text);
  if (tl)
    g_free (tl);
}

static          gint
redraw_again (gpointer label)
{
  textlabel      *tl = (textlabel *) label;
  if (time_id)
    gtk_timeout_remove (time_id);
  gtk_label_set_text (tl->label, tl->text);
  gtk_widget_show (GTK_WIDGET (tl->label));
  free_textlabel (tl);
  show_label = FALSE;
  return TRUE;
}

void
ndisplay_text (GtkWidget * label, gchar * text, guint sec)
{
  gchar          *tmpstring;
  gchar          *copystring;
  textlabel      *tlabel;
  if (show_label == FALSE) {
    show_label = TRUE;
    gtk_label_get (GTK_LABEL (label), &tmpstring);
    copystring = g_strdup (tmpstring);
    tlabel = create_textlabel (GTK_LABEL (label), copystring);
    gtk_label_set (GTK_LABEL (label), text);
    time_id = gtk_timeout_add (1000 * sec, redraw_again, (gpointer) tlabel);
    g_free (copystring);
  }
}



gchar          *
month_name_de (int n)
{
  static gchar   *name[] = {
    "unbekannter Monat",
    "Januar", "Februar", "Mrz",
    "April", "Mai", "Juni",
    "Juli", "August", "September",
    "Oktober", "November", "Dezember"
  };
  return (n < 1 || n > 12) ? name[0] : name[n];
}
