/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "menubar.h"
#include "about_dlg.h"
#include "main.h"
#include "krunenv.h"
#include "krunning_time.h"
#include "editrundlg.h"
#include "edvereindlg.h"
#include "time_count.h"
#include "show_runners.h"
#include "io.h"

#include "position.h"
#include "ru_event.h"
#include "preferences.h"
#include "printing.h"
#include "evaluate.h"
#include "certdesign.h"
#include "classteamprint.h"
#include "database_modules.h"
#include "../pixmaps/KRunning.xpm"	/* A little Pic */

GtkItemFactory *factory;
GtkWidget      *window;
GtkWidget      *statusbar;
GtkWidget      *total_label;


static void
gtk_ifactory_cb (gpointer callback_data, guint callback_action, GtkWidget * widget)
{
#ifdef DEBUG
  g_message ("ItemFactory: activated \"%s\"", gtk_item_factory_path_from_widget (widget));
  g_message ("Action : %d Data %d \n", callback_action, GPOINTER_TO_INT (callback_data));
#endif
  switch (callback_action) {
  case 0:
    create_connection_dialog ();
    break;
  case 1:
    load_html_dialog ();
    break;
  case 2:
    save_html_dialog ();
    break;
  case 3:
    create_preferences_dialog ();
    break;
  case 4:
    file_quit_cmd_callback (NULL, _("Thank you for using KRunning !"));
    break;
  case 5:
    create_running_event_dialog ();
    break;
  case 6:
    create_Edit_Runners ();
    break;
  case 7:
    create_ev_window ();
    break;
  case 8:
    create_show_runner ();
    break;
  case 9:
    create_get_rtime_window ();
    break;
  case 10:
    create_position_dialog ();
    break;
  case 11:
    create_evaluate_dialog ();
    break;
  case 12:
    create_About ();
    break;
  case 13:
    create_design_dialog ();
    break;
  case 14:
    create_question_dialog (_("Would you really delete all runners ?"), 2);
    break;
  case 15:
    create_question_dialog (_("Would you really delete all times ?"), 3);
    break;
  case 16:
    create_question_dialog (_("Would you really delete all teams ?"), 1);
    break;
  case 17:
    create_question_dialog (_("Would you really delete all events ?"), 4);
    break;
  case 18:
    create_question_dialog (_("Would you really DELETE all tables ?"), DATABASE_TABLES + 1);
    break;
  case 19:
    create_classtresult_dialog ();
    break;
  case 20:
    create_print_certificates_dialog ();
    break;
  case 21:
    import_times ();
    break;
  case 22:
    /*create_instructions();  */
  default:
    break;
  }
}

GtkItemFactory *
create_menu_branch (GtkItemFactory * factory, gchar * path)
{
  GtkItemFactoryEntry *entry;
  entry = (GtkItemFactoryEntry *) g_malloc (sizeof (GtkItemFactoryEntry));
  entry->path = path;
  entry->accelerator = NULL;
  entry->callback = 0;
  entry->callback_action = 0;
  entry->item_type = "<Branch>";
  gtk_item_factory_create_item (factory, entry, NULL, 1);
  return factory;
}

GtkItemFactory *
create_menu_lastbranch (GtkItemFactory * factory, gchar * path)
{
  GtkItemFactoryEntry *entry;
  entry = (GtkItemFactoryEntry *) g_malloc (sizeof (GtkItemFactoryEntry));
  entry->path = path;
  entry->accelerator = NULL;
  entry->callback = 0;
  entry->callback_action = 0;
  entry->item_type = "<LastBranch>";
  gtk_item_factory_create_item (factory, entry, NULL, 1);
  return factory;
}

GtkItemFactory *
create_menu_tearoff (GtkItemFactory * factory, gchar * path)
{
  GtkItemFactoryEntry *entry;
  entry = (GtkItemFactoryEntry *) g_malloc (sizeof (GtkItemFactoryEntry));
  entry->path = path;
  entry->accelerator = NULL;
  entry->callback = 0;
  entry->callback_action = 0;
  entry->item_type = "<Tearoff>";
  gtk_item_factory_create_item (factory, entry, NULL, 1);
  return factory;
}

GtkItemFactory *
create_menu_separator (GtkItemFactory * factory, gchar * path)
{
  GtkItemFactoryEntry *entry;
  entry = (GtkItemFactoryEntry *) g_malloc (sizeof (GtkItemFactoryEntry));
  entry->path = path;
  entry->accelerator = NULL;
  entry->callback = 0;
  entry->callback_action = 0;
  entry->item_type = "<Separator>";
  gtk_item_factory_create_item (factory, entry, NULL, 1);
  return factory;
}


GtkItemFactory *
create_menu_item (GtkItemFactory * factory, gchar * path, gchar * accelerator, GtkItemFactoryCallback callback, guint callback_action)
{
  GtkItemFactoryEntry *entry;
  entry = (GtkItemFactoryEntry *) g_malloc (sizeof (GtkItemFactoryEntry));
  entry->path = path;
  entry->accelerator = accelerator;
  entry->callback = callback;
  entry->callback_action = callback_action;
  entry->item_type = NULL;
  gtk_item_factory_create_item (factory, entry, NULL, 1);
  return factory;
}

gint
create_window_main ()
{
  GtkWidget      *main_window;
  GtkWidget      *vbox2;
  GtkWidget      *table;
  GtkWidget      *handlebox;

  GtkAccelGroup  *accel_group;

  GdkPixmap      *glade_pixmap;
  GdkBitmap      *glade_mask;
  GtkWidget      *pixmap1;

  guint           context_id;
  gint            database_ret_code;
  gchar          *status_msg;

  main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize (GTK_WIDGET (main_window), 320, 146);
  gtk_object_set_data (GTK_OBJECT (main_window), "main_window", main_window);
  gtk_widget_set_name (main_window, "main window");	/* For Styles !! */
  gtk_window_set_title (GTK_WINDOW (main_window), KRUNNING_INFO);
  gtk_window_set_policy (GTK_WINDOW (main_window), FALSE, FALSE, FALSE);
  gtk_signal_connect (GTK_OBJECT (main_window), "delete_event", GTK_SIGNAL_FUNC (file_quit_cmd_callback), NULL);
  gtk_widget_show (main_window);


  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (main_window), "vbox2", vbox2);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (main_window), vbox2);

  handlebox = gtk_handle_box_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "handlebox", handlebox);
  gtk_widget_show (handlebox);
  gtk_box_pack_start (GTK_BOX (vbox2), handlebox, TRUE, TRUE, 0);

  accel_group = gtk_accel_group_new ();
  factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
  factory = create_menu_branch (factory, _("/_File"));
  factory = create_menu_tearoff (factory, _("/File/tearoff1"));
  factory = create_menu_item (factory, _("/File/Connect"), "<control>C", gtk_ifactory_cb, 0);
  factory = create_menu_item (factory, _("/File/Open"), "<control>O", gtk_ifactory_cb, 1);
  factory = create_menu_item (factory, _("/File/Save"), "<control>S", gtk_ifactory_cb, 2);
  factory = create_menu_branch (factory, _("/File/Import"));
  factory = create_menu_item (factory, _("/File/Import/Times"), NULL, gtk_ifactory_cb, 21);
  factory = create_menu_separator (factory, _("/File/sep1"));
  factory = create_menu_branch (factory, _("/File/Clear"));
  factory = create_menu_item (factory, _("/File/Clear/Table runner"), NULL, gtk_ifactory_cb, 14);
  factory = create_menu_item (factory, _("/File/Clear/Table times"), NULL, gtk_ifactory_cb, 15);
  factory = create_menu_item (factory, _("/File/Clear/Table teams"), NULL, gtk_ifactory_cb, 16);
  factory = create_menu_item (factory, _("/File/Clear/Table running events"), NULL, gtk_ifactory_cb, 17);
  factory = create_menu_separator (factory, _("/File/Clear/sep1"));
  factory = create_menu_item (factory, _("/File/Clear/All Tables"), NULL, gtk_ifactory_cb, 18);

  factory = create_menu_separator (factory, _("/File/sep1"));
  factory = create_menu_item (factory, _("/File/Preferences..."), NULL, gtk_ifactory_cb, 3);
  factory = create_menu_item (factory, _("/File/Create Certification"), NULL, gtk_ifactory_cb, 13);
  factory = create_menu_separator (factory, _("/File/sep1"));
  factory = create_menu_item (factory, _("/File/Quit"), "<control>Q", gtk_ifactory_cb, 4);
  factory = create_menu_branch (factory, _("/_Edit"));
  factory = create_menu_tearoff (factory, _("/Edit/tearoff1"));
  factory = create_menu_item (factory, _("/Edit/Edit running events"), NULL, gtk_ifactory_cb, 5);
  factory = create_menu_item (factory, _("/Edit/Edit Runners"), NULL, gtk_ifactory_cb, 6);
  factory = create_menu_item (factory, _("/Edit/Edit Teams"), NULL, gtk_ifactory_cb, 7);
  factory = create_menu_separator (factory, _("/Edit/sep1"));
  factory = create_menu_item (factory, _("/Edit/Evaluate"), NULL, gtk_ifactory_cb, 11);
  factory = create_menu_separator (factory, _("/Edit/sep1"));
  factory = create_menu_item (factory, _("/Edit/Show runners"), NULL, gtk_ifactory_cb, 8);
  factory = create_menu_item (factory, _("/Edit/Show classteam-list"), NULL, gtk_ifactory_cb, 19);
  factory = create_menu_separator (factory, _("/Edit/sep1"));
  factory = create_menu_item (factory, _("/Edit/Print Certifcates"), NULL, gtk_ifactory_cb, 20);
  factory = create_menu_separator (factory, _("/Edit/sep1"));
  factory = create_menu_branch (factory, _("/_Times"));
  factory = create_menu_tearoff (factory, _("/Times/tearoff1"));
  factory = create_menu_item (factory, _("/Times/Count time"), NULL, gtk_ifactory_cb, 9);
  factory = create_menu_item (factory, _("/Times/Enter Position"), NULL, gtk_ifactory_cb, 10);
  factory = create_menu_lastbranch (factory, _("/_Help"));
  factory = create_menu_tearoff (factory, _("/Help/tearoff1"));
  factory = create_menu_item (factory, _("/Help/Index"), NULL, gtk_ifactory_cb, 14);
  factory = create_menu_separator (factory, _("/Help/sep1"));
  factory = create_menu_item (factory, _("/Help/About GTK"), NULL, gtk_ifactory_cb, 100);
  factory = create_menu_item (factory, _("/Help/About KRunning"), NULL, gtk_ifactory_cb, 12);
  /*gtk_item_factory_create_items (factory, nmenu_items, menu_items, NULL); */
  gtk_accel_group_attach (accel_group, GTK_OBJECT (main_window));

  gtk_container_add (GTK_CONTAINER (handlebox), gtk_item_factory_get_widget (factory, "<main>"));
  gtk_widget_show (gtk_item_factory_get_widget (factory, "<main>"));


  gtk_widget_realize (main_window);

  menus_set_sensitive (_("/Help/Index"), FALSE);
  menus_set_sensitive (_("/Help/About GTK"), FALSE);
  menus_set_sensitive (_("/File/Clear/All Tables"), FALSE);


  glade_pixmap = gdk_pixmap_create_from_xpm_d (main_window->window, &glade_mask, &main_window->style->bg[GTK_STATE_NORMAL], KRunning_xpm);

  pixmap1 = gtk_pixmap_new (glade_pixmap, glade_mask);
  gdk_pixmap_unref (glade_pixmap);
  gdk_bitmap_unref (glade_mask);
  gtk_object_set_data (GTK_OBJECT (main_window), "pixmap1", pixmap1);
  gtk_widget_show (pixmap1);
  gtk_box_pack_start (GTK_BOX (vbox2), pixmap1, TRUE, TRUE, 0);

  table = gtk_table_new (10, 1, FALSE);
  gtk_box_pack_start (GTK_BOX (vbox2), table, FALSE, TRUE, 2);

  statusbar = gtk_statusbar_new ();
  gtk_table_attach_defaults (GTK_TABLE (table), statusbar, 0, 9, 0, 1);
  context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), KRUNNING_STATUS_ID);
  gtk_object_set_data (GTK_OBJECT (main_window), "statusbar", statusbar);
  status_msg = g_strdup_printf (_("Welcome to KRunning Version %s"), KRUNNING_VERSION);
  gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, status_msg);
  g_free (status_msg);
  total_label = gtk_label_new ("00:00:00");
  gtk_table_attach_defaults (GTK_TABLE (table), total_label, 9, 10, 0, 1);
  gtk_widget_show (total_label);

  gtk_timeout_add (1000, print_time, NULL);
  gtk_widget_show (statusbar);
  // gtk_box_pack_start (GTK_BOX (vbox2), statusbar, TRUE, TRUE, 0);
  gtk_widget_show (table);

  /* To show KRunning before testing database tables */
  while (gtk_events_pending ()) {
    gtk_main_iteration ();
  }
  database_ret_code = test_database ();
  if (database_ret_code != 0 && cfg.connection_ok == TRUE) {
    create_question_dialog (DATABASE_ERROR_TEXT, database_ret_code);
  }

  if (cfg.connection_ok == TRUE) {
    menus_set_sensitive (_("/File/Connect"), FALSE);
  } else {
    /* disable any menuesettings without the preferences 
       and maybe a connect to database button */
    disable_menu_entrys ();
  }
  /* Init Pixmaps before starting ... */
  if (!cfg.show_connection_dialog)
    init_pixmaps (main_window);
  return 1;
}

void
disable_menu_entrys ()
{
  menus_set_sensitive (_("/File/Connect"), TRUE);
  menus_set_sensitive (_("/Edit"), FALSE);
  menus_set_sensitive (_("/Times"), FALSE);
  menus_set_sensitive (_("/File/Open"), FALSE);
  menus_set_sensitive (_("/File/Save"), FALSE);
  menus_set_sensitive (_("/File/Clear"), FALSE);
  menus_set_sensitive (_("/File/Import"), FALSE);
}

gint
print_time (gpointer data)
{
  gchar          *time_string;
  time_ctrl       time;
  time = gettime (time);
  time_string = g_strdup_printf ("%02d:%02d:%02d", time.currenttime[0], time.currenttime[1], time.currenttime[2]);
  gtk_label_set (GTK_LABEL (total_label), time_string);
  g_free (time_string);
  return 1;			/*We have to return 1 because otherwise the counter breaks .. */
}

void
menus_set_sensitive (char *path, int sensitive)
{
  GtkWidget      *menu_path;
  menu_path = gtk_item_factory_get_widget (factory, path);
  if (menu_path)
    gtk_widget_set_sensitive (menu_path, sensitive);
  else
    g_warning ("Unable to set sensitivity for menu which doesn't exist: %s", path);
}

void
menu_separator (GtkWidget * menu)
{
  GtkWidget      *menu_item = gtk_menu_item_new ();
  gtk_menu_append (GTK_MENU (menu), menu_item);
  gtk_widget_set_sensitive (menu_item, FALSE);
  gtk_widget_show (menu_item);
}
