/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "position.h"
#include "krunenv.h"
#include "menubar.h"
#include "krunning_time.h"
#include "classteamprint.h"
#include "database_modules.h"
#define POSITION_CLIST_COLUMNS 7
#define NO_TIME_CLIST_COLUMNS 4
#define WINDOW_WIDTH 640
#define WINDOW_HEIGHT 480

static guint    handler_id = -1;
static gint     menu_popup (GtkWidget * widget, GdkEvent * event);
static void     pos_menuitem_response (GtkWidget * widget, gpointer number);
static void     menuitem_response (gpointer number);
static gint     number_compare_func (gconstpointer a, gconstpointer b);
GList          *remove_runner (GList * list, gint number);
void            delete_time (void);
void            change_time (void);
void            undo_last (void);

GtkWidget      *position_dialog;
GtkWidget      *time_clist_0;
GtkWidget      *time_clist_1;
GtkWidget      *number_entry_;
GtkWidget      *time_entry;
static GtkWidget *message_label;

gint            time_clist_1_rows = 0;
gint            time_number = -1;
static gint     run_number;
gint            selected_row = -1;

enum
{
  TARGET_STRING,
  TARGET_ROOTWIN,
  TARGET_URL
};

static GtkTargetEntry target_table[] = {
  {"STRING", 0, TARGET_STRING},
  {"text/plain", 0, TARGET_STRING},
  {"text/uri-list", 0, TARGET_URL},
  {"application/x-rootwin-drop", 0, TARGET_ROOTWIN}
};

/*
   static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);
 */

void
select_timelist (GtkWidget * widget, gint row, gint column, GdkEventButton * bevent)
{
  gchar          *text;
#ifdef DEBUG
  g_print ("GtkCList Selection: row %d column %d button %d\n", row, column, bevent ? bevent->button : 0);
#endif
  gtk_clist_get_text (GTK_CLIST (widget), row, 0, &text);
  time_number = atoi (text);
  gtk_clist_get_text (GTK_CLIST (widget), row, 2, &text);
  gtk_entry_set_text (GTK_ENTRY (time_entry), (char *) text);
  gtk_widget_grab_focus (GTK_WIDGET (number_entry_));
  selected_row = row;
}

void
unselect_timelist (GtkWidget * widget, gint row, gint column, GdkEventButton * bevent)
{
  gchar          *text;
#ifdef DEBUG
  g_print ("GtkCList Unselection: row %d column %d button %d\n", row, column, bevent ? bevent->button : 0);
#endif
  gtk_clist_get_text (GTK_CLIST (widget), row, 0, &text);
  gtk_entry_set_text (GTK_ENTRY (time_entry), "");
  time_number = -1;
  selected_row = -1;
}

/* DnD Functions */
/*
   void
   target_drag_leave (GtkWidget * widget,
   GdkDragContext * context,
   guint time)
   {
   g_message ("Target drag leave called");
   }

   gboolean
   target_drag_motion (GtkWidget * widget,
   GdkDragContext * context,
   gint x,
   gint y,
   guint time)
   {
   g_message ("Target drag motion called");
   return TRUE;
   }

   gboolean
   target_drag_drop (GtkWidget * widget,
   GdkDragContext * context,
   gint x,
   gint y,
   guint time)
   {
   g_message ("Target drag drop called");
   return TRUE;
   }

   void
   target_drag_data_received (GtkWidget * widget,
   GdkDragContext * context,
   gint x,
   gint y,
   GtkSelectionData * data,
   guint info,
   guint time)
   {
   g_message ("Target_drag_data_received called");
   }
 */
void
create_position_dialog ()
{
  GtkWidget      *dialog_vbox1;
  GtkWidget      *outer_vbox;

  GtkWidget      *event_menue;
  GtkWidget      *hpaned;
  GtkWidget      *frame3;
  GtkWidget      *hbox3;
  GtkWidget      *vbox2;
  GtkWidget      *hbox4;
  GtkWidget      *text_label_;
  GtkWidget      *time_label;
  GtkWidget      *label9;
  GtkWidget      *scrolled_window;
  GtkWidget      *scrolled_window_2;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox;
  GtkWidget      *ok_button_;
  GtkWidget      *table;

  /* Pop-up Menu */
  GtkWidget      *menu;
  GtkWidget      *menu_items;

  GtkStyle       *style;

  gint            a;

  static gchar   *titles[] = {
    N_("InternalNumber"),
    N_("Nr."),
    N_("Time"),
    N_("Startnumber"),
    N_("Name"),
    N_("Vorname"),
    N_("Class")
  };

  static gchar   *titles_2[] = {
    N_("InternalNumber"),
    N_("Startnumber"),
    N_("Name"),
    N_("Vorname")
  };

  position_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (position_dialog), "position_dialog", position_dialog);
  GTK_WINDOW (position_dialog)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (position_dialog), _("Enter Position"));
  gtk_window_position (GTK_WINDOW (position_dialog), GTK_WIN_POS_CENTER);
  gtk_widget_set_usize (position_dialog, WINDOW_WIDTH, WINDOW_HEIGHT);
  /* gtk_window_set_policy (GTK_WINDOW (position_dialog), TRUE, TRUE, FALSE); */

  dialog_vbox1 = GTK_DIALOG (position_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (position_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  outer_vbox = gtk_vbox_new (FALSE, 5);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "outer_vbox", outer_vbox);
  gtk_widget_show (outer_vbox);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), outer_vbox, TRUE, TRUE, 0);

  event_menue = create_event_menu ((void (*)(GtkWidget *, gpointer)) pos_menuitem_response, FALSE);

  gtk_box_pack_start (GTK_BOX (outer_vbox), event_menue, FALSE, TRUE, 0);
  gtk_widget_show (event_menue);

  frame3 = gtk_frame_new (NULL);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "frame3", frame3);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (outer_vbox), frame3, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (frame3), 5);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "hbox3", hbox3);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (frame3), hbox3);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "vbox2", vbox2);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (hbox3), vbox2, FALSE, TRUE, 0);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "hbox4", hbox4);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, TRUE, TRUE, 0);

  time_label = gtk_label_new (_("Time :"));
  gtk_object_set_data (GTK_OBJECT (position_dialog), "time_label", time_label);
  gtk_label_set_justify (GTK_LABEL (time_label), GTK_JUSTIFY_LEFT);
  gtk_widget_show (time_label);
  gtk_box_pack_start (GTK_BOX (hbox4), time_label, TRUE, TRUE, 0);

  time_entry = gtk_entry_new_with_max_length (8);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "time_entry", time_entry);
  gtk_widget_show (time_entry);
  gtk_entry_set_editable (GTK_ENTRY (time_entry), FALSE);
  gtk_box_pack_start (GTK_BOX (hbox4), time_entry, FALSE, TRUE, 5);
  gtk_widget_set_usize (time_entry, 58, 20);


  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "hbox4", hbox4);
  gtk_widget_show (hbox4);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox4, TRUE, TRUE, 0);

  text_label_ = gtk_label_new (_("Number : "));
  gtk_object_set_data (GTK_OBJECT (position_dialog), "text_label_", text_label_);
  gtk_widget_show (text_label_);
  gtk_box_pack_start (GTK_BOX (hbox4), text_label_, TRUE, TRUE, 0);
  gtk_misc_set_padding (GTK_MISC (text_label_), 4, 0);

  number_entry_ = gtk_entry_new_with_max_length (5);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "number_entry_", number_entry_);
  gtk_widget_show (number_entry_);
  gtk_signal_connect (GTK_OBJECT (number_entry_), "activate", GTK_SIGNAL_FUNC (number_entry_callback), NULL);
  gtk_box_pack_start (GTK_BOX (hbox4), number_entry_, FALSE, TRUE, 5);
  gtk_widget_set_usize (number_entry_, 40, 20);

  label9 = gtk_label_new (_("oO0(" ")0Oo"));
  gtk_object_set_data (GTK_OBJECT (position_dialog), "label9", label9);
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (vbox2), label9, TRUE, TRUE, 0);


  hpaned = gtk_hpaned_new ();
  gtk_object_set_data (GTK_OBJECT (position_dialog), "hpaned", hpaned);
  gtk_widget_show (hpaned);

  gtk_box_pack_start (GTK_BOX (hbox3), hpaned, TRUE, TRUE, 0);
  gtk_paned_gutter_size (GTK_PANED (hpaned), 10);


  scrolled_window_2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window_2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  /* Resize is ok but we could not shrink the drawingarea */
  gtk_paned_pack1 (GTK_PANED (hpaned), scrolled_window_2, TRUE, FALSE);
  /*gtk_box_pack_start (GTK_BOX (hbox3), scrolled_window_2, TRUE, TRUE, 0); */
  /* gtk_widget_set_usize (scrolled_window_2, WINDOW_WIDTH - 20, WINDOW_HEIGHT - 70); */
  gtk_widget_show (scrolled_window_2);



  time_clist_0 = gtk_clist_new_with_titles (NO_TIME_CLIST_COLUMNS, titles_2);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "time_clist_0", time_clist_0);
  gtk_widget_show (time_clist_0);

  /*
     gtk_signal_connect (GTK_OBJECT (time_clist_0),
     "select_row",
     (GtkSignalFunc) select_timelist,
     NULL);
     gtk_signal_connect (GTK_OBJECT (time_clist_0),
     "unselect_row",
     (GtkSignalFunc) unselect_timelist,
     NULL);
   */

  gtk_clist_set_column_width (GTK_CLIST (time_clist_0), 0, 80);
  gtk_clist_set_column_auto_resize (GTK_CLIST (time_clist_0), 1, TRUE);
  gtk_clist_set_column_auto_resize (GTK_CLIST (time_clist_0), 2, TRUE);
  gtk_clist_set_column_auto_resize (GTK_CLIST (time_clist_0), 3, TRUE);
  gtk_clist_column_titles_show (GTK_CLIST (time_clist_0));

  gtk_clist_set_column_visibility (GTK_CLIST (time_clist_0), 0, FALSE);

  gtk_clist_set_shadow_type (GTK_CLIST (time_clist_0), GTK_SHADOW_IN);
  gtk_container_add (GTK_CONTAINER (scrolled_window_2), time_clist_0);

  /* For my mouse */
  gtk_widget_realize (time_clist_0);
  /*  
     gtk_signal_connect_object (GTK_OBJECT (time_clist_0), "event", GTK_SIGNAL_FUNC (menu_popup), GTK_OBJECT (menu));
   */
  gdk_window_set_cursor (time_clist_0->window, gdk_cursor_new (GDK_HAND1));


  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_paned_add2 (GTK_PANED (hpaned), scrolled_window);
  /*gtk_box_pack_start (GTK_BOX (hbox3), scrolled_window, TRUE, TRUE, 0); */
  /* gtk_widget_set_usize (scrolled_window, WINDOW_WIDTH - 20, WINDOW_HEIGHT - 70); */
  gtk_widget_show (scrolled_window);

  menu = gtk_menu_new ();
  menu_items = gtk_menu_item_new_with_label (_("Undo"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "Undo", menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response), (gpointer) UNDO_LAST_DOING);
  gtk_widget_show (menu_items);
  gtk_widget_set_sensitive (menu_items, FALSE);
  menu_separator (menu);
  menu_items = gtk_menu_item_new_with_label (_("Delete Time"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response), (gpointer) DELETE_TIME);
  gtk_widget_show (menu_items);
  menu_items = gtk_menu_item_new_with_label (_("Change Time"));
  gtk_menu_append (GTK_MENU (menu), menu_items);
  gtk_signal_connect_object (GTK_OBJECT (menu_items), "activate", GTK_SIGNAL_FUNC (menuitem_response), (gpointer) CHANGE_TIME);
  gtk_widget_show (menu_items);
  menu_separator (menu);

  time_clist_1 = gtk_clist_new_with_titles (POSITION_CLIST_COLUMNS, titles);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "time_clist_1", time_clist_1);
  gtk_widget_show (time_clist_1);
  gtk_signal_connect (GTK_OBJECT (time_clist_1), "select_row", (GtkSignalFunc) select_timelist, NULL);
  gtk_signal_connect (GTK_OBJECT (time_clist_1), "unselect_row", (GtkSignalFunc) unselect_timelist, NULL);

  gtk_clist_set_column_width (GTK_CLIST (time_clist_1), 0, 80);
  for (a = 1; a <= 6; a++) {
    gtk_clist_set_column_auto_resize (GTK_CLIST (time_clist_1), a, TRUE);
  }
  gtk_clist_column_titles_show (GTK_CLIST (time_clist_1));

  gtk_clist_set_column_visibility (GTK_CLIST (time_clist_1), 0, FALSE);

  gtk_clist_set_shadow_type (GTK_CLIST (time_clist_1), GTK_SHADOW_IN);

  gtk_container_add (GTK_CONTAINER (scrolled_window), time_clist_1);

  /* For my mouse */
  gtk_widget_realize (time_clist_1);
  gtk_signal_connect_object (GTK_OBJECT (time_clist_1), "event", GTK_SIGNAL_FUNC (menu_popup), GTK_OBJECT (menu));

  gdk_window_set_cursor (time_clist_1->window, gdk_cursor_new (GDK_HAND1));


  dialog_action_area1 = GTK_DIALOG (position_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (position_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  table = gtk_table_new (1, 2, FALSE);
  gtk_object_set_data (GTK_OBJECT (position_dialog), "table", table);
  gtk_widget_show (table);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), table, TRUE, TRUE, 0);

  message_label = gtk_label_new (_("You can now change some times ..."));
  gtk_object_set_data (GTK_OBJECT (position_dialog), "message_label", message_label);
  gtk_widget_show (message_label);
  gtk_misc_set_alignment (GTK_MISC (message_label), 0, 0.5);
  gtk_table_attach (GTK_TABLE (table), message_label, 0, 1, 0, 1, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

  style = gtk_style_new ();
  style->fg[GTK_STATE_NORMAL] = LRED;
  gtk_widget_set_style (message_label, style);


  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (position_dialog), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_table_attach (GTK_TABLE (table), hbuttonbox, 1, 2, 0, 1, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_container_border_width (GTK_CONTAINER (hbuttonbox), 2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 10);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox), 2, 0);

  ok_button_ = gtk_button_new_with_label (_("Close"));
  gtk_object_set_data (GTK_OBJECT (position_dialog), "ok_button_", ok_button_);
  gtk_widget_show (ok_button_);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ok_button_);
  gtk_signal_connect_object (GTK_OBJECT (ok_button_), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (position_dialog));

  gtk_widget_show (position_dialog);
  pos_menuitem_response (NULL, GINT_TO_POINTER (KRunning_event_number.event_number));
}

static void
pos_menuitem_response (GtkWidget * widget, gpointer number)
{
  gint            a, visible_row = -1;
  gint            time[3];
  /* For the clist with the runners without a time */
  gchar           text_0[NO_TIME_CLIST_COLUMNS][50];
  gchar          *texts_0[NO_TIME_CLIST_COLUMNS];
  /* For the clist that contains the times */
  gchar           text_1[POSITION_CLIST_COLUMNS][50];
  gchar          *texts_1[POSITION_CLIST_COLUMNS];

  GList          *runnerlist = NULL;
  GList          *firstptr = NULL;	/* Points the first list element */
  GList          *runner = NULL;
  KRunnerNames   *runnerptr = NULL;

  guint32         seconds = 0;
  time_clist_1_rows = 0;

  gtk_clist_clear (GTK_CLIST (time_clist_0));
  gtk_clist_clear (GTK_CLIST (time_clist_1));

  run_number = GPOINTER_TO_INT (number);

  for (a = 0; a <= NO_TIME_CLIST_COLUMNS; a++) {
    texts_0[a] = text_0[a];
  }

  for (a = 0; a <= POSITION_CLIST_COLUMNS; a++) {
    texts_1[a] = text_1[a];
  }

  res = execSQL ("select runner.number,name,vorname,geb_jahr from runner where lauf_nr = %d and runner.number > 0", number);

  for (a = 0; a < PQntuples (res); a++) {
    runnerptr = create_runner (atoi (PQgetvalue (res, a, 0)), PQgetvalue (res, a, 1), PQgetvalue (res, a, 2), atoi (PQgetvalue (res, a, 3)), 0);
    runnerlist = g_list_insert_sorted (runnerlist, runnerptr, number_compare_func);
    if (a == 0)
      firstptr = runnerlist;
  }
  PQclear (res);

  res = execSQL ("select number,result_seconds,runner_number from times where run_number = %d order by result_seconds ", number);
  gtk_clist_freeze (GTK_CLIST (time_clist_1));
  for (a = 0; a < PQntuples (res); a++) {
    sprintf (text_1[0], "%d", atoi (PQgetvalue (res, a, 0)));
    sprintf (text_1[1], "%03d", a + 1);
    seconds = atoi (PQgetvalue (res, a, 1));
    time[0] = seconds / 3600L;
    seconds -= time[0] * 3600L;
    time[1] = seconds / 60L;
    time[2] = seconds - time[1] * 60L;
    sprintf (text_1[2], "%02d:%02d:%02d", time[0], time[1], time[2]);
    sprintf (text_1[3], "%s", PQgetvalue (res, a, 2));
    runner = runner_find (runnerlist, atoi (PQgetvalue (res, a, 2)));
    if (runner) {
      g_snprintf (text_1[4], 24, "%s", ((KRunnerNames *) runner->data)->last_name);
      g_snprintf (text_1[5], 24, "%s", ((KRunnerNames *) runner->data)->first_name);
      g_snprintf (text_1[6], 4, "%s", show_klasse (((KRunnerNames *) runner->data)->birthday, 0));
    } else {
      if (visible_row == -1)
	visible_row = a;
      g_snprintf (text_1[4], 24, "----------");
      g_snprintf (text_1[5], 24, "----------");
      g_snprintf (text_1[6], 24, "---");
    }

    gtk_clist_insert (GTK_CLIST (time_clist_1), a, texts_1);
    time_clist_1_rows++;
  }
  PQclear (res);

  gtk_clist_select_row (GTK_CLIST (time_clist_1), visible_row, 0);
  if (gtk_clist_row_is_visible (GTK_CLIST (time_clist_1), visible_row) != GTK_VISIBILITY_FULL) {
    /*gtk_widget_grab_focus (time_clist_1); */
    gtk_clist_moveto (GTK_CLIST (time_clist_1), visible_row, 1, 1.0, 1.0);
    GTK_CLIST (time_clist_1)->focus_row = visible_row;
  }
  gtk_clist_thaw (GTK_CLIST (time_clist_1));

  /* And remove the runners which have already a time in the timelist */

  res = execSQL ("select runner.number from runner, times where run_number = %d  and runner.number = times.runner_number and result_seconds > 0 ", run_number);
  for (a = 0; a < PQntuples (res); a++) {

#ifdef DEBUG
    printf ("Removing = %d\n", atoi (PQgetvalue (res, a, 0)));
#endif
    runnerlist = g_list_first (runnerlist);
    /*runnerlist = firstptr;   runnerlist -> First (List) */

    runnerlist = remove_runner (runnerlist, atoi (PQgetvalue (res, a, 0)));

  }
  PQclear (res);

  runnerlist = g_list_first (runnerlist);
  gtk_clist_freeze (GTK_CLIST (time_clist_0));
  a = 0;

  while (runnerlist) {		/* runnerlist */
    a++;
    sprintf (text_0[0], "%d", a);
    sprintf (text_0[1], "%d", ((KRunnerNames *) runnerlist->data)->number);
    sprintf (text_0[2], "%s", ((KRunnerNames *) runnerlist->data)->last_name);
    sprintf (text_0[3], "%s", ((KRunnerNames *) runnerlist->data)->first_name);
    gtk_clist_insert (GTK_CLIST (time_clist_0), a, texts_0);
    runnerlist = g_list_next (runnerlist);
  }

  gtk_clist_thaw (GTK_CLIST (time_clist_0));
  runnerlist = firstptr;	/* runnerlist -> First (List) */

  if (runnerlist) {
    g_list_foreach (runnerlist, (GFunc) clear_runner, runnerlist->data);
    /*g_list_free (runnerlist); BUG */
  }
  if (runner)
    g_list_free_1 (runner);

}


void
number_entry_callback (GtkWidget * widget, gpointer data)
{
/* Hier stimmt was nicht, es stehen immer 2 Lufer nach dem update in der Liste */
#ifdef DEBUG
  g_message ("Time number %d!\n", time_number);
#endif
  /* Check if in the database is already a runner stored with the same number */
  res = execSQL ("select runner_number from times where runner_number = %d", atoi (gtk_entry_get_text (GTK_ENTRY (number_entry_))));
  if (PQntuples (res) != 0) {
    ndisplay_text (message_label, _("You have already given this runner a time !!"), DISPLAY_TIME);
    PQclear (res);
    return;
  } else
    PQclear (res);
  /* Exist this runner ??? */
  res = execSQL ("select number from runner where number = %d and lauf_nr = %d", atoi (gtk_entry_get_text (GTK_ENTRY (number_entry_))), run_number);
  if (PQntuples (res) != 1) {
    ndisplay_text (message_label, _("You have entered a number which didnt exists"), DISPLAY_TIME);
    PQclear (res);
    return;
  } else
    PQclear (res);

  res = execSQL ("update times set runner_number = '%d' where number = %d " "and run_number = %d", atoi (gtk_entry_get_text (GTK_ENTRY (number_entry_))), time_number, run_number);
  PQclear (res);
  gtk_entry_set_text (GTK_ENTRY (number_entry_), "");
  /* We now have to update the clist !!! */
  pos_menuitem_response (NULL, GINT_TO_POINTER (run_number));
  return;
}

static          gint
menu_popup (GtkWidget * widget, GdkEvent * event)
{
  if (event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bevent = (GdkEventButton *) event;
    if (bevent->button == 3) {
      gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL, bevent->button, bevent->time);
      return TRUE;
    }
  }
  return FALSE;
}

static void
menuitem_response (gpointer number)
{
  switch (GPOINTER_TO_INT (number)) {
  case DELETE_TIME:
    delete_time ();
    break;
  case CHANGE_TIME:
    change_time ();
    break;
  case UNDO_LAST_DOING:
    undo_last ();
    break;
  default:
  }
}


void
undo_last ()
{
  /*
     We should keep all doings in a clist and roll them back .. 
   */ pos_menuitem_response (NULL, GINT_TO_POINTER (run_number));
  gtk_widget_set_sensitive (get_widget (position_dialog, "Undo"), FALSE);
}

void
delete_time ()
{
  gchar          *text = NULL;
  if (selected_row != -1) {
    gtk_clist_get_text (GTK_CLIST (time_clist_1), selected_row, 0, &text);
    if (text != NULL || time_number < 0) {
      time_number = atoi (text);
      res = execSQL ("delete from times where number = %d and run_number = %d", time_number, run_number);
      PQclear (res);
      gtk_clist_remove (GTK_CLIST (time_clist_1), selected_row);
      pos_menuitem_response (NULL, GINT_TO_POINTER (run_number));
      gtk_entry_set_text (GTK_ENTRY (time_entry), "");
    }
    gtk_widget_set_sensitive (get_widget (position_dialog, "Undo"), TRUE);
  }
}


void
timechange_entry_callback (GtkWidget * widget, gpointer data)
{
  gchar         **time;
  guint32         seconds;
  if (strlen (gtk_entry_get_text (GTK_ENTRY (get_widget (position_dialog, "time_entry")))) != 8) {
    ndisplay_text (message_label, _("Wrong Time Format !"), DISPLAY_TIME);
    gtk_widget_grab_focus (GTK_WIDGET (get_widget (position_dialog, "time_entry")));
    return;
  } else {
    time = g_strsplit (gtk_entry_get_text (GTK_ENTRY (get_widget (position_dialog, "time_entry"))), ":", 0);
    seconds = atoi (time[0]) * 3600L;
    seconds += atoi (time[1]) * 60L;
    seconds += atoi (time[2]);
    g_strfreev (time);		/* This should free timestring */
  }
  res = execSQL ("update times set result_seconds = '%d' where number = %d " "and run_number = %d", seconds, time_number, run_number);
  PQclear (res);
  gtk_entry_set_editable (GTK_ENTRY (get_widget (position_dialog, "time_entry")), FALSE);
  gtk_signal_disconnect (GTK_OBJECT (get_widget (position_dialog, "time_entry")), handler_id);
  gtk_label_set (GTK_LABEL (get_widget (position_dialog, "time_label")), _("Time :"));
  /* We now have to update the clist !!! */

  pos_menuitem_response (NULL, GINT_TO_POINTER (run_number));
}


void
change_time ()
{
  gtk_label_set (GTK_LABEL (get_widget (position_dialog, "time_label")), _("New Time :"));
  gtk_entry_set_editable (GTK_ENTRY (get_widget (position_dialog, "time_entry")), TRUE);
  gtk_widget_grab_focus (GTK_WIDGET (get_widget (position_dialog, "time_entry")));
  handler_id = gtk_signal_connect (GTK_OBJECT (get_widget (position_dialog, "time_entry")), "activate", GTK_SIGNAL_FUNC (timechange_entry_callback), NULL);
}

GList          *
remove_runner (GList * list, gint number)
{
  GList          *tmp;
  tmp = list;
  while (tmp) {
    if (((KRunnerNames *) tmp->data)->number != number)
      tmp = tmp->next;
    else {
      if (tmp->prev)
	tmp->prev->next = tmp->next;
      if (tmp->next)
	tmp->next->prev = tmp->prev;
      if (list == tmp)
	list = list->next;
      break;
    }
  }
  return list;
}

static          gint
number_compare_func (gconstpointer a, gconstpointer b)
{
  guint32         one = ((KRunnerNames *) a)->number;
  guint32         two = ((KRunnerNames *) b)->number;
  if (one < two)
    return -1;
  else
    return 1;
}
