/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef PREFERENCES_H
#define PREFERENCES_H
#include "krunning_time.h"
#include "printing.h"
#include "configfile.h"

#define CONFIG_DIR "/.krunning"
#define CONFIG_FILE "/.krunning/config"

#define DATA_PERM S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH	//0666
#define RC_PERM S_IRUSR | S_IWUSR	//0600
#define RC_FOLDER_PERM S_IRWXU	//0700

typedef struct _Config Config;
typedef struct _Certification Certification;
typedef struct _Certification_entry Certification_entry;

struct _Config
{
  gchar          *database_name;
  gchar          *database_host;
  gchar          *database_login;
  gchar          *database_password;
  gchar          *database_port;
  gchar          *database_list;	/* A list of all databases which could connected */
  gchar          *local_dir;
  gint            print_mode;	/* Has currently no function */
  gboolean        connection_ok;	/* 0 = BAD , 1= OK */
  gchar          *main_event_name;
  gchar          *sub_event_name;
  gchar          *event_number;	/* Maybee 12. or XII. Running across a river */
  gchar          *event_date;
  gboolean        print_event_date;	/* if TRUE it prints the event date on certificates */
  gchar          *organisation_name;
  gchar          *location_name;
  gboolean        show_times;
  gint            sort_column_teams;
  gboolean        gray_header;
  gboolean        print_box;
  gboolean        show_connection_dialog;
  gboolean        print_team_separator_lines;
  /* Save file Table color 
   * Format: #rrggbb
   */
  gchar          *html_table_color;
  gchar          *html_background_color;
  /* here are coming the names of all following 
     certificates .. */
  gchar          *actual_certificate;	/* The certification which we print .. */
  gchar          *certificates;
};

#define CERTIFICATION_ENTRYS 6
struct _Certification_entry
{
/* Name of the config file header -> certificates entry */
  gchar          *value_name;
  gint            value_y;
  gint            align;
  gchar          *string;
  guint           font_size;
  gboolean        print;
};

struct _Certification
{
/* Here we store the absolute positions from each element 
 * on the Certification 
 */
  gchar          *certification_name;
  gint            paper_size;
  gint            size;
  Certification_entry *entry[10];
};

gint            load_certification (ConfigFile * cfgfile, gchar * name);
gint            save_certification (ConfigFile * cfgfile);
extern Config   cfg;
extern Certification cert;
extern guint    n_cert_values;
extern gchar   *cert_value_names[];
void            init_config ();
void            save_config (void);
void            create_preferences_dialog (void);
void            create_certification (gchar * cert_name);
#endif /* PREFERENCES_H */
