/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <unistd.h>		/*   For the pipes !!      */

#include <string.h>
#include "printing.h"
#include "krunenv.h"
#include "preferences.h"
#include "io.h"
#include <time.h>
#include <fcntl.h>
#include "krunning_time.h"
#include "database_modules.h"

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/utsname.h>

#define CM 28.346456

struct utsname  sysInfo;
static GtkWidget *print_dialog;
static GtkWidget *fs;
GtkWidget      *checkbutton_file;
static gchar   *cmw_file (GtkWidget * widget, PrintObject * p_obj);

PrintObject    *print_object = NULL;
PrintStruct    *PrintingDatas = NULL;

static GtkWidget *combo;
static GtkWidget *entry;
static void     print_ok (GtkWidget * widget, PrintObject * p_obj);
static void     printPS_EOF (FILE * fd);
static void     printPS_funcs (FILE * fd);
static void     printPS_showpage (FILE * fd);
static void     print_box (FILE * fd);
static void     page_sum (gint lines);
gint            translateCharacter (guchar c, FILE * fd);
void            setSize (PaperFormatNumber size);
void            printPageHeader (FILE * fd);
static void     printPS_Header (FILE * fd);
void            printPageFooter (FILE * fd);
gchar          *insertNewLines (gchar * str);

PaperSize       paper_sizes[] = {
  {"DIN A3", 1, 842, 1190},
  {"DIN A4", 2, 595, 839},
  {"DIN A5", 3, 420, 595},
  {"LETTER", 4, 612, 792},
  {"LEGAL", 5, 612, 1008}
};
guint           paper_sizes_n = sizeof (paper_sizes) / sizeof (paper_sizes[0]);

gchar          *
get_paper_format (gint paper_number)
{
  gint            i;
  for (i = 0; i <= paper_sizes_n; i++) {
    if (paper_number == paper_sizes[i].number)
      return (paper_sizes[i].format_name);
  }
  return NULL;
}

gint
get_paper_number (gchar * paper_format)
{
  gint            i;
  for (i = 0; i <= paper_sizes_n; i++) {
    if (g_strcasecmp (paper_format, paper_sizes[i].format_name) == 0)
      return (i + 1);
  }
  return -1;
}


gchar          *
get_paper_x_size (gint paper_number)
{
  gint            i;
  gchar          *x_size;
  x_size = (gchar *) g_malloc (5);
  for (i = 0; i <= paper_sizes_n; i++) {
    if (paper_number == paper_sizes[i].number) {
      g_snprintf (x_size, 5, "%d", paper_sizes[i].size_x);
      return x_size;
    }

  }
  return NULL;
}

gchar          *
get_paper_y_size (gint paper_number)
{
  gint            i;
  gchar          *y_size;
  y_size = (gchar *) g_malloc (5);
  for (i = 0; i <= paper_sizes_n; i++) {
    if (paper_number == paper_sizes[i].number) {
      g_snprintf (y_size, 5, "%d", paper_sizes[i].size_y);
      return y_size;
    }
  }
  return NULL;
}

void
destroy_global_print_object (void)
{
/* We have to wait until the window is destroyed !!!! */
  if (print_object) {
    g_free (print_object);
    print_object = NULL;
  }
}

void
print_clist (GtkWidget * widget, PrintObject * p_obj)
{
  GtkWidget      *vbox2;
  GList          *combo_items = NULL;
  GtkWidget      *label;
  GtkWidget      *print_button;
  GtkWidget      *save_button;
  GtkWidget      *hbuttonbox;
  GtkWidget      *dialog_action_area1;
  FILE           *filehandler;
  gint            i;
  static char     line[513];
  static char     temp[1024];

  /*
   * Here we set the default file format to Postscript because if we
   * save we can overwrite it ... later 
   */

  p_obj->file_format = POSTSCRIPT;
  p_obj->save_file = FALSE;

  /*
   * Printer Options 
   */

  combo_items = g_list_append (combo_items, "default");
  filehandler = fopen ("/etc/printcap", "r");

  if (filehandler != NULL) {
    while (fgets (line, 255, filehandler) != NULL) {
      if (line != NULL && line[0] != '#' && line[0] != ' ' && line[0] != '\t') {
	bzero (temp, 512);
	for (i = 0; line[i] != '|'; i++) {
	  if (line[i] == '\0')
	    break;
	  if (line[i] == ':')
	    break;
	  if (line[i] == ' ')
	    break;
	  temp[i] = line[i];
	}
	combo_items = g_list_append (combo_items, g_strdup (temp));
      }
      bzero (line, 513);
    }
    fclose (filehandler);
  }
  print_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (print_dialog), "print_dialog", print_dialog);
  GTK_WINDOW (print_dialog)->type = GTK_WINDOW_DIALOG;
  gtk_container_border_width (GTK_CONTAINER (print_dialog), BORDER_WIDTH);
  gtk_window_set_title (GTK_WINDOW (print_dialog), _("Print"));
  gtk_window_position (GTK_WINDOW (print_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (print_dialog), FALSE, FALSE, FALSE);

  vbox2 = GTK_DIALOG (print_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (print_dialog), "vbox2", vbox2);
  gtk_widget_show (vbox2);

  combo = gtk_combo_new ();
  gtk_object_set_data (GTK_OBJECT (print_dialog), "combo", combo);
  gtk_combo_set_popdown_strings (GTK_COMBO (combo), combo_items);
  gtk_widget_show (combo);
  gtk_box_pack_start (GTK_BOX (vbox2), combo, FALSE, TRUE, 0);
  combo_items = g_list_append (combo_items, "default");

  g_list_free (combo_items);

  label = gtk_label_new (_("Print command and optional flags:"));
  gtk_object_set_data (GTK_OBJECT (print_dialog), "label", label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, FALSE, 0);

  entry = gtk_entry_new_with_max_length (100);
  gtk_object_set_data (GTK_OBJECT (print_dialog), "entry", entry);
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (vbox2), entry, FALSE, TRUE, 0);
  gtk_entry_set_text (GTK_ENTRY (entry), "lpr");

  dialog_action_area1 = GTK_DIALOG (print_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (print_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 5);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (print_dialog), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbuttonbox), 2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_EDGE);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 10);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox), 2, 0);

  print_button = gtk_button_new_with_label (_("Print"));
  gtk_object_set_data (GTK_OBJECT (print_dialog), "print_button", print_button);

  gtk_signal_connect (GTK_OBJECT (print_button), "clicked", (GtkSignalFunc) print_ok, p_obj);

  gtk_widget_show (print_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), print_button);
  gtk_widget_grab_focus (print_button);
  /*
   * gtk_widget_grab_default (print_button); 
   */

  save_button = gtk_button_new_with_label (_("Save as ..."));
  gtk_object_set_data (GTK_OBJECT (print_dialog), "save_button", save_button);
  gtk_signal_connect (GTK_OBJECT (save_button), "clicked", GTK_SIGNAL_FUNC (cmw_file), p_obj);
  gtk_widget_show (save_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), save_button);

  gtk_widget_show (print_dialog);
}



static          gboolean
cmw_destroy_cb (GtkWidget * widget)
{
  /*
   * This is needed to get out of gtk_main 
   */
  gtk_main_quit ();

  return FALSE;
}

static void
fileformat_response (GtkWidget * widget, gpointer data)
{
/*
 * We have two file formats in which we can save files  * At first
 * Postscript and the HTML format 
 */
  switch (GPOINTER_TO_INT (data)) {
  case HTML:
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".html");
    print_object->file_format = HTML;
    break;
  case POSTSCRIPT:
    gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".ps");
    print_object->file_format = POSTSCRIPT;
    break;
  default:
  }
}

static gchar   *
cmw_file (GtkWidget * widget, PrintObject * p_obj)
{
  GtkWidget      *optionmenu;
  GtkWidget      *optionmenu_menu;
  GtkWidget      *glade_menuitem;
  GtkWidget      *label;
  gchar          *filename;
  fs = gtk_file_selection_new (_("Save as ..."));

  p_obj->fs = GTK_FILE_SELECTION (fs);
  p_obj->save_file = TRUE;

  gtk_file_selection_set_filename (GTK_FILE_SELECTION (fs), ".ps");
  print_object->file_format = POSTSCRIPT;

  label = gtk_label_new (_("File Format :"));
  gtk_widget_show (label);

  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), label, FALSE, FALSE, 0);
  optionmenu = gtk_option_menu_new ();
  gtk_widget_show (optionmenu);


  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (fs)->action_area), optionmenu, FALSE, FALSE, 0);
  optionmenu_menu = gtk_menu_new ();

  glade_menuitem = gtk_menu_item_new_with_label (_("Postscript"));


  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (POSTSCRIPT));
  gtk_widget_show (glade_menuitem);

  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("HTML"));


  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", GTK_SIGNAL_FUNC (fileformat_response), GINT_TO_POINTER (HTML));
  gtk_widget_show (glade_menuitem);

  gtk_menu_append (GTK_MENU (optionmenu_menu), glade_menuitem);


  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), optionmenu_menu);


  /*
   * Set as modal 
   */
  gtk_window_set_position (GTK_WINDOW (fs), GTK_WIN_POS_MOUSE);
  gtk_window_set_modal (GTK_WINDOW (fs), TRUE);

  /*
   * And mark it as a transient dialog 
   */
  if (p_obj)
    gtk_window_set_transient_for (GTK_WINDOW (fs), GTK_WINDOW (p_obj->parent_window));


  gtk_signal_connect (GTK_OBJECT (fs), "destroy", GTK_SIGNAL_FUNC (cmw_destroy_cb), NULL);



  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked", GTK_SIGNAL_FUNC (print_ok), p_obj);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->ok_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (fs));



  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (fs)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (fs));

  /*
   * wait until destroy calls gtk_main_quit 
   */
  gtk_widget_show (fs);
  if (print_dialog != NULL) {
    gtk_widget_destroy (GTK_WIDGET (print_dialog));
    print_dialog = NULL;
  }
  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
  gtk_main ();
  return filename;
}

void
print_runner_sign (GtkWidget * widget, gpointer * data)
{
/*
 * Pipe und Dialog sollten bergeben werden 
 */ FILE       *fd;
  gint            pfd[2], rc;
  gint            gs_arg = 0;
  gchar          *gs_call[100];
  if (PrintingDatas == NULL) {
    PrintingDatas = (PrintStruct *) g_malloc (sizeof (PrintStruct));
  }
  setSize (DINA4);
  PrintingDatas->left = 40;	/* 60 */
  PrintingDatas->right = 60;
  PrintingDatas->unuseableHeader = 0;	/* 25 */
  PrintingDatas->unuseableFooter = 0;
  PrintingDatas->fontsize = 12;
  PrintingDatas->headerfont = 12;
  PrintingDatas->headerSize = 0;	/* 40 */
  PrintingDatas->footerSize = 0;
  PrintingDatas->border = 10;	/* 5 */
  PrintingDatas->Pages = 1;
  PrintingDatas->lines = 0;
  PrintingDatas->PageLines = 0;
  PrintingDatas->PageSum = 1;
  PrintingDatas->headerSize = 4 * PrintingDatas->headerfont;
  PrintingDatas->linesperpage = (PrintingDatas->max_y - PrintingDatas->unuseableHeader - PrintingDatas->headerSize - PrintingDatas->footerSize - PrintingDatas->unuseableFooter - PrintingDatas->border) / PrintingDatas->fontsize - 1;

  rc = pipe (pfd);		/* Create a new pipe */

  if (rc == -1) {
    g_error ("Could not create pipe");
    exit (1);
  }
  rc = fork ();
  if (rc == -1) {
    g_error ("Cant fork ()");
    exit (1);
  } else if (rc == 0) {
    close (fileno (stdin));	/* Close StdInput */
    dup (pfd[0]);		/* Copy the pipe to stdin */
    close (pfd[0]);
    close (pfd[1]);		/* We dont have to write into the pipe from lpr ;-) */
    gs_call[gs_arg++] = "lpr";
    gs_call[gs_arg++] = 0;
    execvp (gs_call[0], gs_call);
    g_error ("exec funktion failed !");
    exit (1);
  }
  fd = fdopen (pfd[1], "w");	/* Open a pipe for writing */
  close (pfd[0]);		/* Close reading pipe */
  printPS_Header (fd);
  printPS_funcs (fd);
  fprintf (fd, "60 700 (%s) 16 (%s, %s Jg. %s) scaleshow\n", FONTBOLD, gtk_entry_get_text (GTK_ENTRY (get_widget (GTK_WIDGET (data), "name_entry"))),
	   gtk_entry_get_text (GTK_ENTRY (get_widget (GTK_WIDGET (data), "prename_entry"))), gtk_entry_get_text (GTK_ENTRY (get_widget (GTK_WIDGET (data), "geburtsjahr_entry"))));
  fprintf (fd, "(%s) 600 %d (%s) 18 centerText\n", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (get_widget (GTK_WIDGET (data), "combo_box_teams"))->entry)), PrintingDatas->max_x, FONTBOLD);
  printPS_showpage (fd);
  printPS_EOF (fd);
  fclose (fd);
  close (pfd[1]);
}

static void
eval_print_string (FILE * fd, gchar * format, ...)
{
  va_list         args;
  gchar          *formated_string = NULL;
  va_start (args, format);
  formated_string = g_strdup_vprintf (format, args);
  fprintf (fd, formated_string);
  va_end (args);
  g_free (formated_string);
}

FILE           *
create_print_pipe (void)
{
  FILE           *fd;
  gint            pfd[2], rc;
  gint            file;
  gchar          *gs_call[100];
  gint            gs_arg = 0;
  gchar          *line = NULL;
  gchar          *filename;
  line = (gchar *) g_malloc0 (255);

  rc = pipe (pfd);		/* Create a new pipe */

  if (rc == -1) {
    g_error ("Could not create pipe");
  }
  rc = fork ();
  if (rc == -1) {
    g_error ("Cant fork ()");
  } else if (rc == 0) {
    close (fileno (stdin));	/* Close StdInput */
    dup (pfd[0]);		/* Copy the pipe to stdin */
    close (pfd[0]);
    close (pfd[1]);		/* We dont have to write into the pipe from lpr ;-) 
				 */
    if (print_object->save_file == TRUE) {
      filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (print_object->fs));
      file = open (filename, O_CREAT | O_TRUNC | O_WRONLY, RC_PERM);
      while (read (fileno (stdin), &line, sizeof (line)) != 0) {
	write (file, &line, sizeof (line));
      }
      close (file);
      print_object->save_file = FALSE;
    } else if (!strcasecmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry)), "default")) {
      gs_call[gs_arg++] = gtk_entry_get_text (GTK_ENTRY (entry));
      gs_call[gs_arg++] = 0;
      execvp (gs_call[0], gs_call);
      g_error ("exec funktion failed !");
      exit (1);
    } else {
      gchar          *printer = NULL;
      printer = g_strdup_printf ("-P%s", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry)));
      gs_call[gs_arg++] = gtk_entry_get_text (GTK_ENTRY (entry));
      gs_call[gs_arg++] = printer;
      gs_call[gs_arg++] = 0;
      execvp (gs_call[0], gs_call);
      if (printer)
	g_free (printer);
      g_error ("exec funktion failed !");
      exit (1);			/* Test _exit(1) here as Havoc said ... */
    }
  }
  fd = fdopen (pfd[1], "w");	/* Open a pipe for writing */
  close (pfd[0]);		/* Close reading pipe  */
  return fd;
}

static void
print_ok (GtkWidget * widget, PrintObject * p_obj)
{
  guint           i, a;
  gint            row;
  FILE           *fd;
  gint            file;
  gint            pfd[2], rc;
  gchar          *filename;
  /* Array`s are much cooler here ;-) */
  gchar          *eval_string = NULL;
  gchar          *clist_text[11];
  gchar          *clist_text0 = NULL;
  gchar          *clist_text1 = NULL;
  gchar          *clist_text2 = NULL;
  gchar          *clist_text3 = NULL;
  gchar          *clist_text4 = NULL;
  gchar          *clist_text5 = NULL;
  gchar          *clist_text6 = NULL;
  gchar          *clist_text7 = NULL;
  gchar          *clist_text8 = NULL;
  gchar          *clist_text9 = NULL;
  gchar          *clist_text10 = NULL;
  gchar          *gs_call[100];
  gchar          *printabletext = NULL;
  date_format     date;
  gint            gs_arg = 0;
  GArray         *number_array;
  gchar         **date_string;
  gchar          *text_tmp = NULL;
  gchar          *line = NULL;
  line = (gchar *) g_malloc0 (255);
  text_tmp = (gchar *) g_malloc0 (255);

  /*
   * This is for PS Printing 
   */

  if (PrintingDatas == NULL) {
    PrintingDatas = (PrintStruct *) g_malloc (sizeof (PrintStruct));
  }
  if (p_obj->ObjectSize < DINA3 || p_obj->ObjectSize > LEGAL) {
    setSize (DINA4);
  } else {
    setSize (p_obj->ObjectSize);
  }

  PrintingDatas->left = 60;	/* 40 */
  PrintingDatas->right = 60;
  PrintingDatas->unuseableHeader = 25;	/* 30 */
  PrintingDatas->unuseableFooter = 25;
  PrintingDatas->fontsize = 10;
  PrintingDatas->headerfont = 12;
  PrintingDatas->headerSize = 0;	/* 40 */
  PrintingDatas->footerSize = 40;
  PrintingDatas->border = 5;	/* 5 */
  PrintingDatas->Pages = 1;
  PrintingDatas->lines = 0;
  PrintingDatas->PageLines = 0;
  PrintingDatas->PageSum = 1;
  PrintingDatas->type = -1;	/* Should be set in every called function with the document type 
				 */

  uname (&sysInfo);
  if (!(PrintingDatas->eventname = (gchar *) g_malloc (255))) {
    g_error ("No more memory!\n");
    return;
  }
  if (!(PrintingDatas->host = (gchar *) g_malloc ((strlen (sysInfo.nodename) + 1)))) {
    g_error ("No more memory!\n");
    return;
  }
  PrintingDatas->user = g_get_user_name ();

  strcpy (PrintingDatas->host, sysInfo.nodename);
  PrintingDatas->headerSize = 4 * PrintingDatas->headerfont;
  PrintingDatas->linesperpage = (PrintingDatas->max_y - PrintingDatas->unuseableHeader - PrintingDatas->headerSize - PrintingDatas->footerSize - PrintingDatas->unuseableFooter - PrintingDatas->border) / PrintingDatas->fontsize - 1;

  rc = pipe (pfd);		/* Create a new pipe */

  if (rc == -1) {
    g_error ("Could not create pipe");
    exit (1);
  }
  rc = fork ();
  if (rc == -1) {
    g_error ("Cant fork ()");
    exit (1);
  } else if (rc == 0) {		/* child */
    close (fileno (stdin));	/* Close StdInput */
    close (fileno (stderr));	/* added 19.06 */
    close (fileno (stdout));	/* added 19.06 */
    dup (pfd[0]);		/* Copy the pipe to stdin */
    close (pfd[1]);		/* We dont have to write into the pipe from lpr ;-) */
    close (pfd[0]);


    if (p_obj->save_file == TRUE) {
      filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (p_obj->fs));
      file = open (filename, O_CREAT | O_TRUNC | O_WRONLY, RC_PERM);
      while (read (fileno (stdin), &line, sizeof (line)) != 0) {
	write (file, &line, sizeof (line));
      }
      close (file);
      p_obj->save_file = FALSE;
    } else if (!strcasecmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry)), "default")) {
      gs_call[gs_arg++] = gtk_entry_get_text (GTK_ENTRY (entry));
      gs_call[gs_arg++] = 0;
      execvp (gs_call[0], gs_call);
      g_error ("exec funktion failed !");
      exit (1);
    } else {
      char           *printer = NULL;
      printer = g_strdup_printf ("-P%s", gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo)->entry)));
      gs_call[gs_arg++] = gtk_entry_get_text (GTK_ENTRY (entry));
      gs_call[gs_arg++] = printer;
      gs_call[gs_arg++] = 0;
      if (printer)
	g_free (printer);
      execvp (gs_call[0], gs_call);
      g_error ("exec funktion failed !");
      exit (1);
    }
  }

  close (pfd[0]);		/* Close reading pipe  */
  fd = fdopen (pfd[1], "w");	/* Open a pipe for writing */

#ifdef DEBUG
  g_message ("File Format = %d", print_object->file_format);
#endif
  /*
   * The pipes where ready now lets start printing .... 
   */
  printf ("IF HTML\n");
  if (p_obj->file_format == HTML) {
    printf ("TEST\n");
    if (p_obj->listtype != RESULTLIST)
      message_box (INFO, "SORRY NOT IMPLEMENTED YET");
    switch (p_obj->listtype) {
    case RESULTLIST:
      printf ("RESULTLIST\n");
      print_html_header (fd);
      res = execSQL ("select event_name from running_events where number = %d", p_obj->event_number);
      print_page_header (fd, PQgetvalue (res, 0, 0));
      PQclear (res);
      print_table_header (fd);
      for (row = 0; row < p_obj->main_clist->rows; row++) {
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 0, &clist_text0);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 1, &clist_text1);
	clist_text1 = cut_text (clist_text1, 10);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 2, &clist_text2);
	clist_text2 = cut_text (clist_text2, 10);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 3, &clist_text3);
	clist_text3 = cut_text (clist_text3, 20);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 4, &clist_text4);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 5, &clist_text5);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 6, &clist_text6);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 7, &clist_text7);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 8, &clist_text8);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 9, &clist_text9);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 10, &clist_text10);
	print_table_data (fd, clist_text0, clist_text1, clist_text2, clist_text3, clist_text4, clist_text5, clist_text6, clist_text7, clist_text8, clist_text9, clist_text10);
	if (clist_text1)
	  g_free (clist_text1);
	if (clist_text2)
	  g_free (clist_text2);
	if (clist_text3)
	  g_free (clist_text3);
      }
      print_table_footer (fd);
      print_html_footer (fd);
      break;
    default:
    }
  } else {
    printf ("SWITCH\n");
    switch (p_obj->listtype) {
    case CERTIFICATE:
      number_array = g_array_new (FALSE, TRUE, sizeof (guint));
      if (cfg.print_event_date == FALSE) {
	date = get_date ();
      } else {
	date_string = g_strsplit (cfg.event_date, ".", 0);
	date.day = atoi (date_string[0]);
	date.month = atoi (date_string[1]);
	date.year = atoi (date_string[2]);
	date.monthname = month_name_de (date.month);
	g_strfreev (date_string);
      }

      printPS_Header (fd);
      printPS_funcs (fd);
      res = execSQL ("select event_name from running_events where number = %d", p_obj->event_number);
#ifdef DEBUG
      printf ("Tuples %d\n", PQntuples (res));
      printf ("Value %s\n", PQgetvalue (res, 0, 0));
#endif
      p_obj->main_clist->selection = g_list_first (p_obj->main_clist->selection);
      while (p_obj->main_clist->selection) {
	row = GPOINTER_TO_INT (p_obj->main_clist->selection->data);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 1, &clist_text1);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 2, &clist_text2);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 3, &clist_text3);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 4, &clist_text4);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 6, &clist_text5);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 7, &clist_text6);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 8, &clist_text7);
	/* Parameters are (Text) Y PaperSize (Font) Fontsize centerText */
	fprintf (fd, "(%s) 440 %d (%s) 16 centerText\n", cfg.main_event_name, PrintingDatas->max_x, FONTBOLD);
	fprintf (fd, "(%s) 410 %d (%s) 16 centerText\n", cfg.sub_event_name, PrintingDatas->max_x, FONTBOLD);
	fprintf (fd, "(%s) 390 %d (%s) 16 centerText\n", cfg.organisation_name, PrintingDatas->max_x, FONTBOLD);
	fprintf (fd, "(%s  %s, %s) 295 %d (%s) 17 centerText\n", clist_text1, clist_text2, clist_text3, PrintingDatas->max_x, FONTBOLD);
	/* fprintf (fd, "200 300 (%s) 14 (%s) scaleshow\n", FONTBOLD, clist_text3); */
	fprintf (fd, "(%s) 265 %d (%s) 14 centerText\n", clist_text4, PrintingDatas->max_x, FONTBOLD);
	fprintf (fd, "(belegte beim %s in der Klasse %s den) 235 %d (%s) 14 centerText\n", PQgetvalue (res, 0, 0), clist_text5, PrintingDatas->max_x, FONTBOLD);
	fprintf (fd, "(%s. Platz) 205 %d (%s) 16  centerText\n", clist_text7, PrintingDatas->max_x, FONTBOLD);
	fprintf (fd, "(Zeit : %s) 175 %d (%s) 15  centerText\n", clist_text6, PrintingDatas->max_x, FONTBOLD);
	fprintf (fd, "80 80 (%s) 14 (%s, den %d. %s %d) scaleshow\n", FONTBOLD, cfg.location_name, date.day, date.monthname, date.year);
	printPS_showpage (fd);
	a = atoi (clist_text1);
	g_array_append_val (number_array, a);
	p_obj->main_clist->selection = p_obj->main_clist->selection->next;
      }
      PQclear (res);
      printPS_EOF (fd);
      for (i = 0; i <= number_array->len; i++) {
	res = execSQL ("update runner set aktiv = aktiv + 1 where number = %d", g_array_index (number_array, guint, i));
	PQclear (res);
      }
      g_array_free (number_array, TRUE);
      break;
    case TEAMLIST:		/*
				 * This seems to be the Teamlist 
				 */
      page_sum (p_obj->main_clist->rows);
      PrintingDatas->eventname = (gchar *) g_malloc0 (255);
      PrintingDatas->eventname = g_strdup (_("Vereinsliste ... \nName  (Interne Nummer)"));
      printPS_Header (fd);
      printPageHeader (fd);
      /*
       * Be cool use pipes !! --> Be cooler use nothing ;-) 
       */
      for (row = 0; row < p_obj->main_clist->rows; row++) {
	printabletext = (gchar *) g_malloc (255);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 0, &clist_text1);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 1, &clist_text2);
	g_snprintf (printabletext, MAX_TEAMNAME_LENGTH + 10, "%s (%d)\n", clist_text2, atoi (clist_text1));
	printabletext = insertNewLines (printabletext);
	for (i = 0; i < strlen (printabletext); i++) {
	  if (NEW_PAGE == translateCharacter ((guchar) printabletext[i], fd)) {
	    printPageFooter (fd);
	    printPageHeader (fd);
	  }
	}
	g_free (printabletext);
      }
      fprintf (fd, ") printline\n");
      fprintf (fd, "%%%%Pages: %d\n", PrintingDatas->Pages);

      printPageFooter (fd);
      printPS_EOF (fd);

      break;
    case RUNNERLIST:
      printf ("RUNNERLIST\n");
      res = execSQL ("select event_name from running_events where number = %d", p_obj->event_number);
      if (PQntuples (res) != 1) {
	g_snprintf (text_tmp, 255, "%s\nNr.  Name                         Vorname                     Verein                                      Geburtsjahr/Klasse \n", _("All"));
      } else {
	g_snprintf (text_tmp, 255, "%s\nNr.  Name                         Vorname                     Verein                                      Geburtsjahr/Klasse \n", PQgetvalue (res, 0, 0));
      }
      PQclear (res);
      PrintingDatas->eventname = (gchar *) g_malloc0 (255);
      PrintingDatas->eventname = g_strdup (text_tmp);
      g_free (text_tmp);
      PrintingDatas->type = RUNNERLIST;
      text_tmp = NULL;
      page_sum (p_obj->main_clist->rows);
      printPS_Header (fd);
      printPageHeader (fd);
      for (row = 0; row < p_obj->main_clist->rows; row++) {
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 0, &clist_text0);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 1, &clist_text1);
	clist_text1 = cut_text (clist_text1, 15);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 2, &clist_text2);
	clist_text2 = cut_text (clist_text2, 15);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 3, &clist_text3);
	clist_text3 = cut_text (clist_text3, 30);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 4, &clist_text4);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 5, &clist_text5);
	text_tmp = (char *) g_malloc (87);
	g_snprintf (text_tmp, 86, "%3d %-15s %-15s %-30s %-4s %-3s  \n", atoi (clist_text0), clist_text1, clist_text2, clist_text3, clist_text4, clist_text5);
	text_tmp = insertNewLines (text_tmp);
	for (i = 0; i < strlen (text_tmp); i++) {
	  if (NEW_PAGE == translateCharacter ((guchar) text_tmp[i], fd)) {
	    printPageFooter (fd);
	    printPageHeader (fd);
	  }
	}

	g_free (text_tmp);
	text_tmp = NULL;
	if (clist_text1)
	  g_free (clist_text1);
	if (clist_text2)
	  g_free (clist_text2);
	if (clist_text3)
	  g_free (clist_text3);
      }
      fprintf (fd, ") printline\n");
      fprintf (fd, "%%%%Pages: %d\n", PrintingDatas->Pages);
      printPageFooter (fd);
      printPS_EOF (fd);
      break;
    case RESULTLIST:		/*
				 * This seems to be the result list 
				 */
      printf ("RESULTLIST2\n");
      res = execSQL ("select event_name from running_events where number = %d", p_obj->event_number);
      if (PQntuples (res) != 1) {
	g_error ("Couldt find an eventname to print !\n");
      }
      g_snprintf (text_tmp, 255, "%s\nNr.  Name            Vorname        Verein                  Geburtsjahr/Klasse  Zeit                  G     K     S     V\n", PQgetvalue (res, 0, 0));
      PQclear (res);
      PrintingDatas->eventname = (gchar *) g_malloc0 (255);
      PrintingDatas->eventname = g_strdup (text_tmp);
      g_free (text_tmp);
      PrintingDatas->type = RESULTLIST;
      text_tmp = NULL;
      page_sum (p_obj->main_clist->rows);
      printPS_Header (fd);
      if (cfg.print_box == TRUE) {
	print_box (fd);
      }
      printPageHeader (fd);
      for (row = 0; row < p_obj->main_clist->rows; row++) {
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 0, &clist_text0);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 1, &clist_text1);
	clist_text1 = cut_text (clist_text1, 10);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 2, &clist_text2);
	clist_text2 = cut_text (clist_text2, 10);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 3, &clist_text3);
	clist_text3 = cut_text (clist_text3, 20);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 4, &clist_text4);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 5, &clist_text5);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 6, &clist_text6);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 7, &clist_text7);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 8, &clist_text8);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 9, &clist_text9);
	gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 10, &clist_text10);
	/*
	 * gtk_clist_get_text (GTK_CLIST (p_obj->main_clist), row, 
	 * 11, &clist_text11); 
	 */
	text_tmp = (char *) g_malloc (87);
	g_snprintf (text_tmp, 86, "%3d %-10s %-10s %-20s %-4s %-3s %-8s | %3s %3s %3s %3s \n", atoi (clist_text0), clist_text1, clist_text2, clist_text3, clist_text4, clist_text5, clist_text6, clist_text7, clist_text8, clist_text9, clist_text10	/*
																															 * ,clist_text11 
																															 */
	  );
	text_tmp = insertNewLines (text_tmp);
	for (i = 0; i < strlen (text_tmp); i++) {
	  if (NEW_PAGE == translateCharacter ((guchar) text_tmp[i], fd)) {
	    printPageFooter (fd);
	    printPageHeader (fd);
	  }
	}

	g_free (text_tmp);
	text_tmp = NULL;
	if (clist_text1)
	  g_free (clist_text1);
	if (clist_text2)
	  g_free (clist_text2);
	if (clist_text3)
	  g_free (clist_text3);
      }
      fprintf (fd, ") printline\n");
      fprintf (fd, "%%%%Pages: %d\n", PrintingDatas->Pages);

      printPageFooter (fd);
      printPS_EOF (fd);
      break;
    case CLASSLIST:
      page_sum (p_obj->lines);
      res = execSQL ("select event_name from running_events where number = %d", p_obj->event_number);
      if (PQntuples (res) != 1) {
	g_error ("Couldt find an eventname to print !\n");
      }
      g_snprintf (text_tmp, 255, _("Klassenwertung ...  %s \nNummer         Name"), PQgetvalue (res, 0, 0));
      PQclear (res);
      PrintingDatas->eventname = (gchar *) g_malloc0 (255);
      PrintingDatas->eventname = g_strdup (text_tmp);
      printPS_Header (fd);
      printPageHeader (fd);
      for (i = 0; i < gtk_text_get_length (GTK_TEXT (p_obj->main_text)); i++) {
	text_tmp = gtk_editable_get_chars (GTK_EDITABLE (p_obj->main_text), i, i + 1);
	text_tmp = insertNewLines (text_tmp);
	for (a = 0; a < strlen (text_tmp); a++) {
	  if (NEW_PAGE == translateCharacter ((guchar) text_tmp[a], fd)) {
	    printPageFooter (fd);
	    printPageHeader (fd);
	  }
	}
      }
      fprintf (fd, ") printline\n");
      fprintf (fd, "%%%%Pages: %d\n", PrintingDatas->Pages);

      printPageFooter (fd);
      printPS_EOF (fd);
      break;
    case TEAMRESULTLIST:
      page_sum (p_obj->lines);
      res = execSQL ("select event_name from running_events where number = %d", p_obj->event_number);
      if (PQntuples (res) != 1) {
	g_error ("Couldt find an eventname to print !\n");
      }
      g_snprintf (text_tmp, 255, (_("Mannschaftswertung ... %s \nNummer         Name")), PQgetvalue (res, 0, 0));
      PQclear (res);
      PrintingDatas->eventname = (gchar *) g_malloc0 (255);
      PrintingDatas->eventname = g_strdup (text_tmp);
      printPS_Header (fd);
      printPageHeader (fd);
      for (i = 0; i < gtk_text_get_length (GTK_TEXT (p_obj->main_text)); i++) {
	text_tmp = gtk_editable_get_chars (GTK_EDITABLE (p_obj->main_text), i, i + 1);
	text_tmp = insertNewLines (text_tmp);
	for (a = 0; a < strlen (text_tmp); a++) {
	  if (NEW_PAGE == translateCharacter ((guchar) text_tmp[a], fd)) {
	    printPageFooter (fd);
	    printPageHeader (fd);
	  }
	}
      }
      fprintf (fd, ") printline\n");
      fprintf (fd, "%%%%Pages: %d\n", PrintingDatas->Pages);

      printPageFooter (fd);
      printPS_EOF (fd);
      break;
    default:
    }
  }
  printf ("Here we are 4\n");
  fclose (fd);
  close (pfd[1]);
  if (PrintingDatas)
  g_free (PrintingDatas);
  PrintingDatas = NULL;
  if (line)
  g_free (line);
  if (text_tmp) {
    g_free (text_tmp);
  }
  if (print_dialog) {
    gtk_widget_destroy (GTK_WIDGET (print_dialog));
    print_dialog = NULL;
  }
/* Free the globale print object */
  destroy_global_print_object ();
}

static void
page_sum (gint lines)
{
  PrintingDatas->PageSum = (lines / PrintingDatas->linesperpage) + 1;
}

gchar          *
cut_text (gchar * text, gint length)
{
  gchar          *text_tmp;
  if (text) {
    text_tmp = (gchar *) g_malloc0 (length);
    text_tmp = g_strndup (text, length);
  } else
    text_tmp = NULL;

  return text_tmp;
}

gint
translateCharacter (guchar c, FILE * fd)
{
  int             anzChars = 0;
  int             i = 0;
  if (c == '\0')
    return 0;
  if (c == '\n') {
    fprintf (fd, ") printline\n");
    PrintingDatas->PageLines++;
    PrintingDatas->characters = 0;
    if (PrintingDatas->PageLines >= PrintingDatas->linesperpage)
      return NEW_PAGE;
    fprintf (fd, "(");
    return 0;
  }				/*
				   * if 
				 */
  if (c == '\t') {
    anzChars = 4 - (PrintingDatas->characters - ((PrintingDatas->characters / 4) * 4));

    for (i = 0; i < anzChars; i++)
      fprintf (fd, " ");
    return 0;
  }				/*
				   * if 
				 */
  if (c >= ' ' && c < 0177) {
    if (c == '(' || c == ')' || c == '\\')
      fprintf (fd, "\\");
    fprintf (fd, "%c", c);
    PrintingDatas->characters++;
    return 0;
  }
  if (c > 0177) {
    fprintf (fd, "\\%o", c);
    PrintingDatas->characters++;
    return 0;
  }
  if (c < ' ') {
    fprintf (fd, "^");
    if ((c = c + '@') == '(' || c == ')' || c == '\\')
      fprintf (fd, "\\");
    fprintf (fd, "%c", c);
    PrintingDatas->characters++;
  } else if (c == 0177) {
    fprintf (fd, "^?");
    PrintingDatas->characters++;
  } else {
    if (c == '(' || c == ')' || c == '\\')
      fprintf (fd, "\\");
    fprintf (fd, "%c", c);
    PrintingDatas->characters++;
  }
  return 1;
}				/*

				 * translateCharacter 
				 */

void
setSize (PaperFormatNumber size)
{
  switch (size) {
  case DINA4:
    PrintingDatas->max_x = 595;
    PrintingDatas->max_y = 839;
    break;
  case DINA3:
    PrintingDatas->max_x = 842;
    PrintingDatas->max_y = 1190;
    break;
  case DINA5:
    PrintingDatas->max_x = 420;
    PrintingDatas->max_y = 595;
    break;
  case LETTER:
    PrintingDatas->max_x = 612;
    PrintingDatas->max_y = 792;
    break;
  case LEGAL:
    PrintingDatas->max_x = 612;
    PrintingDatas->max_y = 1008;
    break;
  default:
    /*
     * DIN A4 
     */
    PrintingDatas->max_x = 595;
    PrintingDatas->max_y = 839;
  }
}
static void
printPS_EOF (FILE * fd)
{
  fprintf (fd, "%%%% EOF");
}
static void
printPS_showpage (FILE * fd)
{
  fprintf (fd, "showpage\n");
}
static void
printPS_funcs (FILE * fd)
{
  fprintf (fd, "%% Aufruf : x y (Font) Skalierung (Text) scaleshow\n");
  fprintf (fd, "/scaleshow\n");
  fprintf (fd, "{/Text exch def\n");
  fprintf (fd, "/Skalierung exch def\n");
  fprintf (fd, "/Basisfont exch def\n");
  fprintf (fd, "/y exch def\n");
  fprintf (fd, "/x exch def\n");
  fprintf (fd, "newpath\n");
  fprintf (fd, "Basisfont findfont\n");
  fprintf (fd, "Skalierung scalefont\n");
  fprintf (fd, "setfont\n");
  fprintf (fd, "x y moveto ");
  fprintf (fd, "Text show\n");
  fprintf (fd, "} def\n");
}
static void
print_box (FILE * fd)
{
  /*
   * Newline didn't work because vpos was alwasy decreased !! 
   */ fprintf (fd, "/vpos %d def\n",
	       PrintingDatas->max_y - 32);
  fprintf (fd, "%%Newline function\n");
  fprintf (fd, "/newline\n");
  fprintf (fd, "{\n");
  fprintf (fd, " newpath\n");
  fprintf (fd, " /vpos");
  fprintf (fd, "   vpos 8 sub def\n");
  fprintf (fd, "   %d vpos moveto\n", PrintingDatas->max_x - 148);
  fprintf (fd, " closepath\n");
  fprintf (fd, "} def\n");
/*
 * Prints a box at the upper right corner of a sheet 
 */
  fprintf (fd, "%% DrawInfoBox\n");
  fprintf (fd, "/DrawInfoBox\n");
  fprintf (fd, "{\n");
  fprintf (fd, "newpath\n");
  fprintf (fd, "%d %d moveto\n", PrintingDatas->max_x - 150, PrintingDatas->max_y - 64);
  fprintf (fd, "110 0 rlineto\n");
  fprintf (fd, "0 36 rlineto\n");
  fprintf (fd, "110 -1 mul 0 rlineto\n");
  fprintf (fd, "closepath\n");
  fprintf (fd, "stroke\n");
  fprintf (fd, "newpath\n");
  fprintf (fd, "/%s findfont\n", FONT);
  fprintf (fd, "%d scalefont\n", 8);
  fprintf (fd, "%d %d moveto\n", PrintingDatas->max_x - 148, PrintingDatas->max_y - 37);
  fprintf (fd, "(G = Gesamt) show\n");
  fprintf (fd, "%d %d moveto\n", PrintingDatas->max_x - 148, PrintingDatas->max_y - 45);
  fprintf (fd, "(K = Klasse) show\n");
  fprintf (fd, "%d %d moveto\n", PrintingDatas->max_x - 148, PrintingDatas->max_y - 53);
  fprintf (fd, "(S = Geschlecht) show\n");
  fprintf (fd, "%d %d moveto\n", PrintingDatas->max_x - 148, PrintingDatas->max_y - 61);
  fprintf (fd, "(V = Verein) show\n");
  fprintf (fd, "closepath\n");
  fprintf (fd, "} def\n");
}

gchar          *
insertNewLines (gchar * str)
{
  gint            i = 0;
  gint            j = 0;

  while ('\0' != str[i]) {
    if ('\n' == str[i]) {
      j = 0;
      PrintingDatas->lines++;
    }				/*
				   * if 
				 */
    if (j > 85) {		/*
				 * 79 
				 */
      for (j = i; str[j] != ' ' && (j > i - 79); j--);
      if (str[j] == ' ') {
	str[j] = '\n';
	j = 86 - j;		/*
				 * 80 
				 */
      } else {
	str[i] = '\n';
	j = 0;
      }				/*
				 * if 
				 */
      PrintingDatas->lines++;
    }				/*
				   * if 
				 */
    i++;
    j++;
  }				/*
				 * while 
				 */
  return str;
}				/*

				 * newLine 
				 */



void
printPageFooter (FILE * fd)
{
  fprintf (fd, "%% \n");
  fprintf (fd, "%% Here starts a new Page.\n");
  fprintf (fd, "%% \n");
  /* fprintf (fd, "/ly1 %d def\n", 44);
     fprintf (fd, "/ly2 %d.%d def\n", 44, 5);
     fprintf (fd, "/y0 %d def\n", 35); */
  fprintf (fd, "/ly1 %d def\n", 64);
  fprintf (fd, "/ly2 %d.%d def\n", 64, 5);
  fprintf (fd, "/y0 %d def\n", 55);
  fprintf (fd, "headerunderline\n");
  fprintf (fd, "x0 10 moveto");
  fprintf (fd, "/fs 10 def\n");
  fprintf (fd, "/" FONT " findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont\n");
  fprintf (fd, "%d (Page %d/%d) stringwidth pop sub ", (PrintingDatas->max_x - 40), PrintingDatas->Pages, ((PrintingDatas->lines + PrintingDatas->linesperpage - 1) / PrintingDatas->linesperpage));
  fprintf (fd, "y0 ");
  fprintf (fd, "moveto\n");
  fprintf (fd, "(Seite %d/%d) show\n", PrintingDatas->Pages, PrintingDatas->PageSum);

  fprintf (fd, "(Printed by: %s\100%s with KRunning Version %s (c) 1998-2001 Andreas Scherf)printline\n", PrintingDatas->user, PrintingDatas->host, KRUNNING_VERSION);

/*  fprintf (fd, "(Mit freundlicher Unterst\374tzung von ) printline\n");
  fprintf (fd, "/fs 14 def\n");
  fprintf (fd, "/BrushStroke findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont");
  fprintf (fd, "(Bubble Jet) show\n");

  fprintf (fd, "/fs 10 def\n");
  fprintf (fd, "/" FONT " findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont\n");
  fprintf (fd,
	   "(Alles f\374r den preiswerten PC Ausdruck - Theodor-Heuss-Allee 1, 54292 Trier, Tel.: 0651/9911459) printline\n");
*/
  fprintf (fd, "showpage\n");
  fprintf (fd, "%% Page %d\n", PrintingDatas->Pages);
  fprintf (fd, "%% \n");

  PrintingDatas->Pages++;
  PrintingDatas->PageLines = 0;

}				/*

				 * printPageFoooter 
				 */
static void
printPS_Header (FILE * fd)
{
  time_t          zeit;
  zeit = time (NULL);
  fprintf (fd, "%%!PS-Adobe-2.0\n");
  fprintf (fd, "%%%%Creator: %s %s\n", KRUNNING_INFO, KRUNNING_VERSION);
  fprintf (fd, "%%%%CreationDate: %s", ctime (&zeit));


  fprintf (fd, "%%%%Pages: %d\n", ((PrintingDatas->lines + PrintingDatas->linesperpage - 1) / PrintingDatas->linesperpage));
  fprintf (fd, "%%%%DocumentFonts: Courier " FONT "\n");
  fprintf (fd, "%%%%EndComments\n");
  fprintf (fd, "%% Lines: %d\n", PrintingDatas->lines);
  fprintf (fd, "%% LinesPerPage: %d\n", PrintingDatas->linesperpage);
  fprintf (fd, "%% by Andreas Scherf\n");


  fprintf (fd, "%% Function for Printing-Lines relativ to x0 y0\n");
  fprintf (fd, "/printline {\n");
  fprintf (fd, "	x0 y0 moveto\n");
  fprintf (fd, "	show\n");
  fprintf (fd, "	/y0 y0 fs sub def\n");
  fprintf (fd, "} def\n");


  fprintf (fd, "%% Function centerText Call : (Text) Y PaperSize (Font) Fontsize centerText\n");
  fprintf (fd, "/centerText  \n");
  fprintf (fd, " {\n");
  fprintf (fd, "  /Skalierung exch def\n");
  fprintf (fd, "  /Basisfont exch def\n");
  fprintf (fd, "  Basisfont findfont\n");
  fprintf (fd, "  Skalierung scalefont\n");
  fprintf (fd, "  setfont\n");
  fprintf (fd, "  2 index stringwidth\n");
  fprintf (fd, "  pop sub 2 div\n");
  fprintf (fd, "  exch moveto\n");
  fprintf (fd, "  show\n");
  fprintf (fd, " } def\n");


  fprintf (fd, "%% Function to draw a box from lx1, ly1 to lx2, ly2\n");
  fprintf (fd, "/headerunderline {\n");
  fprintf (fd, "	newpath\n");
  fprintf (fd, "	lx1 ly1 moveto\n");
  fprintf (fd, "	lx2 ly1 lineto\n");
  fprintf (fd, "	lx2 ly2 lineto\n");
  fprintf (fd, "	lx1 ly2 lineto\n");
  fprintf (fd, "	closepath fill\n");
  fprintf (fd, "} def\n");




  fprintf (fd, "%% Function to draw a box from lx1, ly1 to lx2, ly2\n");
  fprintf (fd, "/headerfill {\n");
  fprintf (fd, "	newpath\n");
  fprintf (fd, "	lx1 2 sub y0 moveto\n");
  fprintf (fd, "	lx2 2 add y0 lineto\n");
  fprintf (fd, "	lx2 2 add ly2 lineto\n");
  fprintf (fd, "	lx1 2 sub ly2 lineto\n");
  fprintf (fd, "	closepath fill\n");
  fprintf (fd, "} def\n");

  /*
   * print text and calc max 
   */
  fprintf (fd, "/printmax {\n");
  fprintf (fd, "  dup\n");
  fprintf (fd, "  printline\n");
  fprintf (fd, "  stringwidth pop\n");
  fprintf (fd, "  2 copy\n");
  fprintf (fd, "  lt {2 1 roll} if\n");
  fprintf (fd, "  pop\n");
  fprintf (fd, "} def\n");

  fprintf (fd, "%% Initialisation\n");
  fprintf (fd, "/x0 %d def\n", PrintingDatas->left);
  fprintf (fd, "/fs %d def\n", PrintingDatas->fontsize);
  fprintf (fd, "/lx1 x0 def\n");
  fprintf (fd, "/lx2 %d def\n", (PrintingDatas->max_x - 40));


  fprintf (fd, "/ly1 %d def\n", (PrintingDatas->max_y - PrintingDatas->unuseableHeader - PrintingDatas->headerSize));
  fprintf (fd, "/ly2 %d def\n", (PrintingDatas->max_y - PrintingDatas->unuseableHeader - PrintingDatas->headerSize));

  fprintf (fd, "%% Enabeling ISOLatin1-Fonts\n");
  fprintf (fd, "/" FONT " findfont\n");
  fprintf (fd, "dup length dict begin\n");



  fprintf (fd, "	{1 index /FID ne {def}{pop pop} ifelse}forall\n");
  fprintf (fd, "	/Encoding ISOLatin1Encoding def\n");
  fprintf (fd, "	currentdict\n");
  fprintf (fd, "end\n");

  fprintf (fd, "/" FONT " exch definefont pop\n");

  fprintf (fd, "/" FONTBOLD " findfont\n");

  fprintf (fd, "dup length dict begin\n");



  fprintf (fd, "	{1 index /FID ne {def}{pop pop} ifelse}forall\n");
  fprintf (fd, "	/Encoding ISOLatin1Encoding def\n");
  fprintf (fd, "	currentdict\n");
  fprintf (fd, "end\n");

  fprintf (fd, "/" FONTBOLD " exch definefont pop\n");


  fprintf (fd, "/Courier findfont\n");

  fprintf (fd, "dup length dict begin\n");



  fprintf (fd, "	{1 index /FID ne {def}{pop pop} ifelse}forall\n");
  fprintf (fd, "	/Encoding ISOLatin1Encoding def\n");
  fprintf (fd, "	currentdict\n");
  fprintf (fd, "end\n");
  fprintf (fd, "/Courier exch definefont pop\n");

  fprintf (fd, "/Courier findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont\n");

}				/*

				 * printPSHeader 
				 */
void
printPageHeader (FILE * fd)
{
  int             i;
  fprintf (fd, "%%%%Page: %d %d\n", PrintingDatas->Pages, PrintingDatas->Pages);
  fprintf (fd, "%% \n");
  fprintf (fd, "%% Page Prologue\n");
  fprintf (fd, "%% \n");
  fprintf (fd, "/y0 %d def\n", (PrintingDatas->max_y - PrintingDatas->unuseableHeader));


  fprintf (fd, "/ly1 %d def\n", (PrintingDatas->max_y - PrintingDatas->unuseableHeader - PrintingDatas->headerSize - 2));
  fprintf (fd, "/ly2 %d.50000 def\n", (PrintingDatas->max_y - PrintingDatas->unuseableHeader - PrintingDatas->headerSize - 2));
  fprintf (fd, "0 setgray\n");
  if (cfg.print_box == TRUE && PrintingDatas->type == RESULTLIST) {
    fprintf (fd, "DrawInfoBox\n");
  }
  if (cfg.gray_header == FALSE) {
    fprintf (fd, "%%headerfill\n");
  } else {
    fprintf (fd, "headerfill\n");
  }
  fprintf (fd, "headerunderline\n");
  if (cfg.gray_header == FALSE) {
    fprintf (fd, "0 setgray\n");
  } else {
    fprintf (fd, "1 setgray\n");
  }

/*
 * Set Headerfont 
 */
  fprintf (fd, "/fs %d def\n", PrintingDatas->headerfont);
  fprintf (fd, "/y0 y0 fs sub def\n");
  fprintf (fd, "/" FONT " findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont\n");

/*
 * print headers 
 */
  fprintf (fd, "/pout {\n0\n");
/*
 * subject 
 */
  fprintf (fd, "(");
  for (i = 0; i < strlen (cfg.main_event_name); i++)
    translateCharacter ((unsigned char) cfg.main_event_name[i], fd);
  fprintf (fd, ") printmax\n");

/*
 */
  fprintf (fd, "(");
  for (i = 0; i < strlen (cfg.sub_event_name); i++)
    translateCharacter ((unsigned char) cfg.sub_event_name[i], fd);
  fprintf (fd, ") printmax\n");

/*
 * printf (  "("); for (i=0; i< strlen (cfg.organisation_name); i++)
 * translateCharacter ( (unsigned char) cfg.organisation_name[i]); printf
 * (  ") printmax\n");
 * 
 * 
 */
  fprintf (fd, "(");
  for (i = 0; i < strlen (PrintingDatas->eventname); i++)
    translateCharacter ((unsigned char) PrintingDatas->eventname[i], fd);
  fprintf (fd, ") printmax\n");

/*
 * maxlen on stack 
 */
  fprintf (fd, "x0 add\n");
  fprintf (fd, "8 add\n");
  fprintf (fd, "} def\n");
  fprintf (fd, "/xzwi pout def\n");
  fprintf (fd, "/y0 %d def\n", (PrintingDatas->max_y - PrintingDatas->unuseableHeader));

/*
 * print header text 
 */
/*
 * main event name 
 */
  fprintf (fd, "/" FONTBOLD " findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont\n");

  fprintf (fd, "/y0 y0 fs sub def\n");
  fprintf (fd, "xzwi y0 moveto\n");

  fprintf (fd, "(");
  for (i = 0; i < strlen (cfg.organisation_name); i++)
    translateCharacter ((unsigned char) cfg.organisation_name[i], fd);
  fprintf (fd, ") show\n");

/*
 * main event name 2 
 */
  fprintf (fd, "/" FONT " findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont\n");

  fprintf (fd, "/y0 y0 fs sub def\n");
  fprintf (fd, "xzwi y0 moveto\n");

  fprintf (fd, "(");
  for (i = 0; i < strlen (cfg.location_name); i++)
    translateCharacter ((unsigned char) cfg.location_name[i], fd);
  fprintf (fd, ") show\n");

/*
 * organisation  fprintf (fd,  "/"FONT" findfont\n"); fprintf (fd,  "fs
 * scalefont\n"); fprintf (fd,  "setfont\n");      
 * 
 * fprintf (fd,  "/y0 y0 fs sub def\n"); fprintf (fd,  "xzwi y0
 * moveto\n");
 * 
 * fprintf (fd,  "("); for (i=0; i< strlen (cfg.organisation_name); i++)
 * translateCharacter ( (unsigned char) cfg.organisation_name[i]); fprintf 
 * (fd,  ") show\n"); 
 */

  fprintf (fd, "/y0 y0 fs sub def\n");
  fprintf (fd, "xzwi y0 moveto\n");
  fprintf (fd, "/yzwi y0 %d sub def\n", PrintingDatas->border);
  fprintf (fd, "( ) show\n");

  fprintf (fd, "/yzwi y0 %d sub def\n", PrintingDatas->border);
  fprintf (fd, "/fs %d def\n", PrintingDatas->fontsize);
  fprintf (fd, "xzwi yzwi moveto\n0 setgray\n");

  fprintf (fd, "/Courier findfont\n");
  fprintf (fd, "fs scalefont\n");
  fprintf (fd, "setfont\n");
  fprintf (fd, "/y0 %d def\n", (PrintingDatas->max_y - PrintingDatas->unuseableHeader - PrintingDatas->headerSize - PrintingDatas->fontsize - PrintingDatas->border));
  fprintf (fd, "(");

}				/* printPageHeader */
