/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "krunenv.h"
#include "preferences.h"
#include "database_modules.h"
#include "ru_event.h"
 /* 
    Table Name "EVENTS"

    Database Format :

    |event_number(int)|event_name(char *)|min(void)|max(void)|starttime(gint32)|distance(int)

  */

GtkWidget      *running_event_dialog;

GtkWidget      *event_number_entry;
GtkWidget      *event_name_entry;
/*GtkWidget *min_number_entry;
   GtkWidget *max_number_entry; */

/*GtkWidget *starttime_nr_entry; */
static GtkCTreeNode *main_node = NULL;
static GtkCTreeNode *leaf_node = NULL;
/*
   The event numbers are stored in this array 
 */
static GArray  *event_number_array;

gboolean        toggleButtonstate = FALSE;

void
clear_dialog (void)
{
  g_array_free (event_number_array, TRUE);
}

static          gint
find_index (GArray * array, guint number)
{
  guint           i;
  if (array && array->len > 0) {
    for (i = 0; i <= array->len; i++) {
      if (g_array_index (array, guint, i) == number)
	return i;
    }
  }
  return -1;
}

static void
focus_change (GtkWidget * widget, gpointer data)
{
#ifdef DEBUG
  g_message ("Focusnumber : %d\n", (gint) data);
#endif
  switch ((gint) data) {
  case 1:
    gtk_widget_grab_focus (GTK_WIDGET (get_widget (running_event_dialog, "starttime_entry")));
    break;
  case 2:
    gtk_widget_grab_focus (GTK_WIDGET (get_widget (running_event_dialog, "distance_entry")));
    break;
  default:
    gtk_widget_grab_focus (GTK_WIDGET (get_widget (running_event_dialog, "event_name_entry")));
  }
}


void
create_running_event_dialog ()
{

  GtkWidget      *mainhbox;
  GtkWidget      *navigation_tree;

  GtkWidget      *notebookframe;
  GtkWidget      *notebook;
  GtkWidget      *vbox1;
  GtkWidget      *label;
  GtkWidget      *starttime_entry;
  GtkWidget      *distance_entry;
  GtkWidget      *temp_vbox;
  GtkWidget      *notebook_vbox;
  GtkWidget      *hbox;

  GtkWidget      *dialog_action_area2;
  GtkWidget      *hbuttonbox;
  GtkWidget      *ok_button;
  GtkWidget      *cancel_button;

  gint            a, number;

  event_number_array = g_array_new (FALSE, TRUE, sizeof (guint));

  running_event_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "running_event_dialog", running_event_dialog);
  GTK_WINDOW (running_event_dialog)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (running_event_dialog), _("Enter running events"));
  gtk_window_set_policy (GTK_WINDOW (running_event_dialog), TRUE, TRUE, FALSE);
  gtk_window_position (GTK_WINDOW (running_event_dialog), GTK_WIN_POS_CENTER);
  gtk_signal_connect (GTK_OBJECT (running_event_dialog), "delete_event", GTK_SIGNAL_FUNC (clear_dialog), NULL);
  gtk_signal_connect (GTK_OBJECT (running_event_dialog), "destroy", GTK_SIGNAL_FUNC (clear_dialog), NULL);


  vbox1 = GTK_DIALOG (running_event_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  /*gtk_container_add (GTK_CONTAINER (frame1), vbox1); */

  /* The main box */
  mainhbox = gtk_hbox_new (FALSE, 6);
  gtk_container_set_border_width (GTK_CONTAINER (mainhbox), 6);
  gtk_box_pack_start (GTK_BOX (vbox1), mainhbox, TRUE, TRUE, 0);
  gtk_widget_show (mainhbox);

  /* The categories tree */
  navigation_tree = gtk_ctree_new (1, 0);
  gtk_clist_column_titles_show (GTK_CLIST (navigation_tree));
  gtk_clist_set_column_title (GTK_CLIST (navigation_tree), 0, _("Events"));
  gtk_ctree_set_indent (GTK_CTREE (navigation_tree), 15);
  gtk_clist_column_titles_passive (GTK_CLIST (navigation_tree));
  gtk_widget_set_usize (navigation_tree, 140, 0);
  gtk_clist_set_selection_mode (GTK_CLIST (navigation_tree), GTK_SELECTION_SINGLE);
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "navigation_tree", navigation_tree);
  gtk_box_pack_start (GTK_BOX (mainhbox), navigation_tree, FALSE, FALSE, 0);
  gtk_widget_show (navigation_tree);

  gtk_signal_connect (GTK_OBJECT (navigation_tree), "tree_select_row", GTK_SIGNAL_FUNC (select_event_node), leaf_node);

  notebookframe = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (notebookframe), GTK_SHADOW_IN);
  gtk_box_pack_start (GTK_BOX (mainhbox), notebookframe, TRUE, TRUE, 0);
  gtk_widget_show (notebookframe);

  notebook = gtk_notebook_new ();
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "notebook", notebook);
  gtk_widget_show (notebook);
  gtk_container_add (GTK_CONTAINER (notebookframe), notebook);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);

  gtk_object_set_user_data (GTK_OBJECT (navigation_tree), notebook);

  res = execSQL ("select number,event_name,starttime_0,distance from running_events order by number");

  for (a = 0; a < PQntuples (res); a++) {
    number = atoi (PQgetvalue (res, a, 0));
    g_array_append_val (event_number_array, number);
  }
  PQclear (res);
/*

   Database name  (main_node)
   [+]
   | 
   +-Event 1    (leaf_node)
   +-Event 2    (    "    )     
   +- ..        (    "    )
   +-Event n    (    "    )     

 */

  temp_vbox = KRunningAppendPage (GTK_NOTEBOOK (notebook), _("Event Properties"), GTK_CTREE (navigation_tree), cfg.database_name, NULL, &main_node, 0);
  gtk_ctree_node_set_selectable (GTK_CTREE (navigation_tree), main_node, FALSE);

  notebook_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (notebook_vbox), 2);
  gtk_box_pack_start (GTK_BOX (temp_vbox), notebook_vbox, FALSE, FALSE, 0);
  gtk_widget_show (notebook_vbox);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
  gtk_box_pack_start (GTK_BOX (notebook_vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  label = gtk_label_new (_("Event name"));
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "label", label);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  event_name_entry = gtk_entry_new_with_max_length (60);
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "event_name_entry", event_name_entry);
  gtk_widget_show (event_name_entry);
  gtk_box_pack_end (GTK_BOX (hbox), event_name_entry, TRUE, TRUE, 0);
  gtk_widget_set_usize (event_name_entry, 140, -1);
  gtk_signal_connect (GTK_OBJECT (event_name_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (1));


  hbox = gtk_hbox_new (FALSE, 3);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
  gtk_box_pack_start (GTK_BOX (notebook_vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  label = gtk_label_new (_("Starttime"));

  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  starttime_entry = gtk_entry_new_with_max_length (8);
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "starttime_entry", starttime_entry);
  gtk_widget_show (starttime_entry);
  gtk_box_pack_start (GTK_BOX (hbox), starttime_entry, TRUE, FALSE, 0);
  gtk_widget_set_usize (starttime_entry, 60, -1);
  gtk_signal_connect (GTK_OBJECT (starttime_entry), "activate", GTK_SIGNAL_FUNC (focus_change), GINT_TO_POINTER (2));

  label = gtk_label_new (_("(hh:mm:ss)"));
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "label", label);
  gtk_widget_show (label);
  gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  hbox = gtk_hbox_new (FALSE, 3);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
  gtk_box_pack_start (GTK_BOX (notebook_vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);

  label = gtk_label_new (_("Distance"));
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  distance_entry = gtk_entry_new_with_max_length (6);
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "distance_entry", distance_entry);
  gtk_widget_show (distance_entry);
  gtk_box_pack_start (GTK_BOX (hbox), distance_entry, TRUE, FALSE, 0);
  gtk_widget_set_usize (distance_entry, 60, -1);
  gtk_signal_connect (GTK_OBJECT (distance_entry), "activate", GTK_SIGNAL_FUNC (insert_entry), leaf_node);

  label = gtk_label_new (_("(m)"));
  gtk_widget_show (label);
  gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

  /* Changed 1->0 and NULL -> ctree_node */
  res = execSQL ("select event_name from running_events");
  for (a = 0; a < PQntuples (res); a++) {
    AppendTreeNode (GTK_CTREE (navigation_tree), PQgetvalue (res, a, 0), main_node, &leaf_node, g_array_index (event_number_array, guint, a));
  }
  PQclear (res);
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "main_vbox", temp_vbox);
  gtk_widget_show (temp_vbox);


  dialog_action_area2 = GTK_DIALOG (running_event_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "dialog_action_area2", dialog_action_area2);
  gtk_widget_show (dialog_action_area2);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), hbuttonbox, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbuttonbox), 2);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 10);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox), 2, 0);

  ok_button = gtk_button_new_with_label (_("New"));
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "ok_button", ok_button);
  gtk_widget_show (ok_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ok_button);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (create_new_entry), NULL);

  ok_button = gtk_button_new_with_label (_("Delete"));
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "ok_button", ok_button);
  gtk_widget_show (ok_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ok_button);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (delete_entry), NULL);

  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (running_event_dialog), "cancel_button", cancel_button);
  gtk_widget_show (cancel_button);
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (running_event_dialog));
  gtk_container_add (GTK_CONTAINER (hbuttonbox), cancel_button);

  gtk_widget_show (running_event_dialog);

}

void
select_event_node (GtkWidget * widget, GtkCTreeNode * node)
{
  int             a;
  int             event_number = -1;
  guint32         seconds;
  gint            hour, min, sec;
  gboolean        in_database = FALSE;
  gchar          *timestring = NULL;

  if (GTK_CLIST (widget)->selection) {
    event_number = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (widget), node);
    leaf_node = node;
#ifdef DEBUG
    g_message ("event_number --> %d\n", event_number);
#endif
  }
  res = execSQL ("select number from running_events");
  for (a = 0; a < PQntuples (res); a++) {
    if (event_number == atoi (PQgetvalue (res, a, 0))) {	/* Is event already in Database ?? */
      in_database = TRUE;
    }
  }
  PQclear (res);
  if (in_database == TRUE) {
    res = execSQL ("select event_name,min_number,max_number,distance,starttime_0 " "from running_events where number = %d", event_number);
    for (a = 0; a < PQntuples (res); a++) {
      gtk_entry_set_text (GTK_ENTRY (get_widget (running_event_dialog, "event_name_entry")), PQgetvalue (res, a, 0));
      /*
         gtk_entry_set_text (GTK_ENTRY (min_number_entry), gtrans_ifase_result_get_item (res, a, 1));
         gtk_entry_set_text (GTK_ENTRY (max_number_entry), gtrans_ifase_result_get_item (res, a, 2)); */
      gtk_entry_set_text (GTK_ENTRY (get_widget (running_event_dialog, "distance_entry")), PQgetvalue (res, a, 3));
      if (PQgetvalue (res, a, 4) != NULL) {
	seconds = atoi (PQgetvalue (res, a, 4));
	hour = seconds / 3600L;
	seconds -= hour * 3600L;
	min = seconds / 60L;
	sec = seconds - min * 60L;
	timestring = g_strdup_printf ("%02d:%02d:%02d", hour, min, sec);
	gtk_entry_set_text (GTK_ENTRY (get_widget (running_event_dialog, "starttime_entry")), timestring);
      }
    }
    PQclear (res);
  } else {
    clear_all_entrys ();
    gtk_widget_grab_focus (event_name_entry);
  }
  if (timestring)
    g_free (timestring);
}

void
delete_entry (void)
{
  gint            event_number = -1;
  if (GTK_CLIST (get_widget (running_event_dialog, "navigation_tree"))->selection) {
    event_number = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (get_widget (running_event_dialog, "navigation_tree")), leaf_node);


    res = execSQL ("delete from running_events where number = %d", event_number);
    PQclear (res);
    /* CRASH after 2 delete !! */
#ifdef DEBUG
    g_message ("delete index -> %d", find_index (event_number_array, event_number));
#endif
    event_number_array = g_array_remove_index (event_number_array, find_index (event_number_array, event_number));
    gtk_ctree_remove_node (GTK_CTREE (get_widget (running_event_dialog, "navigation_tree")), leaf_node);
    clear_all_entrys ();
  } else {
    message_box (WARNING, _("Select event first !!!"));
  }

}


void
create_new_entry (void)
{
  gint            a, new_number = 1;

  for (a = 0; a < event_number_array->len; a++) {
    if (g_array_index (event_number_array, guint, a) == new_number) {
      new_number++;
      a = 0;
    }
  }
/* Start with 1 becaus 0 is the main_node !! */
  AppendTreeNode (GTK_CTREE (get_widget (running_event_dialog, "navigation_tree")), _("New event (*)"), main_node, &leaf_node, new_number);
  g_array_append_val (event_number_array, new_number);
  gtk_ctree_expand (GTK_CTREE (get_widget (running_event_dialog, "navigation_tree")), main_node);
  gtk_ctree_select (GTK_CTREE (get_widget (running_event_dialog, "navigation_tree")), leaf_node);
}

/* 
   insert_entry(GtkWidget * widget, GtkCTreeNode * node)
   Inserts the entered text into the CTree structure 
   and into the database
 */

void
insert_entry (GtkWidget * widget, GtkCTreeNode * node)
{
  gchar         **time;
  guint32         seconds;
  gint            event_number = -1;
  if (GTK_CLIST (get_widget (running_event_dialog, "navigation_tree"))->selection) {
    event_number = (gint) gtk_ctree_node_get_row_data (GTK_CTREE (get_widget (running_event_dialog, "navigation_tree")), leaf_node);
  }
  if (check_entrys2 () == 0) {

    if (strlen (gtk_entry_get_text (GTK_ENTRY (get_widget (running_event_dialog, "starttime_entry")))) != 8) {
      message_box (WARNING, _("Wrong Time Format !"));
      gtk_widget_grab_focus (GTK_WIDGET (get_widget (running_event_dialog, "starttime_entry")));
      return;
    } else {
      time = g_strsplit (gtk_entry_get_text (GTK_ENTRY (get_widget (running_event_dialog, "starttime_entry"))), ":", 0);
      seconds = atoi (time[0]) * 3600L;
      seconds += atoi (time[1]) * 60L;
      seconds += atoi (time[2]);
      g_strfreev (time);
    }


    /*
       res = execSQL ("insert into running_events (number,event_name,min_number,max_number,starttime_0,distance) "
       "values ('%d','%s','%d','%d','%d','%d')",
       atoi (gtk_entry_get_text (GTK_ENTRY (event_number_entry))),
       gtk_entry_get_text (GTK_ENTRY (event_name_entry)),
       atoi (gtk_entry_get_text (GTK_ENTRY (min_number_entry))),
       atoi (gtk_entry_get_text (GTK_ENTRY (max_number_entry))),
       seconds,
       atoi (gtk_entry_get_text (GTK_ENTRY (get_widget (running_event_dialog, "distance_entry")))));
     */

    res = execSQL ("insert into running_events (number,event_name,min_number,max_number,starttime_0,distance) "
		   "values ('%d','%s','%d','%d','%d','%d')", event_number, gtk_entry_get_text (GTK_ENTRY (event_name_entry)), -1, -1, seconds, atoi (gtk_entry_get_text (GTK_ENTRY (get_widget (running_event_dialog, "distance_entry")))));

    PQclear (res);

    gtk_ctree_node_set_text (GTK_CTREE (get_widget (running_event_dialog, "navigation_tree")), leaf_node, 0, gtk_entry_get_text (GTK_ENTRY (get_widget (running_event_dialog, "event_name_entry"))));

  }
}



gint
check_entrys2 (void)
{
  if (GTK_ENTRY (event_name_entry)->text_length == 0) {
    message_box (INFO, _("Enter a valid name first !"));
    gtk_widget_grab_focus (GTK_WIDGET (event_name_entry));
    return 1;
  }
  /*if (atoi (gtk_entry_get_text (GTK_ENTRY (min_number_entry))) >
     atoi (gtk_entry_get_text (GTK_ENTRY (max_number_entry))))
     {
     message_box ( _("The MAX value must be greater\nthan the MIN value !!"));
     gtk_widget_grab_focus (GTK_WIDGET (min_number_entry));
     return 1;
     } */
  return 0;
}

void
entry_toggle_editable (GtkWidget * checkbutton, GtkWidget * entry)
{
  gtk_widget_set_sensitive (entry, GTK_TOGGLE_BUTTON (checkbutton)->active);
  toggleButtonstate = (GTK_TOGGLE_BUTTON (checkbutton)->active);
}

void
clear_all_entrys (void)
{
  gtk_entry_set_text (GTK_ENTRY (event_name_entry), "");
  gtk_entry_set_text (GTK_ENTRY (get_widget (running_event_dialog, "starttime_entry")), "");
  gtk_entry_set_text (GTK_ENTRY (get_widget (running_event_dialog, "distance_entry")), "");
  /*
     gtk_entry_set_text (GTK_ENTRY (min_number_entry), "");
     gtk_entry_set_text (GTK_ENTRY (max_number_entry), ""); */
}
