/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "show_runners.h"
#include "krunenv.h"
#include "editrundlg.h"
#include "edvereindlg.h"
#include "printing.h"
#include "krunning_time.h"
#include "database_modules.h"
#define RUNNERS_CLIST_COLUMNS 12
#define WINDOW_WIDTH 640
#define WINDOW_HEIGHT 480


static gint     clist_rows = 0;
GtkWidget      *optionmenu1_menu;
GtkWidget      *combo_events;

extern PrintObject *print_object;

static KListDatas listdatast;
static void     init_list (void);
static void     copystruct_event (GtkWidget * widget, gpointer number);
static void     copystruct_sorting (GtkWidget * widget, gpointer number);

GtkWidget      *show_runner;

void
create_show_runner ()
{

  GtkWidget      *frame;
  GtkWidget      *vbox1;

  GtkWidget      *hbuttonbox;
  GtkWidget      *hbox1;
  GtkWidget      *vbox4;

  GtkWidget      *scrolled_window;
  GtkWidget      *optionmenu1;
  GtkWidget      *glade_menuitem;
  GtkWidget      *ok_button;
  GtkWidget      *print_button;
  gint            a;

  gchar          *titles[] = {
    N_("Nr."),
    N_("Name"),
    N_("Vorname"),
    N_("Team"),
    N_("Geburtsjahr"),
    N_("Class"),
    N_("Time"),
    N_("Gesamt"),
    N_("Klasse"),
    N_("Geschlecht"),
    N_("Verein"),
    N_("Nation")
  };

  print_object = (PrintObject *) g_malloc (sizeof (PrintObject));

  show_runner = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (show_runner), "show_runner", show_runner);
  gtk_widget_set_usize (show_runner, WINDOW_WIDTH, WINDOW_HEIGHT);
  gtk_window_set_title (GTK_WINDOW (show_runner), _("Show Runners"));
  gtk_window_position (GTK_WINDOW (show_runner), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (show_runner), TRUE, TRUE, FALSE);

  print_object->parent_window = show_runner;	/* To set it modal while saving .. */

  frame = gtk_frame_new (_("Show Runners"));
  gtk_object_set_data (GTK_OBJECT (show_runner), "frame", frame);
  gtk_widget_show (frame);
  gtk_container_add (GTK_CONTAINER (show_runner), frame);
  gtk_container_border_width (GTK_CONTAINER (frame), 5);
  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (show_runner), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (frame), vbox1);
  gtk_container_border_width (GTK_CONTAINER (vbox1), 5);

  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolled_window, TRUE, TRUE, 0);
/*gtk_widget_set_usize (scrolled_window, WINDOW_WIDTH - 20, WINDOW_HEIGHT - 110); */
  gtk_widget_show (scrolled_window);

  print_object->main_clist = GTK_CLIST (gtk_clist_new_with_titles (RUNNERS_CLIST_COLUMNS, titles));
  gtk_object_set_data (GTK_OBJECT (show_runner), "print_object->main_clist", print_object->main_clist);


  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 0, 40);
  gtk_clist_set_column_justification (GTK_CLIST (print_object->main_clist), 0, GTK_JUSTIFY_RIGHT);
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 1, 100);
  gtk_clist_set_column_width (GTK_CLIST (print_object->main_clist), 2, 100);
  for (a = 3; a <= 11; a++) {
    gtk_clist_set_column_auto_resize (GTK_CLIST (print_object->main_clist), a, TRUE);
  }
  gtk_clist_set_column_justification (GTK_CLIST (print_object->main_clist), 4, GTK_JUSTIFY_RIGHT);
  for (a = 7; a <= 11; a++) {
    gtk_clist_set_column_justification (GTK_CLIST (print_object->main_clist), a, GTK_JUSTIFY_RIGHT);
  }
  gtk_clist_column_titles_show (GTK_CLIST (print_object->main_clist));
  gtk_clist_column_titles_passive (GTK_CLIST (print_object->main_clist));
  gtk_clist_set_shadow_type (GTK_CLIST (print_object->main_clist), GTK_SHADOW_IN);
  gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (print_object->main_clist));
  gtk_widget_show (GTK_WIDGET (print_object->main_clist));

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (show_runner), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 0);

  optionmenu1 = create_event_menu ((void (*)(GtkWidget *, gpointer)) copystruct_event, TRUE);
  gtk_box_pack_start (GTK_BOX (hbox1), optionmenu1, FALSE, FALSE, 0);

  init_list ();

  frame = gtk_frame_new (_("Sort by"));
  gtk_object_set_data (GTK_OBJECT (show_runner), "frame", frame);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (hbox1), frame, FALSE, FALSE, 0);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (show_runner), "vbox4", vbox4);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (frame), vbox4);
  gtk_container_border_width (GTK_CONTAINER (vbox4), 5);

  optionmenu1 = gtk_option_menu_new ();
  gtk_object_set_data (GTK_OBJECT (show_runner), "optionmenu1", optionmenu1);
  gtk_widget_show (optionmenu1);
  gtk_box_pack_start (GTK_BOX (vbox4), optionmenu1, FALSE, FALSE, 0);

  optionmenu1_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label (_("Number"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", (GtkSignalFunc) copystruct_sorting, (gpointer) ORDER_BY_NUMBER_N);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("Place"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", (GtkSignalFunc) copystruct_sorting, (gpointer) ORDER_BY_RANG_N);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("Name"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", (GtkSignalFunc) copystruct_sorting, (gpointer) ORDER_BY_NAME_N);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("Time"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", (GtkSignalFunc) copystruct_sorting, (gpointer) ORDER_BY_TIME_N);
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);

  glade_menuitem = gtk_menu_item_new_with_label (_("Class"));
  gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate", (GtkSignalFunc) copystruct_sorting, (gpointer) ORDER_BY_CLASS_N);
  gtk_widget_show (glade_menuitem);


  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), optionmenu1_menu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu1), 0);
  /* If all was selected we have to disable times and rang !!! */
  /* gtk_widget_set_sensitive (menu_path->widget, sensitive); */

  frame = gtk_frame_new (_("else"));
  gtk_object_set_data (GTK_OBJECT (show_runner), "frame", frame);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (hbox1), frame, TRUE, TRUE, 0);

  hbox1 = gtk_label_new ("      ");
  gtk_object_set_data (GTK_OBJECT (show_runner), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 0);

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (show_runner), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);

  print_button = gtk_button_new_with_label (_("Print"));
  gtk_object_set_data (GTK_OBJECT (show_runner), "print_button", print_button);
  gtk_signal_connect (GTK_OBJECT (print_button), "clicked", GTK_SIGNAL_FUNC (print_clist), print_object);
  gtk_widget_show (print_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), print_button);

  ok_button = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (show_runner), "ok_button", ok_button);
  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (show_runner));
  gtk_widget_show (ok_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ok_button);


  gtk_widget_show (show_runner);
}

static void
copystruct_event (GtkWidget * widget, gpointer number)
{
  listdatast.running_event = GPOINTER_TO_INT (number);
  if (GPOINTER_TO_INT (number) == ALL_RUNNERS) {
    /*setinactive print */
    gtk_widget_set_sensitive (get_widget (show_runner, "print_button"), FALSE);
  } else {
    gtk_widget_set_sensitive (get_widget (show_runner, "print_button"), TRUE);
  }
  init_list ();
}

static void
copystruct_sorting (GtkWidget * widget, gpointer number)
{
  listdatast.sorting = GPOINTER_TO_INT (number);
  init_list ();
}

static void
init_list (void)
{
  /* clean anything */
  guint           a, place = 0;
  guint           time[3];


  gchar           text[RUNNERS_CLIST_COLUMNS][50];
  gchar          *texts[RUNNERS_CLIST_COLUMNS];
  gchar          *run_number_state = NULL;
  gchar          *sorting_string = NULL;
  gchar          *show_time_string = NULL;
  gchar          *show_from_string = NULL;
  gchar          *show_select_string = NULL;
  gboolean        show_time = FALSE;
  guint32         seconds = 0;
  clist_rows = 0;


  print_object->event_number = listdatast.running_event;
  gtk_clist_clear (GTK_CLIST (print_object->main_clist));
  for (a = 0; a <= RUNNERS_CLIST_COLUMNS; a++) {
    texts[a] = text[a];
  }

  switch (listdatast.sorting) {
  case ORDER_BY_NUMBER_N:
    sorting_string = g_strdup_printf ("%s", ORDER_BY_NUMBER);
    show_time = FALSE;
    break;
  case ORDER_BY_RANG_N:
    sorting_string = g_strdup_printf ("%s", ORDER_BY_RANG);
    show_time = TRUE;
    break;
  case ORDER_BY_NAME_N:
    sorting_string = g_strdup_printf ("%s", ORDER_BY_NAME);
    show_time = FALSE;
    break;
  case ORDER_BY_TIME_N:
    sorting_string = g_strdup_printf ("%s", ORDER_BY_TIME);
    show_time = TRUE;
    break;
  case ORDER_BY_CLASS_N:
    sorting_string = g_strdup_printf ("%s", ORDER_BY_CLASS);
    show_time = TRUE;
    break;
  default:
    sorting_string = g_strdup_printf ("%s", " ");
  }

  if (show_time == TRUE) {
    show_time_string = g_strdup (" and runner.number = times.runner_number ");
    show_select_string = g_strdup ("times.result_seconds,");
    show_from_string = g_strdup (", times ");
    print_object->listtype = RESULTLIST;
  } else {
    show_time_string = g_strdup (" ");
    show_from_string = g_strdup (" ");
    show_select_string = g_strdup (" ");
    print_object->listtype = RUNNERLIST;
  }

  if (listdatast.running_event == 255 /* 255 == All */ ) {
    res = execSQL ("select runner.number,name,vorname,vereine.v_name,geb_jahr, runner.sex ,0,0 " "from runner, vereine " "where runner.verein = vereine.number %s", sorting_string);
  } else {
    run_number_state = g_strdup_printf (" and  runner.lauf_nr = '%d' ", listdatast.running_event);


    res = execSQL ("select runner.number,name,vorname,vereine.v_name,geb_jahr, runner.sex,%s "
		   " place_total ,place_class ,place_sex,place_team,place_nation " "from runner, vereine %s" "where runner.verein = vereine.number " "%s %s %s", show_select_string, show_from_string, show_time_string, run_number_state, sorting_string);
  }
  gtk_clist_freeze (GTK_CLIST (print_object->main_clist));
  /* PQntuples -> sonstiges */
  for (a = 0; a < PQntuples (res); a++) {
    if (atoi (PQgetvalue (res, a, 0)) > 0) {
      place++;

      sprintf (text[0], "%d", atoi (PQgetvalue (res, a, 0)));	/* Number */
      sprintf (text[1], "%s", PQgetvalue (res, a, 1));	/* Name */
      sprintf (text[2], "%s", PQgetvalue (res, a, 2));	/* Pre-Name */
      sprintf (text[3], "%s", PQgetvalue (res, a, 3));	/* Team - Name */
      sprintf (text[4], "%s", PQgetvalue (res, a, 4));	/* Year of Birth */

      sprintf (text[5], "%s", show_klasse (atoi (PQgetvalue (res, a, 4)), atoi (PQgetvalue (res, a, 5))));

      if (PQgetvalue (res, a, 6)) {
	seconds = atoi (PQgetvalue (res, a, 6));
      } else {
	seconds = 0;
      }
      /*time[0] = seconds / 3600L;
         seconds -= time[0] * 3600L;
         time[1] = seconds / 60L;
         time[2] = seconds - time[1] * 60L;
         sprintf (text[6], "%02d:%02d:%02d", time[0], time[1], time[2]);
       */
      sprintf (text[6], "%s", "");
      if (show_time == TRUE) {
	sprintf (text[7], "%s", PQgetvalue (res, a, 7));
	sprintf (text[8], "%s", PQgetvalue (res, a, 8));
	sprintf (text[9], "%s", PQgetvalue (res, a, 9));
	sprintf (text[10], "%s", PQgetvalue (res, a, 10));
	sprintf (text[11], "%s", PQgetvalue (res, a, 11));
      } else {
	sprintf (text[7], "%s", "");
	sprintf (text[8], "%s", "");
	sprintf (text[9], "%s", "");
	sprintf (text[10], "%s", "");
	sprintf (text[11], "%s", "");
      }
      gtk_clist_insert (GTK_CLIST (print_object->main_clist), clist_rows, texts);
      /* Test Phase */
      gtk_clist_set_text (GTK_CLIST (print_object->main_clist), clist_rows, 6, sec_to_string (seconds));
      clist_rows++;
    }
  }

  PQclear (res);
  if (show_time == FALSE) {
    for (a = 6; a <= 11; a++) {
      gtk_clist_set_column_visibility (GTK_CLIST (print_object->main_clist), a, FALSE);
    }
  } else {
    for (a = 6; a <= 11; a++) {
      gtk_clist_set_column_visibility (GTK_CLIST (print_object->main_clist), a, TRUE);
    }
  }
  gtk_clist_thaw (GTK_CLIST (print_object->main_clist));
  if (run_number_state)
    g_free (run_number_state);
  if (sorting_string)
    g_free (sorting_string);
  if (show_time_string)
    g_free (show_time_string);
  if (show_from_string)
    g_free (show_from_string);
  if (show_select_string)
    g_free (show_select_string);
}
