/*  KRunning
 *  Copyright (C) 1998-2001 Andreas Scherf
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "time_count.h"
#include "krunning_time.h"
#include "krunenv.h"
#include "preferences.h"
#include "classteamprint.h"
#include "database_modules.h"

#define TIME_LIST_COLUMNS 2
#define WINDOW_WIDTH 480
#define WINDOW_HEIGHT 380


static timestruct time_old;

static void     time_list_add (gchar * timestring);
static void     init_time_clist (void);

time_ctrl       main_running_time;

/* Number for the timing event called every second */
static int      time_id;

/* New number format from asclock */
GdkPixmap      *led_pixmap = NULL;
GdkBitmap      *led_mask = NULL;

GdkImage       *map = NULL;
GdkImage       *clock_img = NULL;
GdkImage       *hour_img = NULL;
GdkImage       *min_img = NULL;
GdkImage       *sec_img = NULL;

gchar          *hour_map = NULL;
gchar          *min_map = NULL;
gchar          *sec_map = NULL;
/* end */

static GtkWidget *glade_pixmap_a = NULL;
static GtkWidget *glade_pixmap_b = NULL;
static GtkWidget *glade_pixmap0 = NULL;
static GtkWidget *glade_pixmap1 = NULL;
static GtkWidget *glade_pixmap2 = NULL;
static GtkWidget *glade_pixmap3 = NULL;
static GtkWidget *glade_pixmap4 = NULL;
static GtkWidget *glade_pixmap5 = NULL;


static GtkWidget *time_count_window;


static GtkWidget *fixed2;
GtkWidget      *label3;
GtkWidget      *name_label;

static GtkWidget *spinbutton2;
static GtkWidget *spinbutton3;
static GtkWidget *spinbutton4;
/* Problems with counted times 
 * because there are diferent numbers that come from the sequence !!! 
 */
static int      counted_times = 0;

static GtkWidget *time_list;

static GtkWidget *statusbar;
GtkWidget      *StartNumberEntry;
GtkWidget      *scrolledwindow1;

static GtkWidget *
new_pixmap (int number)
{
  switch (number) {
  case 0:
    return gtk_pixmap_new (KRunningxpm.null, KRunningxpm.null_mask);
  case 1:
    return gtk_pixmap_new (KRunningxpm.one, KRunningxpm.one_mask);
  case 2:
    return gtk_pixmap_new (KRunningxpm.two, KRunningxpm.two_mask);
  case 3:
    return gtk_pixmap_new (KRunningxpm.three, KRunningxpm.three_mask);
  case 4:
    return gtk_pixmap_new (KRunningxpm.four, KRunningxpm.four_mask);
  case 5:
    return gtk_pixmap_new (KRunningxpm.five, KRunningxpm.five_mask);
  case 6:
    return gtk_pixmap_new (KRunningxpm.six, KRunningxpm.six_mask);
  case 7:
    return gtk_pixmap_new (KRunningxpm.seven, KRunningxpm.seven_mask);
  case 8:
    return gtk_pixmap_new (KRunningxpm.eight, KRunningxpm.eight_mask);
  case 9:
    return gtk_pixmap_new (KRunningxpm.nine, KRunningxpm.nine_mask);
  default:
    return gtk_pixmap_new (KRunningxpm.separator, KRunningxpm.separator_mask);
  }
}

static          gint
key_press_event (GtkWidget * widget, GdkEventKey * event)
{
  gtk_widget_grab_focus (StartNumberEntry);
  switch (event->keyval) {
  case 0xff1b:
#ifdef DEBUG
    g_message ("Esc key pressed ...\n");
#endif
    destroy_timeout_window (time_count_window);
    break;
  case 0xff0d:
  case 0xff8d:
#ifdef DEBUG
    g_message ("RETURN pressed \n");
#endif
    get_time (NULL);
    break;

  default:
#ifdef DEBUG
    printf ("I got a %x\n", event->keyval);
#endif
  }
  return TRUE;
}

static int
mytime ()
{
  struct timeval  tv;
  struct timezone tz;

  gettimeofday (&tv, &tz);

  return tv.tv_sec;
}

GtkWidget      *
create_time_count_window (void)
{
  GtkWidget      *event_name_label;

  GtkWidget      *frame1;

  GtkWidget      *time_handlebox;

  GtkWidget      *main_hbox;
  GtkWidget      *left_vbox;
  GtkWidget      *fixed3;
  GtkWidget      *frame2;
  GtkWidget      *label2;
  GtkWidget      *frame3;
  GtkWidget      *dialog_vbox;

  GtkWidget      *dialog_action_area2;

  GtkStyle       *style;

  gint            context_id;

  static gchar   *titles[] = {
    N_("Time"),
    N_("Name")
  };

  time_count_window = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (time_count_window), "time_count_window", time_count_window);
  GTK_WINDOW (time_count_window)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (time_count_window), _("Time count"));
  gtk_window_position (GTK_WINDOW (time_count_window), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (time_count_window), TRUE, TRUE, FALSE);
  gtk_widget_set_usize (time_count_window, WINDOW_WIDTH, WINDOW_HEIGHT);
  gtk_signal_connect (GTK_OBJECT (time_count_window), "key_press_event", (GtkSignalFunc) key_press_event, NULL);

  dialog_vbox = GTK_DIALOG (time_count_window)->vbox;
  gtk_object_set_data (GTK_OBJECT (time_count_window), "dialog_vbox", dialog_vbox);
  gtk_widget_show (dialog_vbox);

  main_hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (dialog_vbox), main_hbox);
  gtk_container_border_width (GTK_CONTAINER (main_hbox), BORDER_WIDTH);
  gtk_widget_show (main_hbox);

  left_vbox = gtk_vbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (main_hbox), left_vbox, TRUE, TRUE, 0);
  gtk_widget_show (left_vbox);

  frame1 = gtk_frame_new (_("Run Number"));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "frame1", frame1);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (left_vbox), frame1, FALSE, TRUE, 0);

  res = execSQL ("select event_name from running_events where number = %d", main_running_time.run_number);

  event_name_label = gtk_label_new (PQgetvalue (res, 0, 0));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "event_name_label", event_name_label);
  gtk_widget_show (event_name_label);
  gtk_container_add (GTK_CONTAINER (frame1), event_name_label);
  gtk_widget_set_usize (event_name_label, 100, 20);

  PQclear (res);

  time_handlebox = gtk_handle_box_new ();
  gtk_object_set_data (GTK_OBJECT (time_count_window), "time_handlebox", time_handlebox);
  gtk_widget_show (time_handlebox);
  gtk_box_pack_start (GTK_BOX (left_vbox), time_handlebox, FALSE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (time_handlebox), BORDER_WIDTH);

  frame2 = gtk_frame_new (_("Elapsed Time:"));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "frame2", frame2);
  gtk_widget_show (frame2);
  gtk_container_add (GTK_CONTAINER (time_handlebox), frame2);
  /*gtk_fixed_put(GTK_FIXED(fixed0), frame2, 5, 50); */
  /* gtk_widget_set_usize (frame2, 150, 60); */

  fixed2 = gtk_fixed_new ();
  gtk_object_set_data (GTK_OBJECT (time_count_window), "fixed2", fixed2);
  gtk_widget_show (fixed2);
  gtk_container_add (GTK_CONTAINER (frame2), fixed2);


  gtk_widget_realize (time_count_window);


  label2 = gtk_label_new (" hh:mm:ss ");
  gtk_object_set_data (GTK_OBJECT (time_count_window), "label2", label2);
  gtk_widget_show (label2);
  gtk_fixed_put (GTK_FIXED (fixed2), label2, 45, 25);

  time_old.hour = main_running_time.currenttime[0];
  time_old.minute = main_running_time.currenttime[1];
  time_old.second = main_running_time.currenttime[2];
  main_running_time = gettime (main_running_time);


  glade_pixmap0 = new_pixmap (calculate_time (0) / 10);
#ifdef DEBUG
  g_message ("TIME current: %d", (main_running_time.currenttime[0]));
  g_message ("TIME start  : %d", main_running_time.starttime[0]);
  g_message ("TIME h1: %d", ABS (calculate_time (0) / 10));
  g_message ("TIME h2: %d", ABS (calculate_time (0) % 10));
#endif
  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap0", glade_pixmap0);
  gtk_widget_show (glade_pixmap0);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap0, 10, 10);

  glade_pixmap1 = new_pixmap (calculate_time (0) % 10);

  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap1", glade_pixmap1);
  gtk_widget_show (glade_pixmap1);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap1, 26, 10);

  glade_pixmap_a = new_pixmap (10);

  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap_a", glade_pixmap_a);
  gtk_widget_show (glade_pixmap_a);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap_a, 42, 10);

  glade_pixmap2 = new_pixmap (calculate_time (1) / 10);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap2", glade_pixmap2);
  gtk_widget_show (glade_pixmap2);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap2, 58, 10);

  glade_pixmap3 = new_pixmap (calculate_time (1) % 10);

  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap3", glade_pixmap3);
  gtk_widget_show (glade_pixmap3);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap3, 74, 10);

  glade_pixmap_b = new_pixmap (10);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap_b", glade_pixmap_b);
  gtk_widget_show (glade_pixmap_b);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap_b, 90, 10);

  glade_pixmap4 = new_pixmap (calculate_time (2) / 10);

  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap4", glade_pixmap4);
  gtk_widget_show (glade_pixmap4);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap4, 106, 10);

  glade_pixmap5 = new_pixmap (calculate_time (2) % 10);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap5", glade_pixmap5);
  gtk_widget_show (glade_pixmap5);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap5, 122, 10);



  frame3 = gtk_frame_new (_("Get Times & Numbers :"));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "frame3", frame3);
  gtk_widget_show (frame3);
  gtk_box_pack_end (GTK_BOX (left_vbox), frame3, TRUE, TRUE, 0);
  /*gtk_widget_set_usize (frame3, 150, 100); */

  fixed3 = gtk_fixed_new ();
  gtk_object_set_data (GTK_OBJECT (time_count_window), "fixed3", fixed3);
  gtk_widget_show (fixed3);
  gtk_container_add (GTK_CONTAINER (frame3), fixed3);

  label3 = gtk_label_new (_("Time Count : "));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "label3", label3);
  gtk_widget_show (label3);
  gtk_fixed_put (GTK_FIXED (fixed3), label3, 5, 5);

  label2 = gtk_label_new (_("Name : "));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "label2", label2);
  gtk_widget_show (label2);
  gtk_fixed_put (GTK_FIXED (fixed3), label2, 5, 25);

  name_label = gtk_label_new (NULL);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "name_label", name_label);
  gtk_widget_show (name_label);
  gtk_fixed_put (GTK_FIXED (fixed3), name_label, 40, 25);

  StartNumberEntry = gtk_entry_new_with_max_length (6);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "StartNumberEntry", StartNumberEntry);
  gtk_widget_show (StartNumberEntry);
  /*
     gtk_signal_connect (GTK_OBJECT (StartNumberEntry), "activate",
     GTK_SIGNAL_FUNC (get_time),
     NULL);
   */
  gtk_fixed_put (GTK_FIXED (fixed3), StartNumberEntry, 5, 45);
  gtk_widget_grab_focus (StartNumberEntry);
  gtk_widget_set_usize (StartNumberEntry, 50, -1);


  /* We have to kill the timeout after destroying the widget */
  time_id = gtk_timeout_add (1000, count_time, &context_id);
  gtk_signal_connect (GTK_OBJECT (time_count_window), "delete_event", GTK_SIGNAL_FUNC (destroy_timeout_window), NULL);

  /* The times window */
  frame1 = gtk_frame_new (_("Times"));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "frame1", frame1);
  gtk_widget_show (frame1);
  gtk_widget_set_usize (frame1, WINDOW_WIDTH - 160, WINDOW_HEIGHT - 50);
  gtk_box_pack_end (GTK_BOX (main_hbox), frame1, TRUE, TRUE, 0);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "scrolledwindow1", scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show (scrolledwindow1);
  GTK_WIDGET_UNSET_FLAGS (scrolledwindow1, GTK_CAN_FOCUS);
  gtk_container_add (GTK_CONTAINER (frame1), scrolledwindow1);

  time_list = gtk_clist_new_with_titles (TIME_LIST_COLUMNS, titles);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "time_list", time_list);
  gtk_clist_set_column_auto_resize (GTK_CLIST (time_list), 0, TRUE);
  gtk_clist_set_column_auto_resize (GTK_CLIST (time_list), 1, TRUE);
  gtk_widget_show (time_list);
  gtk_clist_set_selection_mode (GTK_CLIST (time_list), GTK_SELECTION_MULTIPLE);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), time_list);

  dialog_action_area2 = GTK_DIALOG (time_count_window)->action_area;
  gtk_object_set_data (GTK_OBJECT (time_count_window), "dialog_action_area2", dialog_action_area2);
  gtk_widget_show (dialog_action_area2);
  /* gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10); */

  fixed3 = gtk_fixed_new ();
  gtk_object_set_data (GTK_OBJECT (time_count_window), "fixed3", fixed3);
  gtk_widget_show (fixed3);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), fixed3, FALSE, TRUE, 0);

  statusbar = gtk_label_new (_("Press [Esc] to exit ..."));
  gtk_fixed_put (GTK_FIXED (fixed3), statusbar, 5, 5);

  gtk_object_set_data (GTK_OBJECT (time_count_window), "statusbar", statusbar);
  gtk_widget_set_usize (statusbar, WINDOW_WIDTH - 10, 20);


  style = gtk_style_new ();
  style->fg[GTK_STATE_NORMAL] = LRED;
  gtk_widget_set_style (statusbar, style);

  gtk_widget_show (statusbar);
  gtk_widget_show (time_count_window);
  if (counted_times > 0 && cfg.show_times == TRUE) {
    init_time_clist ();
  }
  return time_count_window;
}

void
destroy_timeout_window (GtkWidget * window)
{
  if (time_id)
    gtk_timeout_remove (time_id);
  counted_times = 0;
  gtk_widget_destroy (window);
}

void
get_time (gpointer data)
{
  guint32         seconds;
  gint            runner_number;
  guint32         time_number;
  gchar          *timestring = NULL;
  gchar          *namestring = NULL;
  gboolean        entry_error = FALSE;

  /* is this save ????? */

  res = execSQL ("select * from times where run_number = %d", main_running_time.run_number);
  counted_times = PQntuples (res) + 1;
  PQclear (res);

  seconds = calculate_time (0) * 3600L + calculate_time (1) * 60L + calculate_time (2);
#ifdef DEBUG
  g_message ("string len = %d\n", strlen (gtk_entry_get_text (GTK_ENTRY (StartNumberEntry))));
#endif

  runner_number = atoi (gtk_entry_get_text (GTK_ENTRY (StartNumberEntry)));
  /* Now we have to test is the number stored before and also is this the 
     right event ?? */
  if (runner_number > 0) {
    res = execSQL ("select runner_number from times where runner_number = %d", runner_number);
    if (PQntuples (res) != 0) {
      /* Runner has already a time !! */
      ndisplay_text (statusbar, _("Runner already has a time !"), DISPLAY_TIME);
      gdk_beep ();
      entry_error = TRUE;
      counted_times--;
      gtk_entry_set_text (GTK_ENTRY (StartNumberEntry), "");
      PQclear (res);
    }
    /* Event test */
    res = execSQL ("select number from runner where number = %d and lauf_nr = %d", runner_number, main_running_time.run_number);
    if (PQntuples (res) != 1) {
      /* There is no such runner or its the wrong event */
      ndisplay_text (statusbar, _("No such Runner or wrong event !"), DISPLAY_TIME);
      gdk_beep ();
      entry_error = TRUE;
      counted_times--;
      gtk_entry_set_text (GTK_ENTRY (StartNumberEntry), "");
      PQclear (res);

    }
  }
  if (strlen (gtk_entry_get_text (GTK_ENTRY (StartNumberEntry))) == 0 || gtk_entry_get_text (GTK_ENTRY (StartNumberEntry)) <= 0) {
    /* Get a number for the time from the sequencer */
    res = execSQL ("select nextval ('time_sequence')");
    time_number = atoi (PQgetvalue (res, 0, 0));
    PQclear (res);

    /* Write a dummy with the time into the database */
    runner_number = -1;
#ifdef DEBUG
    printf ("Lauf %d\n", main_running_time.run_number);
    printf ("Zeit Nummer %d\n", time_number);
    printf ("Luer %d\n", runner_number);
#endif
    res = execSQL ("insert into times (number,run_number,runner_number,result_seconds) values ('%d','%d','%d','%d')", time_number, main_running_time.run_number, runner_number, seconds);

    PQclear (res);
  } else if (entry_error == FALSE) {
    /* Get a number for the time from the sequencer */
    res = execSQL ("select nextval ('time_sequence')");
    time_number = atoi (PQgetvalue (res, 0, 0));
    PQclear (res);
    /* Write the time into the database */
    res = execSQL ("insert into times (number,run_number,runner_number,result_seconds) " "values ('%d','%d','%d','%d')", time_number, main_running_time.run_number, runner_number, seconds);
    PQclear (res);
  }
  timestring = g_strdup_printf (_("Times counted : %d"), counted_times);

  if (entry_error == FALSE) {
    res = execSQL ("select name,vorname from runner where number = %d", runner_number);
    if (PQntuples (res) > 0) {
      namestring = g_strdup_printf ("%s, %s", PQgetvalue (res, 0, 0), PQgetvalue (res, 0, 1));
      gtk_label_set (GTK_LABEL (name_label), namestring);
    } else {
      namestring = g_strdup_printf ("%s, %s", "----------", "----------");
      gtk_label_set (GTK_LABEL (name_label), namestring);
    }
    PQclear (res);

    gtk_label_set (GTK_LABEL (label3), timestring);
    gtk_entry_set_text (GTK_ENTRY (StartNumberEntry), "");

    time_list_add (namestring);
  }
  if (timestring)
    g_free (timestring);
  if (namestring)
    g_free (namestring);
}


gint
count_time (gpointer data)
{
  /* This function was called every second */
  main_running_time = gettime (main_running_time);

  if (time_old.hour != calculate_time (0)) {
    gtk_widget_destroy (glade_pixmap0);
    glade_pixmap0 = new_pixmap (calculate_time (0) / 10);

    gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap0", glade_pixmap0);
    gtk_widget_show (glade_pixmap0);
    gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap0, 10, 10);

    gtk_widget_destroy (glade_pixmap1);
    glade_pixmap1 = new_pixmap (calculate_time (0) % 10);
    gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap1", glade_pixmap1);
    gtk_widget_show (glade_pixmap1);
    gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap1, 26, 10);

    gtk_widget_destroy (glade_pixmap_a);
    glade_pixmap_a = new_pixmap (10);
    gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap_a", glade_pixmap_a);
    gtk_widget_show (glade_pixmap_a);
    gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap_a, 42, 10);

  }
  if (time_old.minute != calculate_time (1)) {
    gtk_widget_destroy (glade_pixmap2);
    glade_pixmap2 = new_pixmap (calculate_time (1) / 10);
    gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap2", glade_pixmap2);
    gtk_widget_show (glade_pixmap2);
    gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap2, 58, 10);

    gtk_widget_destroy (glade_pixmap3);
    glade_pixmap3 = new_pixmap (calculate_time (1) % 10);
    gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap3", glade_pixmap3);
    gtk_widget_show (glade_pixmap3);
    gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap3, 74, 10);

    gtk_widget_destroy (glade_pixmap_b);
    glade_pixmap_b = new_pixmap (10);
    gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap_b", glade_pixmap_b);
    gtk_widget_show (glade_pixmap_b);
    gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap_b, 90, 10);
  }
  if (time_old.second != main_running_time.currenttime[2]) {
    gtk_widget_destroy (glade_pixmap4);
    glade_pixmap4 = new_pixmap (calculate_time (2) / 10);
    gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap4", glade_pixmap4);
    gtk_widget_show (glade_pixmap4);
    gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap4, 106, 10);
  }
  gtk_widget_destroy (glade_pixmap5);
  glade_pixmap5 = new_pixmap (calculate_time (2) % 10);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "glade_pixmap5", glade_pixmap5);
  gtk_widget_show (glade_pixmap5);
  gtk_fixed_put (GTK_FIXED (fixed2), glade_pixmap5, 122, 10);

  time_old.hour = calculate_time (0);
  time_old.minute = calculate_time (1);
  time_old.second = calculate_time (2);

  return 1;
}

void
create_get_rtime_window ()
{
  GtkWidget      *time_count_window;
  GtkWidget      *vbox1;

  GtkWidget      *event_menue;

  GtkWidget      *frame3;
  GtkWidget      *hbox1;
  GtkWidget      *hbuttonbox;
  GtkWidget      *dialog_action_area2;
  GtkObject      *spinbutton2_adj;
  GtkObject      *spinbutton3_adj;
  GtkObject      *spinbutton4_adj;

  GtkWidget      *ok_button;
  guint32         seconds;

  gchar          *spintime = NULL;
  spintime = (gchar *) g_malloc (3);

  time_count_window = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (time_count_window), "time_count_window", time_count_window);
  GTK_WINDOW (time_count_window)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (time_count_window), _("Select event & Starttime"));
  gtk_window_position (GTK_WINDOW (time_count_window), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (time_count_window), FALSE, FALSE, FALSE);

  vbox1 = GTK_DIALOG (time_count_window)->vbox;
  gtk_object_set_data (GTK_OBJECT (time_count_window), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_border_width (GTK_CONTAINER (vbox1), BORDER_WIDTH);

  event_menue = create_event_menu ((void (*)(GtkWidget *, gpointer)) menuitem_response, FALSE);
  gtk_box_pack_start (GTK_BOX (vbox1), event_menue, FALSE, TRUE, 0);
  gtk_widget_show (event_menue);

  frame3 = gtk_frame_new (_("Starttime in hh:mm:ss"));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "frame3", frame3);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox1), frame3, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 10);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_container_add (GTK_CONTAINER (frame3), hbox1);
  gtk_container_border_width (GTK_CONTAINER (hbox1), BORDER_WIDTH);

  /* Should be the same ... */
  /* ndern !!!! */
  main_running_time.run_number = KRunning_event_number.event_number;
#ifdef DEBUG
  printf ("%d\n", main_running_time.run_number);
#endif
  res = execSQL ("select starttime_0 from running_events where number = %d", main_running_time.run_number);
  if (PQntuples (res) == 0 /* FIXME */ ) {
    g_error ("\nSelect didn't return tuples properly\n");
    exit (2);
  }
  if (PQntuples (res) > 0) {
    seconds = atoi (PQgetvalue (res, 0, 0));
  } else
    seconds = 0;
  PQclear (res);
/*END */
  main_running_time.starttime[0] = seconds / 3600L;
  seconds -= main_running_time.starttime[0] * 3600L;
  main_running_time.starttime[1] = seconds / 60L;
  main_running_time.starttime[2] = seconds - main_running_time.starttime[1] * 60L;

  spinbutton2_adj = gtk_adjustment_new (12, 0, 23, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "spinbutton2", spinbutton2);
  gtk_widget_show (spinbutton2);
  g_snprintf (spintime, 3, "%02d", main_running_time.starttime[0]);

  gtk_entry_set_text (GTK_ENTRY (spinbutton2), spintime);
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton2, FALSE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton2), TRUE);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (spinbutton2), GTK_UPDATE_ALWAYS);

  spinbutton3_adj = gtk_adjustment_new (0, 0, 59, 1, 10, 10);
  spinbutton3 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton3_adj), 1, 0);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "spinbutton3", spinbutton3);
  gtk_widget_show (spinbutton3);
  g_snprintf (spintime, 3, "%02d", main_running_time.starttime[1]);
  gtk_entry_set_text (GTK_ENTRY (spinbutton3), spintime);
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton3, FALSE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton3), TRUE);


  spinbutton4_adj = gtk_adjustment_new (0, 0, 59, 1, 10, 10);
  spinbutton4 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton4_adj), 0, 0);
  gtk_object_set_data (GTK_OBJECT (time_count_window), "spinbutton4", spinbutton4);
  g_snprintf (spintime, 3, "%02d", main_running_time.starttime[2]);
  gtk_entry_set_text (GTK_ENTRY (spinbutton4), spintime);

  gtk_widget_show (spinbutton4);

  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton4, FALSE, TRUE, 0);
  /* gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton4), TRUE);
   */

  dialog_action_area2 = GTK_DIALOG (time_count_window)->action_area;
  gtk_object_set_data (GTK_OBJECT (time_count_window), "dialog_action_area2", dialog_action_area2);
  gtk_widget_show (dialog_action_area2);
  /* gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10); */

  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (time_count_window), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), hbuttonbox, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);

  ok_button = gtk_button_new_with_label (_("Ok"));
  gtk_object_set_data (GTK_OBJECT (time_count_window), "ok_button", ok_button);
  gtk_widget_show (ok_button);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (get_time_and_run), NULL);

  gtk_signal_connect_object (GTK_OBJECT (ok_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (time_count_window));
  gtk_container_add (GTK_CONTAINER (hbuttonbox), ok_button);

  g_free (spintime);

  gtk_widget_show (time_count_window);
}

void
menuitem_response (GtkWidget * widget, gpointer number)
{
  gint32          seconds;
  gchar          *spintime = NULL;

  main_running_time.run_number = (gint) number;
#ifdef DEBUG
  g_message ("Selected run number %d\n", (gint) number);
#endif
  res = execSQL ("select starttime_0 from running_events where number = %d", main_running_time.run_number);
  seconds = atoi (PQgetvalue (res, 0, 0));
  PQclear (res);
  main_running_time.starttime[0] = seconds / 3600L;
  seconds -= main_running_time.starttime[0] * 3600L;
  main_running_time.starttime[1] = seconds / 60L;
  main_running_time.starttime[2] = seconds - main_running_time.starttime[1] * 60L;

  spintime = g_strdup_printf ("%02d", main_running_time.starttime[0]);
  gtk_entry_set_text (GTK_ENTRY (spinbutton2), spintime);
  spintime = g_strdup_printf ("%02d", main_running_time.starttime[1]);
  gtk_entry_set_text (GTK_ENTRY (spinbutton3), spintime);
  spintime = g_strdup_printf ("%02d", main_running_time.starttime[2]);
  gtk_entry_set_text (GTK_ENTRY (spinbutton4), spintime);
  if (spintime)
    g_free (spintime);
}


void
get_time_and_run (gpointer data)
{
  gint32          seconds = 0;
  gint            tuples = 0;
  main_running_time.starttime[0] = atoi (gtk_entry_get_text (GTK_ENTRY (spinbutton2)));	/* Stunden */
  main_running_time.starttime[1] = atoi (gtk_entry_get_text (GTK_ENTRY (spinbutton3)));	/* Minuten */
  main_running_time.starttime[2] = atoi (gtk_entry_get_text (GTK_ENTRY (spinbutton4)));	/* Sekunden */
  seconds = main_running_time.starttime[0] * 3600L + main_running_time.starttime[1] * 60L + main_running_time.starttime[2];

#ifdef DEBUG
  g_message ("Getting time ....%d\n", main_running_time.starttime[0]);
#endif
  res = execSQL ("select COUNT(*) from times where run_number = %d", main_running_time.run_number);
  tuples = atoi (PQgetvalue (res, 0, 0));
  PQclear (res);
  if (tuples > 0) {
    /*Zeiteintrge in Datenbank vorhanden */
    /*Entweder Tabelle lschen oder die Nummer der Zeiten und PQntubles setzen ! */
    res = execSQL ("select event_name from running_events where number = %d", main_running_time.run_number);
    gtk_widget_show (create_db_question_dialog_ (PQgetvalue (res, 0, 0), tuples));
    PQclear (res);
  } else {
    res = execSQL ("update running_events set starttime_0 ='%d' where number = %d", seconds, main_running_time.run_number);

    PQclear (res);
    create_time_count_window ();
  }
}

gint
calculate_time (gint number)
{
  gint32          sek_akt, sek_start, timediff;
  gint            hour, min, sec;

  sek_akt = main_running_time.currenttime[0] * 3600L + main_running_time.currenttime[1] * 60L + main_running_time.currenttime[2];
  sek_start = main_running_time.starttime[0] * 3600L + main_running_time.starttime[1] * 60L + main_running_time.starttime[2];
  timediff = sek_akt - sek_start;

  hour = timediff / 3600L;
  timediff -= hour * 3600L;
  min = timediff / 60L;
  sec = timediff - min * 60L;

  switch (number) {
  case 0:
    return ABS (hour);
  case 1:
    return ABS (min);
  case 2:
    return ABS (sec);
  }
  return -1;
}



static void
time_list_add (gchar * namestring)
{
  guint           time_number;
  char            text[TIME_LIST_COLUMNS][24];
  char           *texts[TIME_LIST_COLUMNS];

  texts[0] = text[0];
  texts[1] = text[1];

  res = execSQL ("select last_value from time_sequence");
  time_number = atoi (PQgetvalue (res, 0, 0));
  PQclear (res);
#ifdef DEBUG
  g_message ("Last Number %d", time_number);
#endif
  gtk_clist_freeze (GTK_CLIST (time_list));
  if (cfg.show_times == TRUE) {
    g_snprintf (text[0], 24, _("%03d Time : %02d:%02d:%02d"), time_number, calculate_time (0), calculate_time (1), calculate_time (2));
    g_snprintf (text[1], 24, "%s", namestring);
    gtk_clist_append (GTK_CLIST (time_list), texts);
  }
  gtk_clist_thaw (GTK_CLIST (time_list));
/* is clist entry visible  ?? 
 * GTK_VISIBILITY_NONE,
 * GTK_VISIBILITY_PARTIAL,
 * GTK_VISIBILITY_FULL */
  /* Is this a bug that here we couldnt see the last clist entry ?? */
  if (gtk_clist_row_is_visible (GTK_CLIST (time_list), GTK_CLIST (time_list)->rows
/*- 1*/
      ) != GTK_VISIBILITY_FULL) {
    gtk_clist_moveto (GTK_CLIST (time_list), GTK_CLIST (time_list)->rows - 1, 0, 0.0, 0.0);
  }

}

GtkWidget      *
create_db_question_dialog_ (char *event_name, gint number_of_entrys)
{
  GtkWidget      *db_question_dialog_;
  GtkWidget      *dialog_vbox1;
  GtkWidget      *table2;
  GtkWidget      *label1;
  GtkWidget      *dialog_action_area1;
  GtkWidget      *hbuttonbox;
  GtkWidget      *del_button_;
  GtkWidget      *app_button;
  GtkWidget      *cancel_button;

  gchar          *text_string = NULL;

  text_string = g_strdup_printf (_("\n There are already #%d Times in event %s !! \n"), number_of_entrys, event_name);

  db_question_dialog_ = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "db_question_dialog_", db_question_dialog_);
  GTK_WINDOW (db_question_dialog_)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (db_question_dialog_), _("Warning"));
  gtk_window_position (GTK_WINDOW (db_question_dialog_), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (db_question_dialog_), FALSE, FALSE, FALSE);

  dialog_vbox1 = GTK_DIALOG (db_question_dialog_)->vbox;
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  table2 = gtk_table_new (2, 2, FALSE);
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "table2", table2);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), table2, TRUE, TRUE, 0);

  label1 = gtk_label_new (text_string);
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "label1", label1);
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table2), label1, 0, 1, 0, 1, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);

  dialog_action_area1 = GTK_DIALOG (db_question_dialog_)->action_area;
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), BORDER_WIDTH);


  hbuttonbox = gtk_hbutton_box_new ();
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "hbuttonbox", hbuttonbox);
  gtk_widget_show (hbuttonbox);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbuttonbox, TRUE, TRUE, 0);
  /*gtk_container_border_width (GTK_CONTAINER (hbuttonbox), BORDER_WIDTH); */
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_EDGE);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 10);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox), 2, 0);

  del_button_ = gtk_button_new_with_label (_("Delete Entrys"));
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "del_button_", del_button_);
  gtk_widget_show (del_button_);
  gtk_signal_connect (GTK_OBJECT (del_button_), "clicked", (GtkSignalFunc) adjust_database, (gpointer) 1);
  gtk_signal_connect_object (GTK_OBJECT (del_button_), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (db_question_dialog_));
  gtk_container_add (GTK_CONTAINER (hbuttonbox), del_button_);
  gtk_widget_grab_focus (del_button_);

  app_button = gtk_button_new_with_label (_("Append"));
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "app_button", app_button);
  gtk_widget_show (app_button);
  gtk_container_add (GTK_CONTAINER (hbuttonbox), app_button);
  gtk_signal_connect (GTK_OBJECT (app_button), "clicked", (GtkSignalFunc) adjust_database, (gpointer) 2);
  gtk_signal_connect_object (GTK_OBJECT (app_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (db_question_dialog_));

  cancel_button = gtk_button_new_with_label (_("Cancel"));
  gtk_object_set_data (GTK_OBJECT (db_question_dialog_), "cancel_button", cancel_button);
  gtk_widget_show (cancel_button);
  gtk_signal_connect_object (GTK_OBJECT (cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT (db_question_dialog_));
  gtk_container_add (GTK_CONTAINER (hbuttonbox), cancel_button);
  if (text_string)
    g_free (text_string);
  return db_question_dialog_;
}

void
adjust_database (GtkWidget * widget, gint data)
{
  guint32         seconds = 0;

  switch (data) {
  case 1:			/* drop all times and insert the new ones */
    res = execSQL ("drop sequence time_sequence");
    PQclear (res);
    res = execSQL ("create sequence time_sequence increment 1 start 1 minvalue 1 cycle");
    PQclear (res);
    res = execSQL ("grant ALL on time_sequence to public");
    PQclear (res);
    seconds = main_running_time.starttime[0] * 3600L + main_running_time.starttime[1] * 60L + main_running_time.starttime[2];
    res = execSQL ("delete from times where run_number = %d", main_running_time.run_number);
    PQclear (res);
    /* Should we wrote the new time into the database ?? */
    res = execSQL ("update running_events set starttime_0 ='%d' where number = %d", seconds, main_running_time.run_number);
    PQclear (res);
    /* Yes, I think so */
    create_time_count_window ();
    break;
  case 2:			/* append times to database */
    res = execSQL ("select COUNT(*) from times where run_number = %d", main_running_time.run_number);
    counted_times = atoi (PQgetvalue (res, 0, 0));
    PQclear (res);
    create_time_count_window ();
    break;
  }
}

static void
init_time_clist ()
{
/* Initialisierung wenn schon Zeiten vorhanden sind !! */
  gint            a;
  gint            time[3];
  char            text[TIME_LIST_COLUMNS][24];
  char           *texts[TIME_LIST_COLUMNS];
  GList          *runnerlist = NULL;
  GList          *runner = NULL;
  KRunnerNames   *runnerptr = NULL;
  guint32         seconds = 0;
  texts[0] = text[0];
  texts[1] = text[1];
  gtk_clist_freeze (GTK_CLIST (time_list));
/* Here is the problem with the -1 entrys in the timelist */
  res = execSQL ("select runner.number,name,vorname  " "from runner " "where lauf_nr = %d and runner.number > 0", main_running_time.run_number);
  for (a = 0; a < PQntuples (res); a++) {
    runnerptr = create_runner (atoi (PQgetvalue (res, a, 0)), PQgetvalue (res, a, 1), PQgetvalue (res, a, 2), 0, 0);
    runnerlist = g_list_append (runnerlist, runnerptr);
  }
  PQclear (res);
  res = execSQL ("select number,runner_number,result_seconds " "from times " "where run_number = %d " "order by result_seconds ", main_running_time.run_number);

  for (a = 0; a < PQntuples (res); a++) {
    seconds = atoi (PQgetvalue (res, a, 2));
    time[0] = seconds / 3600L;
    seconds -= time[0] * 3600L;
    time[1] = seconds / 60L;
    time[2] = seconds - time[1] * 60L;
    g_snprintf (text[0], 24, _("%03d Time : %02d:%02d:%02d"), atoi (PQgetvalue (res, a, 0)), time[0], time[1], time[2]);
    runner = runner_find (runnerlist, atoi (PQgetvalue (res, a, 1)));
    if (runner) {
      g_snprintf (text[1], 24, "%s, %s", ((KRunnerNames *) runner->data)->last_name, ((KRunnerNames *) runner->data)->first_name);
    } else {
      g_snprintf (text[1], 24, "----------, ----------");
    }
    gtk_clist_append (GTK_CLIST (time_list), texts);
  }
  PQclear (res);
  gtk_clist_thaw (GTK_CLIST (time_list));
  /*CHANGE */

  if (runnerlist) {
    g_list_foreach (runnerlist, (GFunc) clear_runner, runnerlist->data);
    g_list_free_1 (runnerlist);
  }
  if (runner)
    g_list_free_1 (runner);
}
